/*******************************************************************************
 * Copyright (c) 2012-2013 Biomedical Image Group (BIG), EPFL, Switzerland.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Ricard Delgado-Gonzalo (ricard.delgado@gmail.com)
 *     Nicolas Chenouard (nicolas.chenouard@gmail.com)
 *     Philippe Th&#233;venaz (philippe.thevenaz@epfl.ch)
 *     Emrah Bostan (emrah.bostan@gmail.com)
 *     Ulugbek S. Kamilov (kamilov@gmail.com)
 *     Ramtin Madani (ramtin_madani@yahoo.com)
 *     Masih Nilchian (masih_n85@yahoo.com)
 *     C&#233;dric Vonesch (cedric.vonesch@epfl.ch)
 *     Virginie Uhlmann (virginie.uhlmann@epfl.ch)
 *     Cl&#233;ment Marti (clement.marti@epfl.ch)
 *     Julien Jacquemot (julien.jacquemot@epfl.ch)
 *     Daniel Schmitter (daniel.schmitter@epfl.ch)
 ******************************************************************************/
package plugins.big.bigsnake;

import icy.gui.component.BorderedPanel;
import icy.gui.dialog.MessageDialog;
import icy.gui.frame.IcyFrameAdapter;
import icy.gui.frame.IcyFrameEvent;
import icy.gui.frame.progress.AnnounceFrame;
import icy.gui.frame.progress.ProgressFrame;
import icy.gui.main.ActiveSequenceListener;
import icy.gui.main.ActiveViewerListener;
import icy.gui.main.GlobalSequenceListener;
import icy.gui.main.GlobalViewerListener;
import icy.gui.viewer.Viewer;
import icy.gui.viewer.ViewerEvent;
import icy.main.Icy;
import icy.plugin.abstract_.PluginActionable;
import icy.resource.ResourceUtil;
import icy.roi.ROI;
import icy.sequence.Sequence;
import icy.sequence.SequenceEvent;
import java.awt.CardLayout;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import javax.swing.AbstractAction;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import org.apache.poi.ss.usermodel.Workbook;

import plugins.adufour.blocks.lang.Block;
import plugins.adufour.blocks.lang.BlockDescriptor;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.ezplug.EzGUI;
import plugins.adufour.roi.ROIMeasures;
import plugins.adufour.vars.lang.Var;
import plugins.adufour.vars.lang.VarBoolean;
import plugins.adufour.vars.lang.VarDouble;
import plugins.adufour.vars.lang.VarEnum;
import plugins.adufour.vars.lang.VarInteger;
import plugins.adufour.vars.lang.VarSequence;
import plugins.adufour.vars.lang.VarWorkbook;
import plugins.big.bigsnake.core.ImageLUTContainerSD;
import plugins.big.bigsnake.core.Snake2DClipboardSD;
import plugins.big.bigsnake.gui.GlobalSnakeSettingsPaneSD;
import plugins.big.bigsnake.gui.ImageSettingsPaneSD;
import plugins.big.bigsnake.gui.InitializationSettingsPaneSD;
import plugins.big.bigsnake.gui.NucleiDetectionSettingsPaneSD;
import plugins.big.bigsnake.gui.PlugInFrameSD;
import plugins.big.bigsnake.gui.ToolTipsMessagesSD;
import plugins.big.bigsnake.jaccard.JaccardIndexMode;
import plugins.big.bigsnake.jaccard.JaccardSubsnakeStatistics;
import plugins.big.bigsnake.io.InputFormatSD;
import plugins.big.bigsnake.keeper.KeepersListSD;
import plugins.big.bigsnake.keeper.SnakeKeeperSD;
import plugins.big.bigsnake.nuclei.NucleiDetector;
import plugins.big.bigsnake.roi.SnakeEditModeSD;
import plugins.big.bigsnake.snake.ESnakeSD;
import plugins.big.bigsnake.snake.ESnakeParametersSD;
import plugins.big.bigsnake.snake.GlobalSnakeInitializer;
import plugins.big.bigsnake.snake.SubsnakeSD;
import plugins.big.bigsnake.snake.Enums.ESnakeEnergyTypeSD;
import plugins.big.bigsnake.snake.Enums.ESnakePriorShapeTypeSD;
import plugins.big.bigsnake.snake.Enums.ESnakeTargetTypeSD;
import plugins.big.bigsnake.snake.Enums.GlobalSnakeInitialization;
import plugins.big.bigsnake.snake.Enums.GlobalSnakeSortingType;
import plugins.big.bigsnake.snake.Enums.MultiresolutionTypeSD;
import plugins.big.bigsnake.snake.Enums.ShapeSpaceTypeSD;
import plugins.big.bigsnake.snake.Enums.SnakeTypeSD;
import plugins.big.bigsnakeutils.icy.snake2D.Snake2DNode;
import plugins.big.bigsnakeutils.process.process1D.BSplineBasis.BSplineBasisType;
import plugins.big.bigsnake.voronoi.VoronoiDiagram;
import plugins.big.bigsnake.snake.GlobalSnakeSD;
import plugins.big.bigsnake.snake.SubsnakeInitialShape;
import plugins.big.bigsnake.snake.GlobalSnakeParametersSD;

/**
 * Main class of the Active Cells plug-in.
 * 
 * @version January 13, 2018
 * 
 * @author Nicolas Chenouard (nicolas.chenouard@gmail.com)
 * @author Ricard Delgado-Gonzalo (ricard.delgado@gmail.com)
 * @author Emrah Bostan (emrah.bostan@gmail.com)
 * @author Daniel Schmitter (daniel.schmitter@epfl.ch)
 * @author Anais Badoual (anais.badoual@epfl.ch)
 * @author Aymeric Galan (aymeric.galan@gmail.com)
 */
public class SubdivSnakeGS extends PluginActionable implements Block,
		ActionListener, ActiveSequenceListener, ActiveViewerListener,
		GlobalSequenceListener, GlobalViewerListener, KeyListener {

	/** Reference to the main plug-in class. */
	private SubdivSnakeGS mainPlugin_ = null;
	
	// ----------------------------------------------------------------------------
	// SNAKES

	/**
	 * Map that associates each open <code>Sequence</code> with a list of
	 * snakes.
	 */
	private final HashMap<Sequence, KeepersListSD> keepersListTable_ = new HashMap<Sequence, KeepersListSD>();
	/** Lock used to preserve synchronization among different open images. */
	private final Lock keepersListLock_ = new ReentrantLock();
	/**
	 * Index that counts the number of snakes added since the plug-in was
	 * created.
	 */
	private int nSnakes_ = 0;
	
	/**
	 * Nuclei detector engine
	 */
	private NucleiDetector nucleiDetector_ = new NucleiDetector(true);
	
	/**
	 * GlobalSnake initializer
	 */
	private GlobalSnakeInitializer initializer_ = null;
	

	// ----------------------------------------------------------------------------
	// IMAGES AND VIEWERS

	/** Map that associates each open image with its corresponding LUTs. */
	private final HashMap<Sequence, ImageLUTContainerSD> imageLUTs_ = new HashMap<Sequence, ImageLUTContainerSD>();
	/**
	 * Map that keeps track of which <code>Viewer</code> have a
	 * <code>KeyListeners</code> attached.
	 */
	private final HashMap<Viewer, Boolean> hasKeyListenerTable_ = new HashMap<Viewer, Boolean>();

	// ----------------------------------------------------------------------------
	// GUI

	/** Main frame of the plug-in. */
	private final PlugInFrameSD plugInMainFrame_ = new PlugInFrameSD(this);
	
	/** Pane that contains the image related settings. */
	private ImageSettingsPaneSD imageSettingsPane_ = null;
	/** Panel that contains the settings of the focused image. */
	private final JPanel imageSettingsPanel_ = new BorderedPanel();
	
	/** Pane that contains GlobalSnake related settings. */
	private InitializationSettingsPaneSD initializationSettingsPane_ = null;
	/** Panel that contains the settings of the focused image. */
	private final JPanel initializationSettingsPanel_ = new BorderedPanel();
	
	/** Pane that contains GlobalSnake related settings. */
	private NucleiDetectionSettingsPaneSD nucleiDetectionSettingsPane_ = null;
	/** Panel that contains the settings of the focused image. */
	private final JPanel nucleiDetectionSettingsPanel_ = new BorderedPanel();
	
	/** Pane that contains the snake related settings. */
//	private SnakeSettingsPaneSD eSnakeSettingsPane_ = null;
	/** Panel that contains the settings of the active snake. */
//	private final JPanel snakeParametersPanel_ = new BorderedPanel();
	/** Pane that contains GlobalSnake related settings. */
	private GlobalSnakeSettingsPaneSD globalSnakeSettingsPane_ = null;
	/** Panel that contains the settings of the active snake. */
	private final JPanel globalSnakeSettingsPanel_ = new BorderedPanel();
	
	/** Label in which the tooltips are shown. */
	private final JLabel toolTipMessagesLabel_ = new JLabel(
			ToolTipsMessagesSD.getToolTipMessage(0), SwingConstants.CENTER);

	// ----------------------------------------------------------------------------
	// ACTION BUTTONS

	/**
	 * Button that sets the interaction mode with the snake to translation mode
	 * (control points or whole snake).
	 */
	private final JButton moveSnakeButton_ = new JButton("");
	/**
	 * Button that sets the interaction mode with the snake to resizing/scaling
	 * mode.
	 */
	private final JButton resizeSnakeButton_ = new JButton("");
	/** Button that sets the interaction mode with the snake to rotation mode. */
	private final JButton rotateSnakeButton_ = new JButton("");
	/**
	 * Button that fires the process to create global snake from the interface
	 * parameters.
	 */
	private final JButton createGlobalSnakeButton_ = new JButton("");
	/**
	 * Button that fires the process to create a new set of subsnakes from the interface
	 * parameters.
	 */
	private final JButton decomposeGlobalSnakeButton_ = new JButton("");
	/**
	 * Button that fires the process of nuclei detection.
	 */
	private final JButton runNucleiDetectionButton_ = new JButton("");
	/**
	 * Button that fires the process to optimize the active snake of the focused
	 * image.
	 */
	private final JButton optimizeSnakeButton_ = new JButton("");
	/**
	 * Button that fires the process to optimize all of the snakes of the
	 * focused image.
	 */
	private final JButton optimizeAllSnakesButton_ = new JButton("");
	/**
	 * Button that fires the process of deletion of the active snake of the
	 * focused image.
	 */
	private final JButton deleteSnakeButton_ = new JButton("");

	/** Size of the icons in the menu. */
	private static int ICONSIZE = 20;

	// ----------------------------------------------------------------------------
	// EXTERNAL PLUGINS

	/**
	 * External plug-in that performs measurements on the ROI defined by the
	 * snakes.
	 */
	private ROIMeasures roiMeasures_ = null;

	// ----------------------------------------------------------------------------
	// OTHER

	/** Determines the way the user can interact with the snake. */
	private SnakeEditModeSD editingMode_ = null;

	/**
	 * Stores the information of the snake when using the COPY and PASTE
	 * commands.
	 */
	private final Snake2DClipboardSD clipboard_ = new Snake2DClipboardSD();

	/** List of running threads. */
	private final ArrayList<Thread> threadList_ = new ArrayList<Thread>();

	// ============================================================================
	// BLOCK FIELDS

	// ----------------------------------------------------------------------------
	// INPUT

	/** Input image for the snake plug-in within the protocol environment. */
	private final VarSequence inputSequenceBlock_ = new VarSequence("Input",
			null);
	// TODO Add javadoc
	private final VarDouble sigmaBlock_ = new VarDouble("Smoothing", 10);
	// TODO Add javadoc
	private final VarEnum<ESnakeTargetTypeSD> targetBrightnessBlock_ = new VarEnum<ESnakeTargetTypeSD>(
			"Target brightness", ESnakeTargetTypeSD.DARK);
	// TODO Add javadoc
	private final VarInteger MBlock_ = new VarInteger("Control points", 3);
	// TODO Add javadoc
	private final VarInteger MLevelBlock_ = new VarInteger("Number of subdivision", 2);
	// TODO Add javadoc
	private final VarInteger MmScalingBlock_ = new VarInteger("m scaling factor", 2);
	// TODO Add javadoc
	private final VarEnum<ESnakeEnergyTypeSD> energyTypeBlock_ = new VarEnum<ESnakeEnergyTypeSD>(
			"Energy type", ESnakeEnergyTypeSD.REGION);
	// TODO Add javadoc
	private final VarDouble alphaBlock_ = new VarDouble("Alpha", 0);
	// TODO Add javadoc
	private final VarEnum<ESnakePriorShapeTypeSD> priorShapeTypeBlock_ = new VarEnum<ESnakePriorShapeTypeSD>(
			"Prior shape", ESnakePriorShapeTypeSD.NONE);
	// TODO Add javadoc
	private final VarEnum<ShapeSpaceTypeSD> shapeSpaceTypeBlock_ = new VarEnum<ShapeSpaceTypeSD>(
				"Shape space", ShapeSpaceTypeSD.SIMILARITY);
	// TODO Add javadoc
		private final VarEnum<MultiresolutionTypeSD> multiresolutionTypeBlock_ = new VarEnum<MultiresolutionTypeSD>(
					"Multiresolution", MultiresolutionTypeSD.MANUAL);
	// TODO Add javadoc
	private final VarEnum<BSplineBasisType> basisFunctionTypeBlock_ = new VarEnum<BSplineBasisType>(
				"Basis function", BSplineBasisType.ESPLINE3);
	// TODO Add javadoc
	private final VarDouble betaBlock_ = new VarDouble("Beta", 0);
	// TODO Add javadoc
	private final VarInteger maxIterationsBlock_ = new VarInteger(
			"Max iterations", 100);
	// TODO Add javadoc
	private final VarBoolean isImmortalBlock_ = new VarBoolean("Immortal",
			false);

	// TODO Add javadoc
	private final VarEnum<InputFormatSD> inputFormatROI_ = new VarEnum<InputFormatSD>(
			"Input snake format", InputFormatSD.ROI_Array);
	// TODO Add javadoc
	private final VarWorkbook inputSnakeWorkbook_ = new VarWorkbook(
			"Snake input workbook", (Workbook) null);
	// TODO Add javadoc
	private final Var<ROI[]> roiArray_ = new Var<ROI[]>("Array of ROIs",
			ROI[].class);

	// ----------------------------------------------------------------------------
	// OUTPUT

	// TODO Add javadoc
	private final VarWorkbook outputSnakeWorkbook_ = new VarWorkbook(
			"Snake output workbook", "outputSnakeWorkbook_");

	// ============================================================================
	// PUBLIC METHODS

	/** Method executed when launching the plug-in. */
	@Override
	public void run() {
		if (Thread.currentThread().getStackTrace()[2].getClassName().equals(
				BlockDescriptor.class.getName())) {
			return; // runBlock();
		} else {
			runStandalone();
		}
	}

	// ----------------------------------------------------------------------------

	/** Method executed when launching the plug-in in standalone mode. */
	private void runStandalone() {
		mainPlugin_ = this;
		
		nucleiDetectionSettingsPane_ = 
				new NucleiDetectionSettingsPaneSD("Nuclei detection", nucleiDetector_);
		imageSettingsPane_ = 
				new ImageSettingsPaneSD("Image", imageLUTs_);
		initializationSettingsPane_ = 
				new InitializationSettingsPaneSD("Initialization", mainPlugin_);
		globalSnakeSettingsPane_ = 
				new GlobalSnakeSettingsPaneSD("GlobalSnake", mainPlugin_);

		Viewer existedViewer = getActiveViewer();
		if (existedViewer != null) {
			addKeyListenerToViewer(existedViewer);
		}

		// create the main Panel
		Container mainPane = plugInMainFrame_.getContentPane();
		mainPane.setLayout(new BoxLayout(mainPane, BoxLayout.PAGE_AXIS));
		
		nucleiDetectionSettingsPanel_.setLayout(new CardLayout());
		nucleiDetectionSettingsPanel_.add("pane0", nucleiDetectionSettingsPane_);
		mainPane.add(nucleiDetectionSettingsPanel_);
		
		initializationSettingsPanel_.setLayout(new CardLayout());
		initializationSettingsPanel_.add("pane1", initializationSettingsPane_);
		mainPane.add(initializationSettingsPanel_);
		
		imageSettingsPanel_.setLayout(new CardLayout());
		imageSettingsPanel_.add("pane2", imageSettingsPane_);
		mainPane.add(imageSettingsPanel_);

		// create the snake parameters Panel
		globalSnakeSettingsPanel_.setLayout(new CardLayout());
		globalSnakeSettingsPanel_.add("pane3", globalSnakeSettingsPane_);
		mainPane.add(globalSnakeSettingsPanel_);

		JPanel interactionPanelHeader = new JPanel();
		interactionPanelHeader.setLayout(new BoxLayout(interactionPanelHeader,
				BoxLayout.LINE_AXIS));
		JLabel snakeEditLabel = new JLabel("Snake editing");
		interactionPanelHeader.add(snakeEditLabel);
		mainPane.add(interactionPanelHeader);

		// create the bar of interaction buttons
		JPanel interactionPanel = new JPanel();
		
		Icon nucleiDetectionIcon = ResourceUtil.getAlphaIcon("target.png", ICONSIZE);
		runNucleiDetectionButton_.setIcon(nucleiDetectionIcon);
		runNucleiDetectionButton_.setToolTipText("Launch nuclei detection");
		
		Icon newGlobalSnakeIcon = ResourceUtil.getAlphaIcon("round_plus.png", ICONSIZE);
		createGlobalSnakeButton_.setIcon(newGlobalSnakeIcon);
		createGlobalSnakeButton_.setToolTipText("Add GlobalSnake");
		createGlobalSnakeButton_.setEnabled(false);
		
		Icon decomposeGlobalSnakeIcon = ResourceUtil.getAlphaIcon("clipboard_cut.png", ICONSIZE-2);
		decomposeGlobalSnakeButton_.setIcon(decomposeGlobalSnakeIcon);
		decomposeGlobalSnakeButton_.setToolTipText("Decompose into Subsnakes");
		decomposeGlobalSnakeButton_.setEnabled(false);

		Icon imgiconMoveSnake = ResourceUtil.getAlphaIcon("cursor_arrow.png",
				ICONSIZE);
		moveSnakeButton_.setIcon(imgiconMoveSnake);
		moveSnakeButton_.setToolTipText("Move snake");
		moveSnakeButton_.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				setEditingMode(SnakeEditModeSD.MOVE_SNAKE);
			}
		});

		Icon imgiconDilateSnake = ResourceUtil.getAlphaIcon("expand.png",
				ICONSIZE);
		resizeSnakeButton_.setIcon(imgiconDilateSnake);
		resizeSnakeButton_.setToolTipText("Dilate snake");
		resizeSnakeButton_.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				setEditingMode(SnakeEditModeSD.DILATE_SNAKE);
			}
		});

		Icon imgiconRotateSnake = ResourceUtil.getAlphaIcon("rot_unclock.png",
				ICONSIZE);
		rotateSnakeButton_.setIcon(imgiconRotateSnake);
		rotateSnakeButton_.setToolTipText("Rotate snake");
		rotateSnakeButton_.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				setEditingMode(SnakeEditModeSD.ROTATE_SNAKE);
			}
		});

		// default editing mode
		setEditingMode(SnakeEditModeSD.MOVE_SNAKE);

		interactionPanel.setLayout(new GridLayout(1, 4));
		interactionPanel.add(runNucleiDetectionButton_);
		interactionPanel.add(createGlobalSnakeButton_);
		interactionPanel.add(decomposeGlobalSnakeButton_);
		interactionPanel.add(moveSnakeButton_);
		interactionPanel.add(resizeSnakeButton_);
		interactionPanel.add(rotateSnakeButton_);

		mainPane.add(interactionPanel);

		// create the action panel
		JPanel actionPanelHeader = new JPanel();
		actionPanelHeader.setLayout(new BoxLayout(actionPanelHeader,
				BoxLayout.LINE_AXIS));
		JLabel actionLabel = new JLabel("Snake actions");
		actionPanelHeader.add(actionLabel);
		mainPane.add(actionPanelHeader);

		JPanel actionPane = new JPanel();
		actionPane.setLayout(new GridLayout(1, 3));

		Icon optimizeActiveSnakeIcon = ResourceUtil.getAlphaIcon(
				"playback_play.png", ICONSIZE);
		optimizeSnakeButton_.setIcon(optimizeActiveSnakeIcon);
		optimizeSnakeButton_.setToolTipText("Optimize active snake");

		Icon optimizeAllSnakesIcon = ResourceUtil.getAlphaIcon(
				"playback_ff.png", ICONSIZE);
		optimizeAllSnakesButton_.setIcon(optimizeAllSnakesIcon);
		optimizeAllSnakesButton_.setToolTipText("Optimize all snakes");

		Icon deleteSnakeIcon = ResourceUtil.getAlphaIcon("trash.png", ICONSIZE);
		deleteSnakeButton_.setIcon(deleteSnakeIcon);
		deleteSnakeButton_.setToolTipText("Remove active snake");

		actionPane.add(optimizeSnakeButton_);
		actionPane.add(optimizeAllSnakesButton_);
		actionPane.add(deleteSnakeButton_);

		decomposeGlobalSnakeButton_.addActionListener(this);
		runNucleiDetectionButton_.addActionListener(this);
		createGlobalSnakeButton_.addActionListener(this);
		optimizeSnakeButton_.addActionListener(this);
		optimizeAllSnakesButton_.addActionListener(this);
		deleteSnakeButton_.addActionListener(this);

		// assign DELETE as key binding for deleteSnakeButton
		deleteSnakeButton_.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
				.put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0),
						"doDeleteAction");
		deleteSnakeButton_.getActionMap().put("doDeleteAction",
				new AbstractAction() {
					private static final long serialVersionUID = -4139948974574644979L;

					@Override
					public void actionPerformed(ActionEvent e) {
						deleteSnakeButton_.doClick();
					}
				});

		// assign BACKSPACE as key binding for deleteSnakeButton (for MAC users)
		deleteSnakeButton_.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE, 0),
				"doDeleteAction");
		deleteSnakeButton_.getActionMap().put("doDeleteAction",
				new AbstractAction() {
					private static final long serialVersionUID = 1720306735845428242L;

					@Override
					public void actionPerformed(ActionEvent e) {
						deleteSnakeButton_.doClick();
					}
				});
		mainPane.add(actionPane);

		// create a pane for the selected sequence display
		JPanel sequencePane = new JPanel();
		sequencePane.add(toolTipMessagesLabel_);

		toolTipMessagesLabel_.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				toolTipMessagesLabel_.setText(ToolTipsMessagesSD
						.getToolTipMessage());
			}
		});

		mainPane.add(sequencePane);

		Icy.getMainInterface().addActiveSequenceListener(this);
		Icy.getMainInterface().addActiveViewerListener(this);
		Icy.getMainInterface().addGlobalSequenceListener(this);
		Icy.getMainInterface().addGlobalViewerListener(this);

		plugInMainFrame_.addFrameListener(new IcyFrameAdapter() {
			@Override
			public void icyFrameClosed(IcyFrameEvent e) {
				Icy.getMainInterface()
						.removeActiveSequenceListener(mainPlugin_);
				Icy.getMainInterface().removeActiveViewerListener(mainPlugin_);
				Icy.getMainInterface()
						.removeGlobalSequenceListener(mainPlugin_);
				Icy.getMainInterface().removeGlobalViewerListener(mainPlugin_);
			}
		});

		plugInMainFrame_.pack();

		plugInMainFrame_.addFrameListener(new IcyFrameAdapter() {
			@Override
			public void icyFrameInternalized(IcyFrameEvent e) {
				plugInMainFrame_.pack();
			}

			@Override
			public void icyFrameExternalized(IcyFrameEvent e) {
				plugInMainFrame_.pack();
			}
		});

		addIcyFrame(plugInMainFrame_);
		plugInMainFrame_.center();
		plugInMainFrame_.setVisible(true);
	}

	// ----------------------------------------------------------------------------

	/**
	 * Prepares the plug-in to be terminated. All memory is freed, and auxiliary
	 * plug-ins are closed.
	 */
	public void terminatePlugin() {
		closeROIMeasures();
		convertAllSnakesToROIs();
		removeAllKeyListeners();
	}

	// ----------------------------------------------------------------------------
	// SNAKE CREATION/DESTRUCTION MANAGEMENT

	/**
	 * Returns <code>true</code> if <code>SnakeKeeper</code> passed is the
	 * active one.
	 */
	public boolean isActiveSnake(SnakeKeeperSD keeper) {
		Sequence focusedSequence = getActiveSequence();
		if (focusedSequence == null) {
			return false;
		}
		keepersListLock_.lock();
		try {
			if (keepersListTable_.containsKey(focusedSequence)) {
				KeepersListSD keepersList = keepersListTable_
						.get(focusedSequence);
				if (keepersList != null) {
					return keepersList.isActiveSnakeKeeper(keeper);
				}
			}
		} finally {
			keepersListLock_.unlock();
		}
		return false;
	}

	// ----------------------------------------------------------------------------

	/** Makes a particular snake active and responsive to interactions. */
	public void activateSnake(SnakeKeeperSD keeper) {
		Sequence focusedSequence = getActiveSequence();
		if (focusedSequence == null) {
			return;
		}
		keepersListLock_.lock();
		try {
			if (keepersListTable_.containsKey(focusedSequence)) {
				KeepersListSD keepersList = keepersListTable_
						.get(focusedSequence);
				if (keepersList != null) {
					boolean success = keepersList.activateSnakeKeeper(keeper);
					if (keeper.snake_ instanceof GlobalSnakeSD) {
						globalSnakeSettingsPane_.setSnakeParameters(
								(GlobalSnakeParametersSD) keeper.getESnakeParameters());
					} else {
						// do not update the UI panel in this case
					}
					if (!success) {
						System.err
								.println("activateSnake: The SnakeKeeper could not be activated.");
					}
				}
			}
		} finally {
			keepersListLock_.unlock();
		}
	}

	// ----------------------------------------------------------------------------

	/**
	 * Adds a new snake to the active <code>Sequence</code> and makes it active
	 * and responsive to interactions.
	 */
	private void runNucleiDetection() {
		Thread thr = new Thread() {
			@Override
			public void run() {
				try {
					Sequence focusedSequence = getActiveSequence();
					if (focusedSequence == null) {
						new AnnounceFrame(
								"No image detected. Please, open an image first.");
						return;
					}

					nucleiDetector_.setManualMode(
							nucleiDetectionSettingsPane_.isManualMode());
					
					if (nucleiDetector_.isManualMode()) {
						nucleiDetector_.setSequence(getActiveSequence());
					} else {
						nucleiDetector_.setSequence(
								nucleiDetectionSettingsPane_.getSequenceWithNuclei());
						nucleiDetector_.setSigmaLoG(
								nucleiDetectionSettingsPane_.getSigmaLoG());
						nucleiDetector_.setThreshold(
								nucleiDetectionSettingsPane_.getThreshold());
					}
					/* Launch detection */
					nucleiDetector_.run();
					
					createGlobalSnakeButton_.setEnabled(nucleiDetector_.isDetectionOk());
					decomposeGlobalSnakeButton_.setEnabled(nucleiDetector_.isDetectionOk());
					
				} finally {
					threadList_.remove(this);
				}
			}
		};
		threadList_.add(thr);
		thr.start();
	}
	
	
	/**
	 * Retrieve necessary informations from nuclei detection and initialization panel
	 * to compute initial control nodes positions for each Subsnake of a GlobalSnake
	 */
	private List<SubsnakeInitialShape> getInitCoefsFromInterface() {
		
		GlobalSnakeInitialization initType;
		if (nucleiDetector_.getNucleiPositions().size() <= 1) {
			System.err.println("More than 1 nuclei should be detected or provided, "
					+ "please launch detector again");
			return null;
		}
		else if (nucleiDetector_.getNucleiPositions().size() == 2) {
			// for now, this option is not supported by the GUI (hard coded here)
			initType = GlobalSnakeInitialization.TWO_CELLS;
		}
		else {
			initType = initializationSettingsPane_.getInitializationType();
		}
		
		GlobalSnakeSortingType sortingType = 
				initializationSettingsPane_.getSortingType();
		
		/* Initialize the initializer (lol) */
		initializer_ = new GlobalSnakeInitializer(initType, sortingType);
		
		/* Initialization from Voronoi diagram + dilated convex hull */
		if (initType == GlobalSnakeInitialization.BOUNDED_VORONOI) {
			VoronoiDiagram voronoi = new VoronoiDiagram(
					nucleiDetector_.getWidth(), 
					nucleiDetector_.getHeight(), 
					nucleiDetector_.getNucleiPositionsAsArray(),
					initializationSettingsPane_.getConvexHullDilationCoef());
			
			// Feed the Voronoi diagram to the initializer 
			initializer_.setVoronoiDiagram(voronoi);
		}
		
		/* Special case : initialization from 2 cells (not possible with Voronoi) */
		else if (initType == GlobalSnakeInitialization.TWO_CELLS) {
			
			// Feed directly the positions of nuclei to the initializer
			initializer_.setSpecificSeeds(nucleiDetector_.getNucleiPositions());
		}
		
		return initializer_.getInitialShapes();
	}
	
	
	/**
	 * Adds GlobalSnake
	 */
	private void addAndActivateGlobalSnake() {
		Thread thr = new Thread() {
			@Override
			public void run() {
				try {
					Sequence focusedSequence = getActiveSequence();
					if (focusedSequence == null) {
						new AnnounceFrame(
								"No image detected. Please, open an image first.");
						return;
					}

					ImageLUTContainerSD imageLUTContainer = imageLUTs_
							.get(focusedSequence);

					if (imageLUTContainer == null) {
						System.err.println("LUT not found");
						return;
					}

					if (!imageLUTContainer.isLUTUpToDate()) {
						try {
							imageLUTContainer.buildLUTs();
						} catch (Exception e) {
							new AnnounceFrame(e.getMessage());
							e.printStackTrace();
							return;
						}
					}
					
					if (!nucleiDetector_.isDetectionOk()) {
						System.err.println("First, start nuclei detection !");
						return;
					}
					
					else {
						ProgressFrame progressFrame1 = new ProgressFrame(
								"Initializing GlobalSnake");
						
						/* Set up the initializer and get initial Subsnake shapes */
						List<SubsnakeInitialShape> allInitSubsnakeCoefs = getInitCoefsFromInterface();
								
						progressFrame1.close();
						
						if (allInitSubsnakeCoefs == null) {
							System.out.println("Error occured during initialization");
							return;
						}
						
						ProgressFrame progressFrame2 = new ProgressFrame(
								"Creating a new GlobakSnake");
					
						GlobalSnakeParametersSD parameters = 
								new GlobalSnakeParametersSD(
										allInitSubsnakeCoefs,
										globalSnakeSettingsPane_.getSubdivLevel(),
										globalSnakeSettingsPane_.getTargetBrightness(),
										globalSnakeSettingsPane_.getBasisFunctionType(),
										globalSnakeSettingsPane_.getEnergyType(),
										globalSnakeSettingsPane_.getGlobalEnergyType(),
										globalSnakeSettingsPane_.getMaxIterations(),
										globalSnakeSettingsPane_.isImmortal(),
										globalSnakeSettingsPane_.showCommonEdges(),
										globalSnakeSettingsPane_.getIntermediateSubdiv());
						
						/* Global snake initialization */
						GlobalSnakeSD globalSnake = 
								new GlobalSnakeSD(imageLUTContainer, parameters);
	
//						System.out.println(globalSnake);
					
						/* Display */
						SnakeKeeperSD keeper = new SnakeKeeperSD(
								imageLUTContainer.getOriginalSequence(), globalSnake,
								mainPlugin_);
						
						nSnakes_++;
						keeper.setID(String.valueOf(nSnakes_));
						keeper.setSnakeEditMode(getEditingMode());
						
						keepersListLock_.lock();
						try {
							if (keepersListTable_.containsKey(focusedSequence)) {
								KeepersListSD list = keepersListTable_
										.get(focusedSequence);
								list.addAndActivateKeeper(keeper);
							} else {
								KeepersListSD list = new KeepersListSD();
								list.addAndActivateKeeper(keeper);
								keepersListTable_.put(focusedSequence, list);
							}
						} finally {
							keepersListLock_.unlock();
						}
						
						progressFrame2.close();
						
						if (nucleiDetector_.isManualMode()) {
							nucleiDetector_.removeNucleiROI();
						}
						
						createGlobalSnakeButton_.setEnabled(nucleiDetector_.isDetectionOk());
						
					}
					
				} finally {
					threadList_.remove(this);
				}
			}
		};
		threadList_.add(thr);
		thr.start();
	}
	
	
	/**
	 * Decompose the GlobalSnake in multiple Subsnakes,
	 * then delete the GlobalSnake.
	 */
	private void decomposeGlobalSnake() {
		Thread thr = new Thread() {
			@Override
			public void run() {
				try {
					Sequence focusedSequence = getActiveSequence();
					if (focusedSequence == null) {
						new AnnounceFrame(
								"No image detected. Please, open an image first.");
						return;
					}

					ImageLUTContainerSD imageLUTContainer = imageLUTs_
							.get(focusedSequence);

					if (imageLUTContainer == null) {
						System.err.println("LUT not found");
						return;
					}

					if (!imageLUTContainer.isLUTUpToDate()) {
						try {
							imageLUTContainer.buildLUTs();
						} catch (Exception e) {
							new AnnounceFrame(e.getMessage());
							e.printStackTrace();
							return;
						}
					}

					SnakeKeeperSD activeKeeper = null;
					keepersListLock_.lock();
					try {
						if (keepersListTable_.get(focusedSequence) != null) {
							KeepersListSD keepersList = keepersListTable_
									.get(focusedSequence);
							if (keepersList != null && 
									keepersList.getActiveSnakeKeeper() != null) {
								activeKeeper = keepersList
										.getActiveSnakeKeeper();
							}
						} else {
							new AnnounceFrame(
									"No snakes detected. Add a snake first.");
						}
					} finally {
						keepersListLock_.unlock();
					}
					
					if ((activeKeeper != null) && 
							(activeKeeper.snake_ instanceof GlobalSnakeSD)) {
						
						ProgressFrame progressFrame = new ProgressFrame(
								"Creating Subsnakes");
						
						GlobalSnakeSD activeGlobalSnake = 
								(GlobalSnakeSD) activeKeeper.snake_;
						
						removeActiveSnake(); // delete the GlobalSnake
						
						for (int i = 0 ; i < activeGlobalSnake.getNumSubsnakes() ; i++) {

							SubsnakeSD subsnake = activeGlobalSnake.getSubsnake(i);
							
							SnakeKeeperSD keeper = new SnakeKeeperSD(
									imageLUTContainer.getOriginalSequence(), subsnake,
									mainPlugin_);
							
							nSnakes_++;
							keeper.setID(String.valueOf(nSnakes_));
							keeper.setSnakeEditMode(getEditingMode());
							
							keepersListLock_.lock();
							try {
								if (keepersListTable_.containsKey(focusedSequence)) {
									KeepersListSD list = keepersListTable_
											.get(focusedSequence);
									list.addAndActivateKeeper(keeper);
								} else {
									KeepersListSD list = new KeepersListSD();
									list.addAndActivateKeeper(keeper);
									keepersListTable_.put(focusedSequence, list);
								}
							} finally {
								keepersListLock_.unlock();
							}
						}
						
						progressFrame.close();
						
						decomposeGlobalSnakeButton_.setEnabled(nucleiDetector_.isDetectionOk());
						plugInMainFrame_.enableJaccardMenuItems();
					}

				} finally {
					threadList_.remove(this);
				}
			}
		};
		threadList_.add(thr);
		thr.start();
	}
	
	
	/** Transforms all ROIs present on all images to standard ROIs for Icy. */
	private void convertAllSnakesToROIs() {
		keepersListLock_.lock();
		try {
			for (Entry<Sequence, KeepersListSD> entry : keepersListTable_
					.entrySet()) {
				KeepersListSD keepersList = entry.getValue();
				if (keepersList != null) {
					if (!keepersList.isEmpty()) {
						keepersList.convertAllSnakesToROIs();
					}
				}
			}
			keepersListTable_.clear();
		} finally {
			keepersListLock_.unlock();
		}
	}

	// ----------------------------------------------------------------------------

	/** Makes a particular snake inactive and not responsive to interaction. */
	public void deactivateSnake(SnakeKeeperSD keeper) {
		Sequence focusedSequence = getActiveSequence();
		if (focusedSequence == null) {
			return;
		}
		keepersListLock_.lock();
		try {
			if (keepersListTable_.containsKey(focusedSequence)) {
				KeepersListSD keepersList = keepersListTable_
						.get(focusedSequence);
				if (keepersList != null) {
					boolean success = keepersList.deactivateSnakeKeeper(keeper);
					if (!success) {
						System.err
								.println("activateSnake: The ESnakeKeeper could not be activated.");
					}
				}
			}
		} finally {
			keepersListLock_.unlock();
		}
	}

	// ----------------------------------------------------------------------------

	/** Removes the active snake from the active <code>Sequence</code>. */
	private void removeActiveSnake() {
		Sequence focusedSequence = getActiveSequence();
		if (focusedSequence == null) {
			new AnnounceFrame("No image detected. Please, open an image first.");
			return;
		}
		keepersListLock_.lock();
		try {
			if (keepersListTable_.containsKey(focusedSequence)) {
				KeepersListSD keepersList = keepersListTable_
						.get(focusedSequence);
				if (keepersList != null) {
					keepersList.removeActiveSnakeKeeper();
					
					if (keepersList.getNumKeepers() < 1)
						plugInMainFrame_.disableJaccardMenuItems();
				}
			}
		} finally {
			keepersListLock_.unlock();
		}
	}

	// ----------------------------------------------------------------------------

	/**
	 * Retrieves the snake parameters from the GUI and sets them to the active
	 * snake.
	 */
	public void loadSnakeParametersFromInterface() {
		Sequence focusedSequence = getActiveSequence();
		if (focusedSequence == null) {
			return;
		}
		keepersListLock_.lock();
		try {
			if (keepersListTable_.containsKey(focusedSequence)) {
				
				KeepersListSD keepersList = keepersListTable_
						.get(focusedSequence);
				
				if (keepersList != null) {
					
					SnakeKeeperSD activeSnakeKeeper = keepersList
							.getActiveSnakeKeeper();
					
					if (activeSnakeKeeper != null) {
						
						if (nucleiDetector_.isDetectionOk()) {
							
							ProgressFrame progressFrame1 = new ProgressFrame(
									"Updating GlobalSnake");
							
							List<SubsnakeInitialShape> allInitSubsnakeCoefs = getInitCoefsFromInterface();
				
							if (allInitSubsnakeCoefs == null) {
								System.out.println("Error occured during control points initialization");
								return;
							}
							
							ESnakeParametersSD snakeParameters = new GlobalSnakeParametersSD(
									allInitSubsnakeCoefs,
									globalSnakeSettingsPane_.getSubdivLevel(),
									globalSnakeSettingsPane_.getTargetBrightness(),
									globalSnakeSettingsPane_.getBasisFunctionType(),
									globalSnakeSettingsPane_.getEnergyType(),
									globalSnakeSettingsPane_.getGlobalEnergyType(),
									globalSnakeSettingsPane_.getMaxIterations(),
									globalSnakeSettingsPane_.isImmortal(),
									globalSnakeSettingsPane_.showCommonEdges(),
									globalSnakeSettingsPane_.getIntermediateSubdiv());
							
							activeSnakeKeeper.setSnakeParameters(snakeParameters, null);
							
							System.out.println(activeSnakeKeeper.snake_);
							
							progressFrame1.close();
							
						} else {
							System.err.println("Could not "
									+ "loadSnakeParametersFromInterface "
									+ "(no nuclei detection) !");
						}
					}
				}
			}
		} finally {
			keepersListLock_.unlock();
		}
	}

	// ----------------------------------------------------------------------------
	// I/O

	/** Load the snakes from an XML file. */
	public void loadSnakesFromXML() {
		Thread thr = new Thread() {
			@Override
			public void run() {
				try {
					Sequence focusedSequence = getActiveSequence();
					if (focusedSequence == null) {
						new AnnounceFrame(
								"No sequence loaded. Please open an image first.");
						return;
					}

					keepersListLock_.lock();
					try {
						if (keepersListTable_.containsKey(focusedSequence)) {
							KeepersListSD keepersList = keepersListTable_
									.get(focusedSequence);
							keepersList.deactivateActiveSnakeKeeper();
						}
					} finally {
						keepersListLock_.unlock();
					}
					
					/* should be updated if it appears that XML I/O is useful for GlobalSnake */ 
					MessageDialog.showDialog(
							"XML I/O not supported.", 
							JOptionPane.WARNING_MESSAGE);
					return;

//					try {
//						ImageLUTContainerSD_noGUI imageLUTContainer = imageLUTs_
//								.get(focusedSequence);
//
//						if (imageLUTContainer == null) {
//							System.err.println("LUT not found");
//							return;
//						}
//
//						ArrayList<SnakeKeeperSD> loadedKeepers = IOXMLSD
//								.loadSnakesFromXML(imageLUTContainer,
//										mainPlugin_);
//						if (loadedKeepers != null) {
//							imageSettingsPane_
//									.sequenceFocused(imageLUTContainer
//											.getOriginalSequence());
//							for (SnakeKeeperSD loadedKeeper : loadedKeepers) {
//								if (loadedKeeper != null) {
//									// partial adaptation to GlobalSnakeSD (NOT TESTED !)
//									GlobalSnakeParametersSD snakeParameters = 
//											(GlobalSnakeParametersSD) loadedKeeper
//											.getESnakeParameters();
//									globalSnakeSettingsPane_
//											.setSnakeParameters(snakeParameters);
//									nSnakes_++;
//									loadedKeeper.setID("" + nSnakes_);
//									loadedKeeper
//											.setSnakeEditMode(getEditingMode());
//
//									keepersListLock_.lock();
//									try {
//										if (keepersListTable_
//												.containsKey(focusedSequence)) {
//											KeepersListSD list = keepersListTable_
//													.get(focusedSequence);
//											list.addAndActivateKeeper(loadedKeeper);
//										} else {
//											KeepersListSD list = new KeepersListSD();
//											list.addAndActivateKeeper(loadedKeeper);
//											keepersListTable_.put(
//													focusedSequence, list);
//										}
//									} finally {
//										keepersListLock_.unlock();
//									}
//									deactivateSnake(loadedKeeper);
//								}
//							}
//						}
//					} catch (Exception e) {
//						new AnnounceFrame(
//								"The snakes could not be loaded. XML file is corrupt or non-valid.");
//						System.err
//								.println("The snakes could not be loaded. XML file is corrupt or non-valid.");
//						e.printStackTrace();
//					}
				} finally {
					threadList_.remove(this);
				}
			}
		};
		threadList_.add(thr);
		thr.start();
	}

	// ----------------------------------------------------------------------------

	/** Saves the snakes to an XML file. */
	public void saveSnakesToXML() {
		Thread thr = new Thread() {
			@Override
			public void run() {
				try {
					Sequence focusedSequence = getActiveSequence();
					if (focusedSequence == null) {
						new AnnounceFrame(
								"No image detected. Please, open an image first.");
						return;
					}

					if (nSnakes_ <= 0) {
						new AnnounceFrame(
								"No snake curves detected. Please, add one first.");
						return;
					}
					
					/* should be updated if it appears that XML I/O is useful for GlobalSnake */ 
					MessageDialog.showDialog(
							"XML I/O not supported", 
							JOptionPane.WARNING_MESSAGE);
					return;
					
//					try {
//						IOXMLSD.saveSnakesToXML(imageLUTs_.get(focusedSequence));
//					} catch (RowsExceededException e) {
//						new AnnounceFrame(
//								"An error occurred while saving the snakes. XML file is corrupt or non-valid.");
//						System.err
//								.println("An error occurred while saving the snakes. XML file is corrupt or non-valid.");
//						e.printStackTrace();
//					} catch (WriteException e) {
//						new AnnounceFrame(
//								"An error occurred while saving the snakes. XML file is corrupt or non-valid.");
//						System.err
//								.println("An error occurred while saving the snakes. XML file is corrupt or non-valid.");
//						e.printStackTrace();
//					} catch (Exception e) {
//						new AnnounceFrame(
//								"An error occurred while saving the snakes. XML file is corrupt or non-valid.");
//						System.err
//								.println("An error occurred while saving the snakes. XML file is corrupt or non-valid.");
//						e.printStackTrace();
//					}
				} finally {
					threadList_.remove(this);
				}
			}
		};
		threadList_.add(thr);
		thr.start();
	}

	// ----------------------------------------------------------------------------
	// SNAKE ANALYSIS

	/** Invokes the external plug-in <code>ROIMeasures</code>. */
	public void measureSnakes() {
		MessageDialog.showDialog(
				"Measure ROI not supported.", 
				JOptionPane.WARNING_MESSAGE);
		return;
//		if (roiMeasures_ == null) {
//			roiMeasures_ = new ROIMeasures();
//			roiMeasures_.compute();
//		} else {
//			EzGUI ezGUI = roiMeasures_.getUI();
//			if (ezGUI == null) {
//				roiMeasures_ = new ROIMeasures();
//				roiMeasures_.compute();
//			}
//		}
	}
	
	
	/**
	 * Compute an approximation of a Jaccard index for the GlobalSnake
	 * by averaging over individual Jaccard indices of Subsnakes.
	 */
	public void measureJaccardOverSubsnakes(boolean withGUI, JaccardIndexMode mode) {
	    Thread thr = new Thread() {
			@Override
			public void run() {
				try {
					Sequence focusedSequence = getActiveSequence();
					if (focusedSequence == null) {
						new AnnounceFrame(
								"No image detected. Please, open an image first.");
						return;
					}
					
					keepersListLock_.lock();
					try {
						if (keepersListTable_.get(focusedSequence) != null) {
							KeepersListSD keepersList = keepersListTable_
									.get(focusedSequence);
							if (keepersList != null) {
								
								SnakeKeeperSD activeKeeperBkp =
										keepersList.getActiveSnakeKeeper();
								if (activeKeeperBkp != null) {
									// backup and deactivate currently active snake
									keepersList.deactivateActiveSnakeKeeper();
								}
								
								JaccardSubsnakeStatistics jaccard = new JaccardSubsnakeStatistics(keepersList,
										initializer_.getOrderedSeeds());
								
								jaccard.run(withGUI, mode);
								
								if (activeKeeperBkp != null) {
									// reactivate the backed up snake
									keepersList.activateSnakeKeeper(activeKeeperBkp);
								}
							}
						} else {
							new AnnounceFrame(
									"No snakes detected. Add a snake first.");
						}
					} finally {
						keepersListLock_.unlock();
					}
						
				} finally {
					threadList_.remove(this);
				}
			}
		};
		threadList_.add(thr);
		thr.start();

	}
	

	// ----------------------------------------------------------------------------
	// CLIPBOARD METHODS

	/** Copies the parameters of the snake in the clipboard. */
	private void copySnake() {
		Sequence focusedSequence = getActiveSequence();
		if (focusedSequence == null) {
			return;
		}
		keepersListLock_.lock();
		try {
			if (keepersListTable_.containsKey(focusedSequence)) {
				SnakeKeeperSD keeper = keepersListTable_.get(focusedSequence)
						.getActiveSnakeKeeper();
				if (keeper != null) {
					boolean supportedSnake;
					if (keeper.snake_ instanceof GlobalSnakeSD) {
//						clipboard_.setSnakeType(SnakeTypeSD.GLOBALSNAKE);
						MessageDialog.showDialog(
								"Copy operation not supported for a GlobalSnake", 
								JOptionPane.WARNING_MESSAGE);
						supportedSnake = false;
					} else if (keeper.snake_ instanceof SubsnakeSD) {
						clipboard_.setSnakeType(SnakeTypeSD.SUBSNAKE);
						supportedSnake = true;
					} else {
						clipboard_.setSnakeType(SnakeTypeSD.ESNAKE);
						supportedSnake = true;
					}
					
					if (supportedSnake) {
						clipboard_.setSnakeParameters(keeper.getESnakeParameters());
						clipboard_.setSnakeNodes(keeper.getNodesCopy());
						clipboard_.setCustomPriorShape(
								keeper.getCustomPriorShape());
					}
				}
			}
		} finally {
			keepersListLock_.unlock();
		}
	}

	// ----------------------------------------------------------------------------

	/** Created a new snake from the parameters stored in the clipboard. */
	private void pasteSnake() {
		if (clipboard_.isEmpty()) {
			new AnnounceFrame("The clipboard is empty. Copy a snake before.");
			return;
		}

		Thread thr = new Thread() {
			@Override
			public void run() {
				try {
					Sequence focusedSequence = getActiveSequence();
					if (focusedSequence == null) {
						new AnnounceFrame("Select an image first.");
						return;
					}

					ProgressFrame progressFrame = new ProgressFrame(
							"Pasting a snake");

					ImageLUTContainerSD imageLUTContainer = imageLUTs_
							.get(focusedSequence);

					if (imageLUTContainer == null) {
						System.err.println("LUT not found");
						return;
					}
					if (!imageLUTContainer.isLUTUpToDate()) {
						try {
							imageLUTContainer.buildLUTs();
						} catch (Exception e) {
							new AnnounceFrame(e.getMessage());
							e.printStackTrace();
							return;
						}
					}
					
					boolean supportedSnake = false;
					ESnakeSD snake = null;
					if (clipboard_.getSnakeType() == SnakeTypeSD.GLOBALSNAKE) {
//						snake = new GlobalSnakeSD(imageLUTContainer,
//								(GlobalSnakeParametersSD) 
//								clipboard_.getSnakeParameters());
						MessageDialog.showDialog(
								"Paste operation not supported for a GlobalSnake", 
								JOptionPane.WARNING_MESSAGE);
						supportedSnake = false;
					} else if (clipboard_.getSnakeType() == SnakeTypeSD.SUBSNAKE) {
						snake = new SubsnakeSD(imageLUTContainer,
								clipboard_.getSnakeParameters(), null, 
								null, 0);
						supportedSnake = true;
					} else {
						snake = new ESnakeSD(imageLUTContainer,
								clipboard_.getSnakeParameters(), null,
								clipboard_.getCustomPriorShape());
						supportedSnake = true;
					}
					
					if (supportedSnake) {
						Snake2DNode[] nodes = clipboard_.getSnakeNodes();
						for (int i = 0; i < nodes.length; i++) {
							nodes[i].x += 20;
							nodes[i].y += 20;
						}
						
						snake.setNodes(nodes);
	
						SnakeKeeperSD keeper = new SnakeKeeperSD(
								imageLUTContainer.getOriginalSequence(), snake,
								mainPlugin_);
						nSnakes_++;
						keeper.setID("" + nSnakes_);
						keeper.setSnakeEditMode(getEditingMode());
						keepersListLock_.lock();
					
						try {
							if (keepersListTable_.containsKey(focusedSequence)) {
								KeepersListSD list = keepersListTable_
										.get(focusedSequence);
								list.addAndActivateKeeper(keeper);
								globalSnakeSettingsPane_.setSnakeParameters(
										(GlobalSnakeParametersSD) keeper
										.getESnakeParameters());
							} else {
								KeepersListSD list = new KeepersListSD();
								list.addAndActivateKeeper(keeper);
								globalSnakeSettingsPane_.setSnakeParameters(
										(GlobalSnakeParametersSD) keeper
										.getESnakeParameters());
								keepersListTable_.put(focusedSequence, list);
							}
						} finally {
							keepersListLock_.unlock();
						}
					}
					
					progressFrame.close();
				} finally {
					threadList_.remove(this);
				}
			}
		};
		threadList_.add(thr);
		thr.start();
	}

	// ----------------------------------------------------------------------------
	// SNAKE OPTIMIZATION METHODS

	/**
	 * Launches the optimization routine for the active snake in the active
	 * <code>Sequence</code>.
	 */
	private void optimizeActiveSnake() {
		Thread thr = new Thread() {
			@Override
			public void run() {

				try {
					Sequence focusedSequence = getActiveSequence();
					if (focusedSequence == null) {
						new AnnounceFrame(
								"No image detected. Please, open an image first.");
						return;
					}

					ImageLUTContainerSD imageLUTContainer = imageLUTs_
							.get(focusedSequence);
			

					if (imageLUTContainer == null) {
						System.err.println("LUT not found");
						return;
					}
					
					if (!imageLUTContainer.isLUTUpToDate()) {
						try {
							imageLUTContainer.buildLUTs();
						} catch (Exception e) {
							new AnnounceFrame(e.getMessage());
							e.printStackTrace();
							return;
						}
					}

					SnakeKeeperSD activeKeeper = null;
					keepersListLock_.lock();
					try {
						if (keepersListTable_.get(focusedSequence) != null) {
							KeepersListSD keepersList = keepersListTable_
									.get(focusedSequence);
							if (keepersList != null && 
									keepersList.getActiveSnakeKeeper() != null) {
								activeKeeper = keepersList
										.getActiveSnakeKeeper();
							}
						} else {
							new AnnounceFrame(
									"No snakes detected. Add a snake first.");
						}
					} finally {
						keepersListLock_.unlock();
					}
					if (activeKeeper != null) {
						ProgressFrame progressFrame = new ProgressFrame(
								"Optimizing snake position");
						try {
							activeKeeper.startOptimization();
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						} finally {
							threadList_.remove(this);
						}
						progressFrame.close();
						
						/* this loop was before me (Aymeric) with nothing inside, 
						 * do know if it is important or not, so I leave it here, in case.
						 */
						do{
							//activeKeeper.getIsOptimizing();
							//iter++;
							//System.out.println("i");
						}
						while(activeKeeper.getIsOptimizing());
						//System.out.println("OK");
					}
				} finally {
					threadList_.remove(this);
				}
				
				System.out.println("end optimization");
			}
		};
		threadList_.add(thr);
		thr.start();

	}

	// ----------------------------------------------------------------------------

	/**
	 * Launched the optimization routine for all snakes in the active
	 * <code>Sequence</code>.
	 */
	private void optimizeAllSnakes() {
		Thread thr = new Thread() {
			@Override
			public void run() {

				try {
					
					Sequence focusedSequence = getActiveSequence();
					if (focusedSequence == null) {
						new AnnounceFrame(
								"No image detected. Please, open an image first.");
						return;
					}
					ImageLUTContainerSD imageLUTContainer = imageLUTs_
							.get(focusedSequence);

					if (imageLUTContainer == null) {
						System.err.println("LUT not found");
						return;
					}
					
					if (!imageLUTContainer.isLUTUpToDate()) {
						try {
							imageLUTContainer.buildLUTs();
						} catch (Exception e) {
							new AnnounceFrame(e.getMessage());
							e.printStackTrace();
							return;
						}
					}

					SnakeKeeperSD activeKeeper = null;
					keepersListLock_.lock();
					try {
						if (keepersListTable_.get(focusedSequence) != null) {
							KeepersListSD keepersList = keepersListTable_
									.get(focusedSequence);
							if (keepersList != null) {
								for (int i = 0; i < keepersList.getNumKeepers(); i++) {
									if (keepersList.activateSnakeKeeper(i)) {
										activeKeeper = keepersList
												.getActiveSnakeKeeper();
										
										if (activeKeeper != null) {
											activeKeeper.startOptimization();
										}
									}
									
									/* this loop was before me (with almost only comments inside), 
									 * do know if it is important or not, so I leave it here, in case.
									 */
									do{
										//activeKeeper.getIsOptimizing();
										//iter++;
										System.out.println("i");
									}
									while(activeKeeper.getIsOptimizing());
								}
							}
						} else {
							new AnnounceFrame(
									"No snakes detected. Add a snake first.");
						}
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} finally {
						keepersListLock_.unlock();
					}
				} finally {
					threadList_.remove(this);
				}
				
				System.out.println("end optimization");
			}
		};
		threadList_.add(thr);
		thr.start();
	}
	
	
	// ----------------------------------------------------------------------------
	// GUI METHODS

	/** Retrieves the interaction method specified on the GUI. */
	private SnakeEditModeSD getEditingMode() {
		if (moveSnakeButton_.isSelected()) {
			return SnakeEditModeSD.MOVE_SNAKE;
		}
		if (resizeSnakeButton_.isSelected()) {
			return SnakeEditModeSD.DILATE_SNAKE;
		}
		if (rotateSnakeButton_.isSelected()) {
			return SnakeEditModeSD.ROTATE_SNAKE;
		}
		return null;
	}

	// ----------------------------------------------------------------------------

	/** Sets an interaction method to the GUI. */
	private void setEditingMode(SnakeEditModeSD editingMode) {
		switch (editingMode) {
		case DILATE_SNAKE:
			resizeSnakeButton_.setSelected(true);
			moveSnakeButton_.setSelected(false);
			rotateSnakeButton_.setSelected(false);
			break;
		case MOVE_SNAKE:
			moveSnakeButton_.setSelected(true);
			resizeSnakeButton_.setSelected(false);
			rotateSnakeButton_.setSelected(false);
			break;
		case ROTATE_SNAKE:
			rotateSnakeButton_.setSelected(true);
			resizeSnakeButton_.setSelected(false);
			moveSnakeButton_.setSelected(false);
			break;
		}
		if (editingMode == editingMode_) {
			return;
		}
		editingMode_ = editingMode;
		keepersListLock_.lock();
		try {
			for (KeepersListSD keeperList : keepersListTable_.values()) {
				if (keeperList != null) {
					keeperList.setSnakeEditMode(editingMode);
				}
			}
		} finally {
			keepersListLock_.unlock();
		}
	}

	// ----------------------------------------------------------------------------

	/** Terminates the plug-in <code>ROIMeasures</code>. */
	private void closeROIMeasures() {
		if (roiMeasures_ != null) {
			EzGUI ezGUI = roiMeasures_.getUI();
			if (ezGUI != null) {
				ezGUI.close();
			}
		}
	}

	// ----------------------------------------------------------------------------
	// SNAKE EDITING

	/**
	 * Shifts the active snake within the active <code>Sequence</code> by a
	 * vertical and horizontal offset.
	 */
	private void translateActiveSnake(final int dx, final int dy) {
		Thread thr = new Thread() {
			@Override
			public void run() {
				try {
					Sequence focusedSequence = getActiveSequence();
					if (focusedSequence == null) {
						return;
					}
					ImageLUTContainerSD imageLUTContainer = imageLUTs_
							.get(focusedSequence);

					if (imageLUTContainer == null) {
						System.err.println("LUT not found");
						return;
					}
					if (!imageLUTContainer.isLUTUpToDate()) {
						try {
							imageLUTContainer.buildLUTs();
						} catch (Exception e) {
							new AnnounceFrame(e.getMessage());
							e.printStackTrace();
							return;
						}
					}

					keepersListLock_.lock();
					try {
						if (keepersListTable_.get(focusedSequence) != null) {
							KeepersListSD keepersList = keepersListTable_
									.get(focusedSequence);
							if (keepersList != null) {
								SnakeKeeperSD activeKeeper = keepersList
										.getActiveSnakeKeeper();
								activeKeeper.shiftSnake(dx, dy);
							}
						}
					} finally {
						keepersListLock_.unlock();
					}
				} finally {
					threadList_.remove(this);
				}
			}
		};
		threadList_.add(thr);
		thr.start();
	}

	// ----------------------------------------------------------------------------
	// LISTENER MANAGEMENT

	/**
	 * Adds a <code>KeyListener</code> to a <code>Viewer</code> if it has not
	 * one already.
	 */
	private void addKeyListenerToViewer(Viewer viewer) {
		if (!hasKeyListenerTable_.containsKey(viewer)) {
			hasKeyListenerTable_.put(viewer, true);
			viewer.addKeyListener(this);
		}
	}

	// ----------------------------------------------------------------------------

	/** Removes a <code>KeyListener</code> from a <code>>Viewer</code>. */
	private void removeKeyListenerFromViewer(Viewer viewer) {
		viewer.removeKeyListener(this);
	}

	// ----------------------------------------------------------------------------

	/** Removes all instances of <code>KeyListener</code> from all images. */
	private void removeAllKeyListeners() {
		ArrayList<Sequence> sequences = getSequences();
		for (Sequence sequence : sequences) {
			ArrayList<Viewer> viewers = sequence.getViewers();
			for (Viewer viewer : viewers) {
				removeKeyListenerFromViewer(viewer);
			}
		}
	}

	// ============================================================================
	// LISTENER METHODS

	// ----------------------------------------------------------------------------
	// ACTIONLISTENER METHODS

	/** Gets the events of the buttons from the interface. */
	@Override
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source == decomposeGlobalSnakeButton_) {
			decomposeGlobalSnake();
		} else if (source == runNucleiDetectionButton_) {
			runNucleiDetection();
		} else if (source == createGlobalSnakeButton_) {
			addAndActivateGlobalSnake();
		} else if (source == optimizeSnakeButton_) {
			optimizeActiveSnake();
		} else if (source == optimizeAllSnakesButton_) {
			optimizeAllSnakes();
		} else if (source == deleteSnakeButton_) {
			removeActiveSnake();
		}
	}
	
	

	// ----------------------------------------------------------------------------
	// ACTIVESEQUENCELISTENER METHODS

	/** Gets an event when a <code>Sequence</code> has changed. */
	@Override
	public void activeSequenceChanged(SequenceEvent event) {
	}

	// ----------------------------------------------------------------------------

	/** Gets an event when a <code>Sequence</code> has been activated (focused). */
	@Override
	public void sequenceActivated(Sequence sequence) {
		if (sequence != null) {
			nucleiDetectionSettingsPane_.sequenceFocused(sequence);
			imageSettingsPane_.sequenceFocused(sequence);
		}
	}

	// ----------------------------------------------------------------------------

	/**
	 * Gets an event when a <code>Sequence</code> has been deactivated (lost
	 * focus).
	 */
	@Override
	public void sequenceDeactivated(Sequence sequence) {
	}

	// ----------------------------------------------------------------------------
	// ACTIVEVIEWERLISTENER METHODS

	/** Gets an event when a <code>Viewer</code> has changed. */
	@Override
	public void activeViewerChanged(ViewerEvent event) {
	}

	// ----------------------------------------------------------------------------

	/** Gets an event when a <code>Viewer</code> has been activated (focused). */
	@Override
	public void viewerActivated(Viewer viewer) {
		if (viewer != null) {
			addKeyListenerToViewer(viewer);
		}
	}

	// ----------------------------------------------------------------------------

	/**
	 * Gets an event when a <code>Viewer</code> has been deactivated (lost
	 * focus).
	 */
	@Override
	public void viewerDeactivated(Viewer viewer) {
	}

	// ----------------------------------------------------------------------------
	// GLOBALSEQUENCELISTENER METHODS

	/** Gets an event when a <code>Sequence</code> has been opened. */
	@Override
	public void sequenceOpened(Sequence sequence) {
		if (sequence != null) {
			nucleiDetectionSettingsPane_.sequenceOpened(sequence);
		}
	}

	// ----------------------------------------------------------------------------

	/** Gets an event when a <code>Sequence</code> has been closed. */
	@Override
	public void sequenceClosed(Sequence sequence) {
		if (sequence != null) {
			nucleiDetectionSettingsPane_.sequenceClosed(sequence);
			imageSettingsPane_.sequenceClosed(sequence);
		}
	}

	// ----------------------------------------------------------------------------
	// GLOBALVIEWERLISTENER METHODS

	/** Gets an event when a <code>Viewer</code> has been opened. */
	@Override
	public void viewerOpened(Viewer viewer) {
	}

	// ----------------------------------------------------------------------------

	/** Gets an event when a <code>Viewer</code> has been closed. */
	@Override
	public void viewerClosed(Viewer viewer) {
		if (viewer != null) {
			removeKeyListenerFromViewer(viewer);
		}
	}

	// ----------------------------------------------------------------------------
	// KEYLISTENER METHODS

	/**
	 * Interactions with respect to <code>KeyEvent</code> to detect the COPY and
	 * PASTE commands.
	 */
	@Override
	public void keyPressed(KeyEvent e) {
		int key = e.getKeyCode();
		int keymod = e.getModifiers();
		int defmod = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();

		if (key == KeyEvent.VK_DELETE || key == KeyEvent.VK_BACK_SPACE) {
			removeActiveSnake();
		} else if (key == KeyEvent.VK_C && (keymod & defmod) != 0) {
			copySnake();
		} else if (key == KeyEvent.VK_V && (keymod & defmod) != 0) {
			pasteSnake();
		} else if (key == KeyEvent.VK_LEFT) {
			translateActiveSnake(-1, 0);
		} else if (key == KeyEvent.VK_RIGHT) {
			translateActiveSnake(1, 0);
		} else if (key == KeyEvent.VK_UP) {
			translateActiveSnake(0, -1);
		} else if (key == KeyEvent.VK_DOWN) {
			translateActiveSnake(0, 1);
		}
	}

	// ----------------------------------------------------------------------------

	/** Gets an event when a key is released. */
	@Override
	public void keyReleased(KeyEvent e) {
	}

	// ----------------------------------------------------------------------------

	/** Gets an event when a key is typed. */
	@Override
	public void keyTyped(KeyEvent e) {
	}

	// ============================================================================
	// BLOCK METHODS

	/** Declares the input parameters of the block. */
	@Override
	public void declareInput(VarList inputMap) {
		inputMap.add(inputSequenceBlock_);
		inputMap.add(sigmaBlock_);
		inputMap.add(targetBrightnessBlock_);
		inputMap.add(MBlock_);
		inputMap.add(MLevelBlock_);
		inputMap.add(energyTypeBlock_);
		inputMap.add(alphaBlock_);
		inputMap.add(priorShapeTypeBlock_);
		inputMap.add(betaBlock_);
		inputMap.add(maxIterationsBlock_);
		inputMap.add(inputFormatROI_);
		inputMap.add(inputSnakeWorkbook_);
		inputMap.add(roiArray_);
	}

	// ----------------------------------------------------------------------------

	/** Declares the output parameters of the block. */
	@Override
	public void declareOutput(VarList outputMap) {
		outputMap.add(outputSnakeWorkbook_);
	}
}
