/*******************************************************************************
 * Copyright (c) 2012-2013 Biomedical Image Group (BIG), EPFL, Switzerland.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Ricard Delgado-Gonzalo (ricard.delgado@gmail.com)
 *     Nicolas Chenouard (nicolas.chenouard@gmail.com)
 *     Philippe Th&#233;venaz (philippe.thevenaz@epfl.ch)
 *     Emrah Bostan (emrah.bostan@gmail.com)
 *     Ulugbek S. Kamilov (kamilov@gmail.com)
 *     Ramtin Madani (ramtin_madani@yahoo.com)
 *     Masih Nilchian (masih_n85@yahoo.com)
 *     C&#233;dric Vonesch (cedric.vonesch@epfl.ch)
 *     Virginie Uhlmann (virginie.uhlmann@epfl.ch)
 *     Cl&#233;ment Marti (clement.marti@epfl.ch)
 *     Julien Jacquemot (julien.jacquemot@epfl.ch)
 *     Daniel Schmitter (daniel.schmitter@epfl.ch)
 ******************************************************************************/
package plugins.big.bigsnake.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import plugins.big.bigsnake.SubdivSnakeGS;
import plugins.big.bigsnake.snake.GlobalSnakeParametersSD;
import plugins.big.bigsnake.snake.Enums.ESnakeEnergyTypeSD;
import plugins.big.bigsnake.snake.Enums.ESnakeTargetTypeSD;
import plugins.big.bigsnake.snake.Enums.GlobalSnakeEnergyTypeSD;
import plugins.big.bigsnakeutils.icy.gui.CollapsiblePanel;
import plugins.big.bigsnakeutils.icy.gui.DetailPanelMode;
import plugins.big.bigsnakeutils.process.process1D.BSplineBasis.BSplineBasisType;

/**
 * Panel in which the user specifies the parameters of the GlobalSnake.
 * 
 * @version January 13, 2018
 * 
 * @author Aymeric Galan (aymeric.galan@gmail.com)
 */
public class GlobalSnakeSettingsPaneSD extends CollapsiblePanel implements 
		ChangeListener, ItemListener {
	
	/**
	 * Determines if a de-serialized file is compatible with this class.
	 * 
	 * Maintainers must change this value if and only if the new version of this
	 * class is not compatible with old versions. See Sun docs for <a
	 * href=http://java.sun.com/products/jdk/1.1/docs/guide
	 * /serialization/spec/version.doc.html> details. </a>
	 * 
	 * Not necessary to include in first version of the class, but included here
	 * as a reminder of its importance.
	 */
	private static final long serialVersionUID = -2058162207660421471L;
	
	// Global Snake fields
	private final JComboBox<String> targetBrightnessComboBox_ = new JComboBox<>();
	private final JSpinner subdivLevelSpinner_ = new JSpinner();
	private final JSpinner intermediateSubdivSpinner_ = new JSpinner();
	private final JCheckBox commonEdgesCheckBox_ = new JCheckBox("Common edges");
	private final JComboBox<String> globalEnergyTypeComboBox_ = new JComboBox<>();
	private final JSpinner maxIterationsSpinner_ = new JSpinner();
	private final JCheckBox immortalCheckBox_ = new JCheckBox("Immortal");
	private final JComboBox<String> basisFunctionComboBox_ = new JComboBox<>();
	
	// Global Snake labels
	private final JLabel targetBrightnessLabel_ = new JLabel("Target brightness");
	private final JLabel subdivLevelLabel_ = new JLabel("Subdivision level");
	private final JLabel intermediateSubdivLabel_ = new JLabel("Add control points");
	private final JLabel globalEnergyTypeLabel_ = new JLabel("Energy type");
	private final JLabel maxIterationsLabel_ = new JLabel("Max iterations");
	private final JLabel basisFunctionLabel_ = new JLabel("Interpolant");
	
	// Title
	private final JLabel globalSnakeParametersTitleLabel_ = new JLabel("Global Snake");
	
	// Strings for ComboBoxes
	private final String[] detectTypeStrings_ = new String[] { "Dark", "Bright" };
	private final String[] energyTypeStrings_ = new String[] 
			{ "Ridge", "Ridge (sum)", "Edge (real)", "Edge (sum)" };
	private final String[] basisFunctionStrings_ = new String[] { "Deslauriers-Dubuc" };
	
	// Defaults values
	private final int defaultSubdivLevel_ = 4;
	private final int defaultIntermediateSubdiv_ = 0;
	private final int defaultMaxIterations_ = 1000;
	
	// Main plugin
	private SubdivSnakeGS bigSnake_ = null;
	
	/* flag that tells the lestener that the (spinner) change came 
	 * from user (and not from internal change, see stateChangedd() below)
	 */
	private boolean userInput_ = true;
	
	
	// ===================
	// PUBLIC METHODS
	
	public GlobalSnakeSettingsPaneSD(String title, SubdivSnakeGS bigSnake) {
		super(title);
		bigSnake_ = bigSnake;
		
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[] { 120, 150, 0 };
		gridBagLayout.rowHeights = new int[] { 25, 25, 25, 25, 
				25, 25, 25, 26, 0 };
		gridBagLayout.columnWeights = new double[] { 0.0, 1.0, Double.MIN_VALUE };
		gridBagLayout.rowWeights = new double[] { 0.0, 0.0, 0.0, 
				0.0, 0.0, 0.0, 0.0, 0.0, Double.MIN_VALUE };
		setLayout(gridBagLayout);
		
		// Target brightness ComboBox
		JPanel targetBrightnessLabelPanel = new JPanel();
		GridBagConstraints gbcTargetBrightnessLabelPanel = new GridBagConstraints();
		gbcTargetBrightnessLabelPanel.anchor = GridBagConstraints.EAST;
		gbcTargetBrightnessLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbcTargetBrightnessLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbcTargetBrightnessLabelPanel.gridx = 0;
		gbcTargetBrightnessLabelPanel.gridy = 0;
		add(targetBrightnessLabelPanel, gbcTargetBrightnessLabelPanel,
				DetailPanelMode.BASICS);
		targetBrightnessLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
		targetBrightnessLabelPanel.add(targetBrightnessLabel_);

		JPanel targetBrightnessComboBoxPanel = new JPanel();
		GridBagConstraints gbcTargetBrightnessComboBoxPanel = new GridBagConstraints();
		gbcTargetBrightnessComboBoxPanel.insets = new Insets(0, 0, 5, 0);
		gbcTargetBrightnessComboBoxPanel.fill = GridBagConstraints.BOTH;
		gbcTargetBrightnessComboBoxPanel.gridx = 1;
		gbcTargetBrightnessComboBoxPanel.gridy = 0;
		add(targetBrightnessComboBoxPanel, gbcTargetBrightnessComboBoxPanel,
				DetailPanelMode.BASICS);
		targetBrightnessComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));
		targetBrightnessComboBoxPanel.add(targetBrightnessComboBox_);
		targetBrightnessComboBox_.setModel(new DefaultComboBoxModel<String>(
				detectTypeStrings_));
		targetBrightnessComboBox_.setSelectedIndex(1);
		// end Target brightness ComboBox
		
		// Subdivision level Spinner
		JPanel subdivLevelLabelPanel = new JPanel();
		GridBagConstraints gbcSubdivLevelLabelPanel = new GridBagConstraints();
		gbcSubdivLevelLabelPanel.anchor = GridBagConstraints.EAST;
		gbcSubdivLevelLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbcSubdivLevelLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbcSubdivLevelLabelPanel.gridx = 0;
		gbcSubdivLevelLabelPanel.gridy = 1;
		add(subdivLevelLabelPanel, gbcSubdivLevelLabelPanel,
				DetailPanelMode.BASICS);
		subdivLevelLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
		subdivLevelLabelPanel.add(subdivLevelLabel_);

		JPanel subdivLevelSpinnerPanel = new JPanel();
		GridBagConstraints gbcSubdivLevelSpinnerPanel = new GridBagConstraints();
		gbcSubdivLevelSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbcSubdivLevelSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbcSubdivLevelSpinnerPanel.gridx = 1;
		gbcSubdivLevelSpinnerPanel.gridy = 1;
		add(subdivLevelSpinnerPanel, gbcSubdivLevelSpinnerPanel,
				DetailPanelMode.BASICS);
		subdivLevelSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));
		subdivLevelSpinnerPanel.add(subdivLevelSpinner_);
		final Integer levelStep = new Integer(1);
		final Integer levelMin = new Integer(1);
		final Integer levelMax = new Integer(10);
		final Integer levelDefault = new Integer(defaultSubdivLevel_);
		subdivLevelSpinner_.setModel(new SpinnerNumberModel(
				levelDefault, levelMin, levelMax, levelStep));
		// end Subdivision level Spinner
		
		// Intermediate subdiv Spinner
		JPanel intermediateSubdivLabelPanel = new JPanel();
		GridBagConstraints gbcIntermediateSubdivLabelPanel = new GridBagConstraints();
		gbcIntermediateSubdivLabelPanel.anchor = GridBagConstraints.EAST;
		gbcIntermediateSubdivLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbcIntermediateSubdivLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbcIntermediateSubdivLabelPanel.gridx = 0;
		gbcIntermediateSubdivLabelPanel.gridy = 2;
		add(intermediateSubdivLabelPanel, gbcIntermediateSubdivLabelPanel,
				DetailPanelMode.BASICS);
		intermediateSubdivLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
		intermediateSubdivLabelPanel.add(intermediateSubdivLabel_);
		
		JPanel intermediateSubdivSpinnerPanel = new JPanel();
		GridBagConstraints gbcIntermediateSubdivSpinnerPanel = new GridBagConstraints();
		gbcIntermediateSubdivSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbcIntermediateSubdivSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbcIntermediateSubdivSpinnerPanel.gridx = 1;
		gbcIntermediateSubdivSpinnerPanel.gridy = 2;
		add(intermediateSubdivSpinnerPanel, gbcIntermediateSubdivSpinnerPanel,
				DetailPanelMode.BASICS);
		intermediateSubdivSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));
		intermediateSubdivSpinnerPanel.add(intermediateSubdivSpinner_);
		final Integer interSubdivStep = new Integer(1);
		final Integer interSubdivMin = new Integer(0);
		final Integer interSubdivMax = new Integer(10);
		final Integer interSubdivDefault = new Integer(defaultIntermediateSubdiv_);
		intermediateSubdivSpinner_.setModel(new SpinnerNumberModel(
				interSubdivDefault, interSubdivMin, interSubdivMax, interSubdivStep));
		// end Intermediate subdiv Spinner
		
		// Energy type ComboBox
		JPanel globalEnergyTypeLabelPanel = new JPanel();
		GridBagConstraints gbcGlobalEnergyTypeLabelPanel = new GridBagConstraints();
		gbcGlobalEnergyTypeLabelPanel.anchor = GridBagConstraints.EAST;
		gbcGlobalEnergyTypeLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbcGlobalEnergyTypeLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbcGlobalEnergyTypeLabelPanel.gridx = 0;
		gbcGlobalEnergyTypeLabelPanel.gridy = 3;
		add(globalEnergyTypeLabelPanel, gbcGlobalEnergyTypeLabelPanel,
				DetailPanelMode.BASICS);
		globalEnergyTypeLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
		globalEnergyTypeLabelPanel.add(globalEnergyTypeLabel_);

		JPanel globalEnergyTypeComboBoxPanel = new JPanel();
		GridBagConstraints gbcglobalEnergyTypeComboBoxPanel = new GridBagConstraints();
		gbcglobalEnergyTypeComboBoxPanel.insets = new Insets(0, 0, 5, 0);
		gbcglobalEnergyTypeComboBoxPanel.fill = GridBagConstraints.BOTH;
		gbcglobalEnergyTypeComboBoxPanel.gridx = 1;
		gbcglobalEnergyTypeComboBoxPanel.gridy = 3;
		add(globalEnergyTypeComboBoxPanel, gbcglobalEnergyTypeComboBoxPanel,
				DetailPanelMode.BASICS);
		globalEnergyTypeComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));
		globalEnergyTypeComboBoxPanel.add(globalEnergyTypeComboBox_);
		globalEnergyTypeComboBox_.setModel(new DefaultComboBoxModel<String>(
				energyTypeStrings_));
		globalEnergyTypeComboBox_.setSelectedIndex(0);
		// end Energy type ComboBox
		
		// Immortal CheckBox
		JPanel immortalCheckBoxPanel = new JPanel();
		immortalCheckBoxPanel.setBorder(null);
		GridBagConstraints gbcImmortalCheckBoxPanel = new GridBagConstraints();
		gbcImmortalCheckBoxPanel.fill = GridBagConstraints.BOTH;
		gbcImmortalCheckBoxPanel.gridx = 1;
		gbcImmortalCheckBoxPanel.gridy = 4;
		add(immortalCheckBoxPanel, gbcImmortalCheckBoxPanel,
				DetailPanelMode.BASICS);
		immortalCheckBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));
		immortalCheckBox_.setSelected(false);
		maxIterationsSpinner_.setEnabled(!immortalCheckBox_.isSelected());
		immortalCheckBoxPanel.add(immortalCheckBox_);
		// end Immortal CheckBox
		
		// Max iterations Spinner
		JPanel maxIterationsLabelPanel = new JPanel();
		GridBagConstraints gbcMaxIterationsLabelPanel = new GridBagConstraints();
		gbcMaxIterationsLabelPanel.anchor = GridBagConstraints.EAST;
		gbcMaxIterationsLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbcMaxIterationsLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbcMaxIterationsLabelPanel.gridx = 0;
		gbcMaxIterationsLabelPanel.gridy = 5;
		add(maxIterationsLabelPanel, gbcMaxIterationsLabelPanel,
				DetailPanelMode.BASICS);
		maxIterationsLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
		maxIterationsLabelPanel.add(maxIterationsLabel_);

		JPanel maxIterationsSpinnerPanel = new JPanel();
		GridBagConstraints gbcMaxIterationsSpinnerPanel = new GridBagConstraints();
		gbcMaxIterationsSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbcMaxIterationsSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbcMaxIterationsSpinnerPanel.gridx = 1;
		gbcMaxIterationsSpinnerPanel.gridy = 5;
		add(maxIterationsSpinnerPanel, gbcMaxIterationsSpinnerPanel,
				DetailPanelMode.BASICS);
		maxIterationsSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));
		maxIterationsSpinnerPanel.add(maxIterationsSpinner_);
		final Integer maxIterStep = new Integer(10);
		final Integer maxIterMin = new Integer(50);
		final Integer maxIterDefault = new Integer(defaultMaxIterations_);
		maxIterationsSpinner_.setModel(new SpinnerNumberModel(
				maxIterDefault, maxIterMin, null, maxIterStep));
		// end Max iterations Spinner
		
		// Basis function ComboBox
		JPanel basisFunctionLabelPanel = new JPanel();
		GridBagConstraints gbcBasisFunctionLabelPanel = new GridBagConstraints();
		gbcBasisFunctionLabelPanel.anchor = GridBagConstraints.EAST;
		gbcBasisFunctionLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbcBasisFunctionLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbcBasisFunctionLabelPanel.gridx = 0;
		gbcBasisFunctionLabelPanel.gridy = 6;
		add(basisFunctionLabelPanel, gbcBasisFunctionLabelPanel,
				DetailPanelMode.ADVANCED);
		basisFunctionLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
		basisFunctionLabelPanel.add(basisFunctionLabel_);

		JPanel basisFunctionComboBoxPanel = new JPanel();
		GridBagConstraints gbcBasisFunctionComboBoxPanel = new GridBagConstraints();
		gbcBasisFunctionComboBoxPanel.insets = new Insets(0, 0, 5, 0);
		gbcBasisFunctionComboBoxPanel.fill = GridBagConstraints.BOTH;
		gbcBasisFunctionComboBoxPanel.gridx = 1;
		gbcBasisFunctionComboBoxPanel.gridy = 6;
		add(basisFunctionComboBoxPanel, gbcBasisFunctionComboBoxPanel,
				DetailPanelMode.ADVANCED);
		basisFunctionComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));
		basisFunctionComboBoxPanel.add(basisFunctionComboBox_);
		basisFunctionComboBox_.setModel(new DefaultComboBoxModel<String>(
				basisFunctionStrings_));
		basisFunctionComboBox_.setSelectedIndex(0);
		// end Basis function ComboBox
		
		// Remove common edges CheckBox
		JPanel commonEdgesCheckBoxPanel = new JPanel();
		commonEdgesCheckBoxPanel.setBorder(null);
		GridBagConstraints gbcCommonEdgesCheckBoxPanel = new GridBagConstraints();
		gbcCommonEdgesCheckBoxPanel.fill = GridBagConstraints.BOTH;
		gbcCommonEdgesCheckBoxPanel.gridx = 1;
		gbcCommonEdgesCheckBoxPanel.gridy = 7;
		add(commonEdgesCheckBoxPanel, gbcCommonEdgesCheckBoxPanel,
				DetailPanelMode.ADVANCED);
		commonEdgesCheckBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));
		commonEdgesCheckBox_.setSelected(false);
		commonEdgesCheckBoxPanel.add(commonEdgesCheckBox_);
		// end Remove common edges CheckBox
		
		setVisibility(activeMode_);

		// Listeners
		targetBrightnessComboBox_.addItemListener(this);
		subdivLevelSpinner_.addChangeListener(this);
		intermediateSubdivSpinner_.addChangeListener(this);
		commonEdgesCheckBox_.addItemListener(this);
		globalEnergyTypeComboBox_.addItemListener(this);
		maxIterationsSpinner_.addChangeListener(this);
		immortalCheckBox_.addItemListener(this);
		basisFunctionComboBox_.addItemListener(this);
	}
	
	// ----------------------------------------------------------------------------
	// LISTENER METHODS
	
	/** Handles the events of the spinners. */
	@Override
	public void stateChanged(ChangeEvent e) {
		Object source = e.getSource();
		if (source == maxIterationsSpinner_) {
			bigSnake_.loadSnakeParametersFromInterface();
		} else if ((source == subdivLevelSpinner_) && userInput_) {
			Integer level = (Integer) subdivLevelSpinner_.getValue();
			Integer interSubdiv = (Integer) intermediateSubdivSpinner_.getValue();
			Integer min = (Integer) ((SpinnerNumberModel) 
					intermediateSubdivSpinner_.getModel()).getMinimum();
			while (level <= interSubdiv) {
				userInput_ = false;
				intermediateSubdivSpinner_.setValue(interSubdiv-1);
				interSubdiv = (Integer) intermediateSubdivSpinner_.getValue();
				if (interSubdiv < min) {
					userInput_ = false;
					intermediateSubdivSpinner_.setValue(min);
					subdivLevelSpinner_.setValue(min+1);
					break;
				}
			}
			bigSnake_.loadSnakeParametersFromInterface();
			userInput_ = true;
		} else if ((source == intermediateSubdivSpinner_) && userInput_) {
			Integer level = (Integer) subdivLevelSpinner_.getValue();
			Integer interSubdiv = (Integer) intermediateSubdivSpinner_.getValue();
			Integer max = (Integer) ((SpinnerNumberModel) 
					subdivLevelSpinner_.getModel()).getMaximum();
			while (interSubdiv >= level) {
				userInput_ = false;
				subdivLevelSpinner_.setValue(level+1);
				level = (Integer) subdivLevelSpinner_.getValue();
				if (level > max) {
					subdivLevelSpinner_.setValue(max);
					intermediateSubdivSpinner_.setValue(max-1);
					break;
				}
			}
			bigSnake_.loadSnakeParametersFromInterface();
			userInput_ = true;
		}
	}
	
	// ------------------------------------------------------------------------

	/**
	 * Handles the events of the ComboBoxes and CheckBoxes. It fires an
	 * update of the active snake.
	 */
	@Override
	public void itemStateChanged(ItemEvent e) {
		Object source = e.getSource();
		if (source == targetBrightnessComboBox_) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				bigSnake_.loadSnakeParametersFromInterface();
			}
		} else if (source == globalEnergyTypeComboBox_) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				bigSnake_.loadSnakeParametersFromInterface();
			}
		} else if (source == immortalCheckBox_) {
			maxIterationsSpinner_.setEnabled(!immortalCheckBox_.isSelected());
			bigSnake_.loadSnakeParametersFromInterface();
		} else if (source == commonEdgesCheckBox_) {
			bigSnake_.loadSnakeParametersFromInterface();
		} else if (source == basisFunctionComboBox_) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				subdivLevelSpinner_.setEnabled(basisFunctionComboBox_
						.getSelectedItem().toString()
						.compareTo(basisFunctionStrings_[0]) == 0);
				bigSnake_.loadSnakeParametersFromInterface();
			}
		}

	}

	// ------------------------------------------------------------------------

	/** Returns the energy type selected by the user. */
	public ESnakeEnergyTypeSD getEnergyType() {
		switch (globalEnergyTypeComboBox_.getSelectedIndex()) {
		case 0:
			return ESnakeEnergyTypeSD.RIDGE;
		case 1:
			return ESnakeEnergyTypeSD.RIDGE;
		case 2:
			return ESnakeEnergyTypeSD.CONTOUR;
		case 3:
			return ESnakeEnergyTypeSD.CONTOUR;
		}
		return null;
	}
	
	
	/** Returns the energy type selected by the user. */
	public GlobalSnakeEnergyTypeSD getGlobalEnergyType() {
		switch (globalEnergyTypeComboBox_.getSelectedIndex()) {
		case 0:
			return GlobalSnakeEnergyTypeSD.REAL;
		case 1:
			return GlobalSnakeEnergyTypeSD.SUM;
		case 2:
			return GlobalSnakeEnergyTypeSD.REAL;
		case 3:
			return GlobalSnakeEnergyTypeSD.SUM;
		}
		return null;
	}
	
	// ------------------------------------------------------------------------
	
	/** Returns the common edges treatment selected by the user. */
	public boolean showCommonEdges() {
		return (commonEdgesCheckBox_.isSelected());
	}

	// ------------------------------------------------------------------------
	
	/** Returns the maximum number of iterations introduced by the user. */
	public Integer getMaxIterations() {
		return (Integer) maxIterationsSpinner_.getValue();
	}

	// ------------------------------------------------------------------------
	
	/** Returns the number of subdivision introduced by the user. */
	public Integer getSubdivLevel() {
		return (Integer) subdivLevelSpinner_.getValue();
	}

	// ------------------------------------------------------------------------
	
	/** Set the number of subdivision. */
	public void setSubdivLevelSpinner(int level) {
		subdivLevelSpinner_.setValue(level);
	}
	
	// ------------------------------------------------------------------------
	
	/** Returns the number of subdivision introduced by the user. */
	public Integer getIntermediateSubdiv() {
		return (Integer) intermediateSubdivSpinner_.getValue();
	}

	// ------------------------------------------------------------------------
	
	/** Returns the basis function selected by the user. */
	public BSplineBasisType getBasisFunctionType() {
		switch (basisFunctionComboBox_.getSelectedIndex()) {
		case 0:
			return BSplineBasisType.DESLAURIERSDUBUC;		
		}
		return null;
	}

	// ------------------------------------------------------------------------
	
	/** Returns the brightness detection mode selected by the user. */
	public ESnakeTargetTypeSD getTargetBrightness() {
		switch (targetBrightnessComboBox_.getSelectedIndex()) {
		case 0:
			return ESnakeTargetTypeSD.DARK;
		case 1:
			return ESnakeTargetTypeSD.BRIGHT;
		}
		return null;
	}

	// ------------------------------------------------------------------------
	
	public boolean isImmortal() {
		return immortalCheckBox_.isSelected();
	}
	
	// ------------------------------------------------------------------------
	
	/** Sets the snake parameters to the interface. */
	public void setSnakeParameters(GlobalSnakeParametersSD snakeParameters) {
		subdivLevelSpinner_.setValue(snakeParameters.getSubdivisionLevel());
		maxIterationsSpinner_.setValue(snakeParameters.getMaxLife());
		intermediateSubdivSpinner_.setValue(
				snakeParameters.getIntermediateSubdiv());
		immortalCheckBox_.setSelected(snakeParameters.isImmortal());
		commonEdgesCheckBox_.setSelected(
				snakeParameters.showCommonEdges());
		switch (snakeParameters.getGlobalEnergyType()) {
		case REAL:
			if (snakeParameters.getEnergyType() == ESnakeEnergyTypeSD.RIDGE)
				globalEnergyTypeComboBox_.setSelectedIndex(0);
			else if (snakeParameters.getEnergyType() == ESnakeEnergyTypeSD.CONTOUR)
				globalEnergyTypeComboBox_.setSelectedIndex(2);
			break;
		case SUM:
			if (snakeParameters.getEnergyType() == ESnakeEnergyTypeSD.RIDGE)
				globalEnergyTypeComboBox_.setSelectedIndex(1);
			else if (snakeParameters.getEnergyType() == ESnakeEnergyTypeSD.CONTOUR)
				globalEnergyTypeComboBox_.setSelectedIndex(3);
			break;
		}
		switch (snakeParameters.getDetectType()) {
		case DARK:
			targetBrightnessComboBox_.setSelectedIndex(0);
			break;
		case BRIGHT:
			targetBrightnessComboBox_.setSelectedIndex(1);
			break;
		}
		switch (snakeParameters.getBasisFunctionType()) {
		case DESLAURIERSDUBUC:
			// TODO implement other interpolants ?
			basisFunctionComboBox_.setSelectedIndex(0);
			break;
		default:
			break;
		}
	}

}
