package plugins.big.bigsnake.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.DecimalFormat;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import plugins.big.bigsnake.SubdivSnakeGS;
import plugins.big.bigsnake.snake.Enums.GlobalSnakeInitialization;
import plugins.big.bigsnake.snake.Enums.GlobalSnakeSortingType;
import plugins.big.bigsnakeutils.icy.gui.CollapsiblePanel;
import plugins.big.bigsnakeutils.icy.gui.DetailPanelMode;

/**
 * Panel in which the user specifies the parameters 
 * of GlobalSnake initialization methods.
 * 
 * @version January 13, 2018
 * 
 * @author Aymeric Galan (aymeric.galan@gmail.com)
 */
public class InitializationSettingsPaneSD extends CollapsiblePanel implements ItemListener, ChangeListener {
	/**
	 * Determines if a de-serialized file is compatible with this class.
	 * 
	 * Maintainers must change this value if and only if the new version of this
	 * class is not compatible with old versions. See Sun docs for <a
	 * href=http://java.sun.com/products/jdk/1.1/docs/guide
	 * /serialization/spec/version.doc.html> details. </a>
	 * 
	 * Not necessary to include in first version of the class, but included here
	 * as a reminder of its importance.
	 */
	private static final long serialVersionUID = -1174716044020562682L;
	
	// Initialization fields
	private final JComboBox<String> initializationComboBox_ = new JComboBox<>();
	private final JComboBox<GlobalSnakeSortingType> sortingTypeComboBox_ = new JComboBox<>();
	private final JSpinner convexHullDilationCoefSpinner_ = new JSpinner();
	
	// Initialization labels
	private final JLabel initializationLabel_ = new JLabel("Type");
	private final JLabel sortingTypeLabel_ = new JLabel("Subsnakes sorting");
	private final JLabel convexHullDilationCoefLabel_ = new JLabel("Hull dilation [%]");
	
	// Title
	private final JLabel initializationTitleLabel_ = new JLabel("Initialization");
	
	// Strings for ComboBoxes
	private final String[] initializationTypeStrings_ = new String[] { "Voronoi + convex hull" };
	
	// Defaults values
//	private final double defaultDilationCoef_ = 1.8;
	private final int defaultDilationCoefInt_ = 180;
	private final static double DISPLAY_COEF = 100;
	
	// Main plugin
	private SubdivSnakeGS bigSnake_ = null;
	
	
	// ============================================================================
	// PUBLIC METHODS
	
	public InitializationSettingsPaneSD(String title, SubdivSnakeGS bigSnake) {
		super(title);
		bigSnake_ = bigSnake;

		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[] { 120, 150, 0 };
		gridBagLayout.rowHeights = new int[] { 25, 25, 26, 0 };
		gridBagLayout.columnWeights = new double[] { 0.0, 1.0, Double.MIN_VALUE };
		gridBagLayout.rowWeights = new double[] { 0.0, 0.0, 0.0,
				Double.MIN_VALUE };
		setLayout(gridBagLayout);
		
		// Initialization ComboBox
		JPanel initializationLabelPanel = new JPanel();
		GridBagConstraints gbcInitializationLabelPanel = new GridBagConstraints();
		gbcInitializationLabelPanel.anchor = GridBagConstraints.EAST;
		gbcInitializationLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbcInitializationLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbcInitializationLabelPanel.gridx = 0;
		gbcInitializationLabelPanel.gridy = 0;
		add(initializationLabelPanel, gbcInitializationLabelPanel,
				DetailPanelMode.BASICS);
		initializationLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
		initializationLabelPanel.add(initializationLabel_);
		
		JPanel initializationComboBoxPanel = new JPanel();
		GridBagConstraints gbcInitializationComboBoxPanel = new GridBagConstraints();
		gbcInitializationComboBoxPanel.anchor = GridBagConstraints.EAST;
		gbcInitializationComboBoxPanel.insets = new Insets(0, 0, 5, 0);
		gbcInitializationComboBoxPanel.fill = GridBagConstraints.BOTH;
		gbcInitializationComboBoxPanel.gridx = 1;
		gbcInitializationComboBoxPanel.gridy = 0;
		add(initializationComboBoxPanel, gbcInitializationComboBoxPanel,
				DetailPanelMode.BASICS);
		initializationComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));
		initializationComboBoxPanel.add(initializationComboBox_);
		initializationComboBox_.setModel(new DefaultComboBoxModel<String>(
				initializationTypeStrings_));
		initializationComboBox_.setSelectedIndex(0);
		// end Initialization ComboBox
		
		// Convex hull dilation coef Spinner
		JPanel convexHullDilationCoefLabelPanel = new JPanel();
		GridBagConstraints gbcConvexHullDilationCoefLabelPanel = new GridBagConstraints();
		gbcConvexHullDilationCoefLabelPanel.anchor = GridBagConstraints.EAST;
		gbcConvexHullDilationCoefLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbcConvexHullDilationCoefLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbcConvexHullDilationCoefLabelPanel.gridx = 0;
		gbcConvexHullDilationCoefLabelPanel.gridy = 1;
		add(convexHullDilationCoefLabelPanel, gbcConvexHullDilationCoefLabelPanel,
				DetailPanelMode.BASICS);
		convexHullDilationCoefLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
		convexHullDilationCoefLabelPanel.add(convexHullDilationCoefLabel_);

		JPanel convexHullDilationCoefSpinnerPanel = new JPanel();
		GridBagConstraints gbc_maxIterationsSpinnerPanel = new GridBagConstraints();
		gbc_maxIterationsSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbc_maxIterationsSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbc_maxIterationsSpinnerPanel.gridx = 1;
		gbc_maxIterationsSpinnerPanel.gridy = 1;
		add(convexHullDilationCoefSpinnerPanel, gbc_maxIterationsSpinnerPanel,
				DetailPanelMode.BASICS);
		convexHullDilationCoefSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));
		convexHullDilationCoefSpinnerPanel.add(convexHullDilationCoefSpinner_);
		
		/* NOTE : there are issues if we use a spinner with Doubles...
		 * so we use Integers and divide by 10 to get the double value 
		 * (related to digits and limited machine precision)
		 */
//		final Double coefMin = new Double(1.1);
//		final Double coefStep = new Double(0.1);
//		final Double coefMax = new Double(5.0);
//		final Double coefDefault = new Double(defaultDilationCoef_);
//		SpinnerNumberModel model = new SpinnerNumberModel(
//				coefDefault, coefMin, coefMax, coefStep);
		final Integer coefMinInt = new Integer(110);
		final Integer coefStepInt = new Integer(5);
		final Integer coefMaxInt = new Integer(500);
		final Integer coefDefaultInt = new Integer(defaultDilationCoefInt_);
		SpinnerNumberModel model = new SpinnerNumberModel(
				coefDefaultInt, coefMinInt, coefMaxInt, coefStepInt);
		
		// Sorting type ComboBox
		JPanel sortingTypeLabelPanel = new JPanel();
		GridBagConstraints gbcSortingTypeLabelPanel = new GridBagConstraints();
		gbcSortingTypeLabelPanel.anchor = GridBagConstraints.EAST;
		gbcSortingTypeLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbcSortingTypeLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbcSortingTypeLabelPanel.gridx = 0;
		gbcSortingTypeLabelPanel.gridy = 2;
		add(sortingTypeLabelPanel, gbcSortingTypeLabelPanel,
				DetailPanelMode.ADVANCED);
		sortingTypeLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
		sortingTypeLabelPanel.add(sortingTypeLabel_);
		
		JPanel sortingTypeComboBoxPanel = new JPanel();
		GridBagConstraints gbcSortingTypeComboBoxPanel = new GridBagConstraints();
		gbcSortingTypeComboBoxPanel.anchor = GridBagConstraints.EAST;
		gbcSortingTypeComboBoxPanel.insets = new Insets(0, 0, 5, 0);
		gbcSortingTypeComboBoxPanel.fill = GridBagConstraints.BOTH;
		gbcSortingTypeComboBoxPanel.gridx = 1;
		gbcSortingTypeComboBoxPanel.gridy = 2;
		add(sortingTypeComboBoxPanel, gbcSortingTypeComboBoxPanel,
				DetailPanelMode.ADVANCED);
		sortingTypeComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));
		sortingTypeComboBoxPanel.add(sortingTypeComboBox_);
		sortingTypeComboBox_.setModel(new DefaultComboBoxModel<GlobalSnakeSortingType>(
				GlobalSnakeSortingType.values()));
		sortingTypeComboBox_.setSelectedIndex(0);
		// end Sorting type ComboBox
		
		convexHullDilationCoefSpinner_.setModel(model);
		// end Convex hull dilation coef Spinner
		
		setVisibility(activeMode_);
		
		// Listeners
		initializationComboBox_.addItemListener(this);
		convexHullDilationCoefSpinner_.addChangeListener(this);
		sortingTypeComboBox_.addItemListener(this);
	}
	
	// ------------------------------------------------------------------------

	/** Handles the events of the spinner. */
	@Override
	public void stateChanged(ChangeEvent e) {
		Object source = e.getSource();
		if (source == convexHullDilationCoefSpinner_) {
			bigSnake_.loadSnakeParametersFromInterface();
		}
	}
	
	// ------------------------------------------------------------------------

	/** Handles the events of comboboxes. */
	@Override
	public void itemStateChanged(ItemEvent e) {
		Object source = e.getSource();
		if (source == initializationComboBox_) {
			/* do nothing since it is the only option available for now. */
		} else if (source == sortingTypeComboBox_) {
			bigSnake_.loadSnakeParametersFromInterface();
		}
	}
	
	// ------------------------------------------------------------------------
	
	public double getConvexHullDilationCoef() {
		Integer valueInt = (Integer) convexHullDilationCoefSpinner_.getValue();
		double value = ((double) valueInt.intValue()) / DISPLAY_COEF;
		return value;
	}
	
	// ------------------------------------------------------------------------
	
	public GlobalSnakeInitialization getInitializationType() {
		switch (initializationComboBox_.getSelectedIndex()) {
		case 0:
			return GlobalSnakeInitialization.BOUNDED_VORONOI;
		}
		return null;
	}
	
	// ------------------------------------------------------------------------
	
	public GlobalSnakeSortingType getSortingType() {
		return (GlobalSnakeSortingType) sortingTypeComboBox_.getSelectedItem();
	}
	
	// ------------------------------------------------------------------------

//	private double formatNumber(double value) {
//		DecimalFormat fmt = new DecimalFormat("###.##");  // one decimal
//		String valueFormatted = fmt.format(value);
//		if (valueFormatted.contains(","))
//			valueFormatted = valueFormatted.replace(",", ".");
//		return Double.parseDouble(valueFormatted);
//	}
	
}
