package plugins.big.bigsnake.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import icy.main.Icy;
import icy.sequence.Sequence;
import plugins.big.bigsnake.nuclei.NucleiDetector;
import plugins.big.bigsnakeutils.icy.gui.CollapsiblePanel;
import plugins.big.bigsnakeutils.icy.gui.DetailPanelMode;

/**
 * Panel in which the user specifies the parameters of the nuclei detector.
 * 
 * @version January 13, 2018
 * 
 * @author Aymeric Galan (aymeric.galan@gmail.com)
 */
public class NucleiDetectionSettingsPaneSD extends CollapsiblePanel implements ActionListener, ChangeListener {
	/**
	 * Determines if a de-serialized file is compatible with this class.
	 * 
	 * Maintainers must change this value if and only if the new version of this
	 * class is not compatible with old versions. See Sun docs for <a
	 * href=http://java.sun.com/products/jdk/1.1/docs/guide
	 * /serialization/spec/version.doc.html> details. </a>
	 * 
	 * Not necessary to include in first version of the class, but included here
	 * as a reminder of its importance.
	 */
	private static final long serialVersionUID = -7160511440207462628L;
	
	// Nuclei detection fields
	private final JCheckBox manualModeCheckBox_ = new JCheckBox("Manual mode");
	private final JComboBox<String> sequenceWithNucleiComboBox_ = new JComboBox<>();
	private final JSpinner sigmaLoGSpinner_ = new JSpinner();
	private final JSpinner thresholdSpinner_ = new JSpinner();
	
	// Nuclei detection labels
	private final JLabel sequenceWithNucleiLabel_ = new JLabel("Image with nuclei");
	private final JLabel sigmaLoGLabel_ = new JLabel("Sigma LoG_noGUI filter");
	private final JLabel thresholdLabel_ = new JLabel("Threshold (0, 255)");
	
	// Title
	private final JLabel nucleiDetectionTitleLabel_ = new JLabel("Nuclei detection");
	
	// Defaults values
	private final double defaultSigmaLoG_ = 10;
	private final double defaultThreshold_ = 120;
	
	/** Focused image. */
	private Sequence activeSequence_ = null;
	
	/** NucleiDetector. */
	private NucleiDetector detector_ = null;
	
	
	// ============================================================================
	// PUBLIC METHODS
	
	public NucleiDetectionSettingsPaneSD(String title, NucleiDetector detector) {
		super(title);
		detector_ = detector;
		activeSequence_ = Icy.getMainInterface().getActiveSequence();
		
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[] { 120, 150, 0 };
		gridBagLayout.rowHeights = new int[] { 25, 25, 25, 25, 0 };
		gridBagLayout.columnWeights = new double[] { 0.0, 1.0, Double.MIN_VALUE };
		gridBagLayout.rowWeights = new double[] { 0.0, 0.0, 0.0, 0.0,
				Double.MIN_VALUE };
		setLayout(gridBagLayout);
		
		// Manual mode CheckBox
		JPanel manualModeCheckBoxPanel = new JPanel();
		manualModeCheckBoxPanel.setBorder(null);
		GridBagConstraints gbcManualModeCheckBoxPanel = new GridBagConstraints();
		gbcManualModeCheckBoxPanel.fill = GridBagConstraints.BOTH;
		gbcManualModeCheckBoxPanel.gridx = 1;
		gbcManualModeCheckBoxPanel.gridy = 0;
		add(manualModeCheckBoxPanel, gbcManualModeCheckBoxPanel,
				DetailPanelMode.BASICS);
		manualModeCheckBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));
		manualModeCheckBoxPanel.add(manualModeCheckBox_);
		manualModeCheckBox_.setSelected(true);
		if (Icy.getMainInterface().getSequences().size() < 2)
			manualModeCheckBox_.setEnabled(false);
		else
			manualModeCheckBox_.setEnabled(true);
		// end Manual mode CheckBox
	
		// Image ComboBox
		JPanel sequenceWithNucleiLabelPanel = new JPanel();
		GridBagConstraints gbcSequenceWithNucleiLabelPanel = new GridBagConstraints();
		gbcSequenceWithNucleiLabelPanel.anchor = GridBagConstraints.EAST;
		gbcSequenceWithNucleiLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbcSequenceWithNucleiLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbcSequenceWithNucleiLabelPanel.gridx = 0;
		gbcSequenceWithNucleiLabelPanel.gridy = 1;
		add(sequenceWithNucleiLabelPanel, gbcSequenceWithNucleiLabelPanel,
				DetailPanelMode.BASICS);
		sequenceWithNucleiLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
		sequenceWithNucleiLabelPanel.add(sequenceWithNucleiLabel_);

		JPanel sequenceWithNucleiComboBoxPanel = new JPanel();
		GridBagConstraints gbcSequenceWithNucleiComboBoxPanel = new GridBagConstraints();
		gbcSequenceWithNucleiComboBoxPanel.insets = new Insets(0, 0, 5, 0);
		gbcSequenceWithNucleiComboBoxPanel.fill = GridBagConstraints.BOTH;
		gbcSequenceWithNucleiComboBoxPanel.gridx = 1;
		gbcSequenceWithNucleiComboBoxPanel.gridy = 1;
		add(sequenceWithNucleiComboBoxPanel, gbcSequenceWithNucleiComboBoxPanel,
				DetailPanelMode.BASICS);
		sequenceWithNucleiComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));
		sequenceWithNucleiComboBoxPanel.add(sequenceWithNucleiComboBox_);
		sequenceWithNucleiComboBox_.setEnabled(!manualModeCheckBox_.isSelected());
		// end Image ComboBox
		
		// Sigma LoG_noGUI Spinner
		JPanel sigmaLoGLabelPanel = new JPanel();
		GridBagConstraints gbcSigmaLoGLabelPanel = new GridBagConstraints();
		gbcSigmaLoGLabelPanel.anchor = GridBagConstraints.EAST;
		gbcSigmaLoGLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbcSigmaLoGLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbcSigmaLoGLabelPanel.gridx = 0;
		gbcSigmaLoGLabelPanel.gridy = 2;
		add(sigmaLoGLabelPanel, gbcSigmaLoGLabelPanel,
				DetailPanelMode.BASICS);
		sigmaLoGLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
		sigmaLoGLabelPanel.add(sigmaLoGLabel_);

		JPanel sigmaLoGSpinnerPanel = new JPanel();
		GridBagConstraints gbcSigmaLoGSpinnerPanel = new GridBagConstraints();
		gbcSigmaLoGSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbcSigmaLoGSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbcSigmaLoGSpinnerPanel.gridx = 1;
		gbcSigmaLoGSpinnerPanel.gridy = 2;
		add(sigmaLoGSpinnerPanel, gbcSigmaLoGSpinnerPanel,
				DetailPanelMode.BASICS);
		sigmaLoGSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));
		sigmaLoGSpinnerPanel.add(sigmaLoGSpinner_);
		final Double sigmaStep = new Double(1);
		final Double sigmaMin = new Double(1);
		final Double sigmaDefault = new Double(defaultSigmaLoG_);
		sigmaLoGSpinner_.setModel(new SpinnerNumberModel(sigmaDefault, 
				sigmaMin, null, sigmaStep));
		sigmaLoGSpinner_.setEnabled(!manualModeCheckBox_.isSelected());
		// end Sigma LoG_noGUI Spinner
		
		// Threshold Spinner
		JPanel thresholdLabelPanel = new JPanel();
		GridBagConstraints gbcThresholdLabelPanel = new GridBagConstraints();
		gbcThresholdLabelPanel.anchor = GridBagConstraints.EAST;
		gbcThresholdLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbcThresholdLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbcThresholdLabelPanel.gridx = 0;
		gbcThresholdLabelPanel.gridy = 3;
		add(thresholdLabelPanel, gbcThresholdLabelPanel,
				DetailPanelMode.BASICS);
		thresholdLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
		thresholdLabelPanel.add(thresholdLabel_);

		JPanel thresholdSpinnerPanel = new JPanel();
		GridBagConstraints gbcThresholdSpinnerPanel = new GridBagConstraints();
		gbcThresholdSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbcThresholdSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbcThresholdSpinnerPanel.gridx = 1;
		gbcThresholdSpinnerPanel.gridy = 3;
		add(thresholdSpinnerPanel, gbcThresholdSpinnerPanel,
				DetailPanelMode.BASICS);
		thresholdSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));
		thresholdSpinnerPanel.add(thresholdSpinner_);
		final Double thresholdStep = new Double(1);
		final Double thresholdMin = new Double(1);
		final Double thresholdMax = new Double(254);
		final Double thresholdDefault = new Double(defaultThreshold_);
		thresholdSpinner_.setModel(new SpinnerNumberModel(thresholdDefault, 
				thresholdMin, thresholdMax, thresholdStep));
		thresholdSpinner_.setEnabled(!manualModeCheckBox_.isSelected());
		// end Threshold Spinner
		
		setVisibility(activeMode_);
		
		// Listeners
		sequenceWithNucleiComboBox_.addActionListener(this);
		sigmaLoGSpinner_.addChangeListener(this);
		manualModeCheckBox_.addActionListener(this);
	
		refreshSequenceWithNucleiComboBox();
	}
	
	// ------------------------------------------------------------------------
	
	public void sequenceFocused(Sequence sequence) {
		activeSequence_ = sequence;
		sequenceWithNucleiComboBox_.removeActionListener(this);
		if (sequence != null) {
			refreshSequenceWithNucleiComboBox();
		}
		sequenceWithNucleiComboBox_.addActionListener(this);
		
		// deactivate the check box if less than 2 sequences opened
		if (Icy.getMainInterface().getSequences().size() < 2) {
			manualModeCheckBox_.setSelected(true);
			manualModeCheckBox_.setEnabled(false);
		} else {
			manualModeCheckBox_.setEnabled(true);
		}
	}
	
	// ------------------------------------------------------------------------
	
	public void sequenceOpened(Sequence sequence) {
		sequenceWithNucleiComboBox_.removeActionListener(this);
		if (sequence != null) {
			refreshSequenceWithNucleiComboBox();
		}
		sequenceWithNucleiComboBox_.addActionListener(this);
		
		// deactivate the check box if less than 2 sequences opened
		if (Icy.getMainInterface().getSequences().size() < 2) {
			manualModeCheckBox_.setSelected(true);
			manualModeCheckBox_.setEnabled(false);
		} else {
			manualModeCheckBox_.setEnabled(true);
		}
	}
	
	// ------------------------------------------------------------------------
	
	public void sequenceClosed(Sequence sequence) {
		if (sequence == activeSequence_)
			activeSequence_ = null;
		
		sequenceWithNucleiComboBox_.removeActionListener(this);
		if (sequence != null) {
			refreshSequenceWithNucleiComboBox();
		}
		sequenceWithNucleiComboBox_.addActionListener(this);
	
		// deactivate the check box if less than 2 sequences opened
		if (Icy.getMainInterface().getSequences().size() < 2) {
			manualModeCheckBox_.setSelected(true);
			manualModeCheckBox_.setEnabled(false);
		} else {
			manualModeCheckBox_.setEnabled(true);
		}
	}

	// ------------------------------------------------------------------------
	
	/** Handles the events of the spinner. */
	@Override
	public void stateChanged(ChangeEvent e) {
		Object source = e.getSource();
		if (source == sigmaLoGSpinner_) {
			detector_.setSigmaLoG((double) sigmaLoGSpinner_.getValue());
		} else if (source == thresholdSpinner_) {
			detector_.setThreshold((double) thresholdSpinner_.getValue());
		}
	}
	
	// ------------------------------------------------------------------------

	/** Handles the events of the combobox and checkbox. */
	@Override
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source == manualModeCheckBox_) {
			detector_.setManualMode(manualModeCheckBox_.isSelected());
			sequenceWithNucleiComboBox_.setEnabled(!manualModeCheckBox_.isSelected());
			sigmaLoGSpinner_.setEnabled(!manualModeCheckBox_.isSelected());
			thresholdSpinner_.setEnabled(!manualModeCheckBox_.isSelected());
		} else if (source == sequenceWithNucleiComboBox_) {
			Sequence sequence = Icy.getMainInterface().getSequences()
					.get(sequenceWithNucleiComboBox_.getSelectedIndex());
			detector_.setSequence(sequence);
		}
	}
	
	// ------------------------------------------------------------------------
	
	public boolean isManualMode() {
		return manualModeCheckBox_.isSelected();
	}
	
	// ------------------------------------------------------------------------
	
	public Sequence getSequenceWithNuclei() {
		return Icy.getMainInterface().getSequences()
				.get(sequenceWithNucleiComboBox_.getSelectedIndex());
	}
	
	// ------------------------------------------------------------------------
	
	public double getSigmaLoG() {
		return (double) sigmaLoGSpinner_.getValue();
	}
	
	// ------------------------------------------------------------------------
	
	public void setSigmaLoG(double sigma) {
		sigmaLoGSpinner_.setValue(sigma);
	}
	
	// ------------------------------------------------------------------------
	
	public double getThreshold() {
		return (double) thresholdSpinner_.getValue();
	}
	
	// ------------------------------------------------------------------------
	
	public void setThreshold(double threshold) {
		thresholdSpinner_.setValue(threshold);
	}
	
	
	// ============================================================================
	// PRIVATE METHODS

	/**
	 * Updates the content of the combo box associated to the selection of the
	 * channel.
	 */
	private void refreshSequenceWithNucleiComboBox() {
		sequenceWithNucleiComboBox_.removeAllItems();
		if (activeSequence_ != null) {
			ArrayList<Sequence> allOpenedSequences = 
					Icy.getMainInterface().getSequences();
			for (int i = 0 ; i < allOpenedSequences.size() ; i++) {
				Sequence seq = allOpenedSequences.get(i);
				sequenceWithNucleiComboBox_.addItem(seq.getName());
				if (seq != activeSequence_)
					sequenceWithNucleiComboBox_.setSelectedIndex(i);
			}
		}
	}
}
