/*******************************************************************************
 * Copyright (c) 2012-2013 Biomedical Image Group (BIG), EPFL, Switzerland.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Ricard Delgado-Gonzalo (ricard.delgado@gmail.com)
 *     Nicolas Chenouard (nicolas.chenouard@gmail.com)
 *     Philippe Th&#233;venaz (philippe.thevenaz@epfl.ch)
 *     Emrah Bostan (emrah.bostan@gmail.com)
 *     Ulugbek S. Kamilov (kamilov@gmail.com)
 *     Ramtin Madani (ramtin_madani@yahoo.com)
 *     Masih Nilchian (masih_n85@yahoo.com)
 *     C&#233;dric Vonesch (cedric.vonesch@epfl.ch)
 *     Virginie Uhlmann (virginie.uhlmann@epfl.ch)
 *     Cl&#233;ment Marti (clement.marti@epfl.ch)
 *     Julien Jacquemot (julien.jacquemot@epfl.ch)
 *     Daniel Schmitter (daniel.schmitter@epfl.ch)
 ******************************************************************************/
package plugins.big.bigsnake.gui;

import icy.gui.frame.progress.AnnounceFrame;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;







//import plugins.big.bigsnake.BIGSnake;
import plugins.big.bigsnake.SubdivSnakeGS;
import plugins.big.bigsnake.core.SettingsSD;
import plugins.big.bigsnake.io.IOXMLSD;
import plugins.big.bigsnake.snake.ESnakeParametersSD;
import plugins.big.bigsnake.snake.Enums.ESnakeEnergyTypeSD;
import plugins.big.bigsnake.snake.Enums.ESnakePriorShapeTypeSD;
import plugins.big.bigsnake.snake.Enums.ESnakeTargetTypeSD;
import plugins.big.bigsnake.snake.Enums.MultiresolutionTypeSD;
import plugins.big.bigsnake.snake.Enums.ShapeSpaceTypeSD;
import plugins.big.bigsnakeutils.icy.gui.CollapsiblePanel;
import plugins.big.bigsnakeutils.icy.gui.DetailPanelMode;
import plugins.big.bigsnakeutils.icy.snake2D.Snake2DNode;
import plugins.big.bigsnakeutils.process.process1D.BSplineBasis.BSplineBasisType;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Banana;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Boomerang;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.BrainWhite;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.CorpusCallosum;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Cross;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Custom;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.ElectricGuitar;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Femur;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Fly;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.MouseOrgan;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Rod;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Semicircle;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Square;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.UpperLip;

/**
 * Panel in which the user specifies the parameters of the E-Snake.
 * 
 * @version May 3, 2014
 * 
 * @author Ricard Delgado-Gonzalo (ricard.delgado@gmail.com)
 * @author Daniel Schmitter (daniel.schmitter@epfl.ch)
 * @author Ana�s Badoual (anais.badoual@gmail.com)
 */
public class SnakeSettingsPaneSD extends CollapsiblePanel implements
		ChangeListener, ItemListener {

	/**
	 * Determines if a de-serialized file is compatible with this class.
	 * 
	 * Maintainers must change this value if and only if the new version of this
	 * class is not compatible with old versions. See Sun docs for <a
	 * href=http://java.sun.com/products/jdk/1.1/docs/guide
	 * /serialization/spec/version.doc.html> details. </a>
	 * 
	 * Not necessary to include in first version of the class, but included here
	 * as a reminder of its importance.
	 */
	private static final long serialVersionUID = -2078162621447160540L;

	private final JComboBox targetBrightnessComboBox_ = new JComboBox();
	private final JSpinner numControlPointsSpinner_ = new JSpinner();
	private final JSpinner numSubdivLevelSpinner_ = new JSpinner();
	private final JSpinner mScalingSpinner_ = new JSpinner();
	private final JComboBox energyTypeComboBox_ = new JComboBox();
	private final JSpinner alphaSpinner_ = new JSpinner();
	private final JComboBox priorShapeComboBox_ = new JComboBox();
	private final JComboBox shapeSpaceComboBox_ = new JComboBox();
	private final JSpinner betaSpinner_ = new JSpinner();
	private final JSpinner maxIterationsSpinner_ = new JSpinner();
	private final JCheckBox immortalCheckbox_ = new JCheckBox("Immortal");
	private final JComboBox basisFunctionComboBox_ = new JComboBox();
	private final JComboBox multiresolutionComboBox_ = new JComboBox();


	private final JLabel targetBrightnessLabel_ = new JLabel(
			"Target brightness");
	private final JLabel numControlPointsLabel_ = new JLabel("Control points");
	private final JLabel numSubdivLevelLabel_ = new JLabel("Iterations");
	private final JLabel mScalingLabel_ = new JLabel("Refinement factor");
	private final JLabel energyTypeLabel_ = new JLabel("Energy type");
	private final JLabel alphaLabel_ = new JLabel("Alpha");
	private final JLabel priorShapeLabel_ = new JLabel("Prior shape");
	private final JLabel shapeSpaceLabel_ = new JLabel("Shape space");
	private final JLabel betaLabel_ = new JLabel("Beta");
	private final JLabel maxIterationsLabel_ = new JLabel("Max iterations");
	private final JLabel basisFunctionLabel_ = new JLabel("Mask");
	private final JLabel snakeParametersTitleLabel_=new JLabel("Snake parameters");
	private final JLabel subdivisionParametersTitleLabel_=new JLabel("Subdivision parameters");
	private final JLabel multiresolutionTitleLabel_=new JLabel("Multiresolution");


	private final String[] energyTypeStrings_ = new String[] { "Contour",
			"Region", "Mixture" };
	private final String[] detectTypeStrings_ = new String[] { "Dark", "Bright" };
	private final String[] priorShapeStrings_ = new String[] { "None", "Rod",
			"Banana", "Boomerang", "Brain white", "Circle", "Corpus callosum",
			"Cross", "Electric guitar", "Femur", "Fly", "Mouse organ",
			"Semicircle", "Square", "Upper lip", "Custom" };
	private final String[] shapeSpaceStrings_ = new String[] {"Similarity", "Affine"};
	private final String[] basisFunctionStrings_ = new String[] {
			"Quadratic polynomial B-Spline", 
			"Cubic polynomial B-Spline", 
			"Ellipse-reproducing exponential B-Spline", 
			"Deslauriers-Dubuc function", 
			"Ell-rep D-D func gamma=[0,0,2jPi/N0,2jPi/N0,-2jPi/N0,-2jPi/N0]",
			"Ell-rep D-D func gamma=[0,0,2jPi/N0,-2jPi/N0]"};
	private final String[] multiresolutionStrings_ = new String[] {"Manual", "Automatic"};
	
	private SubdivSnakeGS bigSnake_ = null;

	private Custom customPriorShape_ = null;

	// ============================================================================
	// PUBLIC METHODS

	/** Default constructor. */
	public SnakeSettingsPaneSD(String title, SubdivSnakeGS bigSnake) {
		super(title);
		bigSnake_ = bigSnake;

		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[] { 120, 150, 0 };
		gridBagLayout.rowHeights = new int[] { 27, 28, 27, 28, 28, 28, 28, 23,
				0 };
		gridBagLayout.columnWeights = new double[] { 0.0, 1.0, Double.MIN_VALUE };
		gridBagLayout.rowWeights = new double[] { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
				0.0, 0.0, Double.MIN_VALUE };
		setLayout(gridBagLayout);

		//Title:snake parameters
		JPanel snakeParametersTitleLabelPanel = new JPanel();
		GridBagConstraints gbc_snakeParametersTitleLabelPanel = new GridBagConstraints();
		gbc_snakeParametersTitleLabelPanel.anchor = GridBagConstraints.WEST;
		gbc_snakeParametersTitleLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_snakeParametersTitleLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_snakeParametersTitleLabelPanel.gridx = 0;
		gbc_snakeParametersTitleLabelPanel.gridy = 0;
		add(snakeParametersTitleLabelPanel, gbc_snakeParametersTitleLabelPanel,
				DetailPanelMode.BASICS);
		snakeParametersTitleLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		snakeParametersTitleLabelPanel.add(snakeParametersTitleLabel_);
		snakeParametersTitleLabel_.setFont(new Font("", Font.BOLD, 13));
		
		//Brightness
		JPanel targetBrightnessLabelPanel = new JPanel();
		GridBagConstraints gbc_targetBrightnessLabelPanel = new GridBagConstraints();
		gbc_targetBrightnessLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_targetBrightnessLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_targetBrightnessLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_targetBrightnessLabelPanel.gridx = 0;
		gbc_targetBrightnessLabelPanel.gridy = 1;
		add(targetBrightnessLabelPanel, gbc_targetBrightnessLabelPanel,
				DetailPanelMode.BASICS);
		targetBrightnessLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		targetBrightnessLabelPanel.add(targetBrightnessLabel_);

		JPanel targetBrightnessComboBoxPanel = new JPanel();
		GridBagConstraints gbc_targetBrightnessComboBoxPanel = new GridBagConstraints();
		gbc_targetBrightnessComboBoxPanel.insets = new Insets(0, 0, 5, 0);
		gbc_targetBrightnessComboBoxPanel.fill = GridBagConstraints.BOTH;
		gbc_targetBrightnessComboBoxPanel.gridx = 1;
		gbc_targetBrightnessComboBoxPanel.gridy = 1;
		add(targetBrightnessComboBoxPanel, gbc_targetBrightnessComboBoxPanel,
				DetailPanelMode.BASICS);
		targetBrightnessComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));

		targetBrightnessComboBoxPanel.add(targetBrightnessComboBox_);
		targetBrightnessComboBox_.setModel(new DefaultComboBoxModel(
				detectTypeStrings_));
		targetBrightnessComboBox_.setSelectedIndex(0);

		//Control points
		JPanel numControlPointsLabelPanel = new JPanel();
		GridBagConstraints gbc_numControlPointsLabelPanel = new GridBagConstraints();
		gbc_numControlPointsLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_numControlPointsLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_numControlPointsLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_numControlPointsLabelPanel.gridx = 0;
		gbc_numControlPointsLabelPanel.gridy = 2;
		add(numControlPointsLabelPanel, gbc_numControlPointsLabelPanel,
				DetailPanelMode.BASICS);
		numControlPointsLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		numControlPointsLabelPanel.add(numControlPointsLabel_);
		

		JPanel numControlPointsSpinnerPanel = new JPanel();
		GridBagConstraints gbc_numControlPointsSpinnerPanel = new GridBagConstraints();
		gbc_numControlPointsSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbc_numControlPointsSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbc_numControlPointsSpinnerPanel.gridx = 1;
		gbc_numControlPointsSpinnerPanel.gridy = 2;
		add(numControlPointsSpinnerPanel, gbc_numControlPointsSpinnerPanel,
				DetailPanelMode.BASICS);
		numControlPointsSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));

		numControlPointsSpinnerPanel.add(numControlPointsSpinner_);
		numControlPointsSpinner_.setModel(new SpinnerNumberModel(new Integer(
				SettingsSD.M_DEFAULT), new Integer(3), new Integer(20),
				new Integer(1)));

		//Energy
		JPanel energyTypeLabelPanel = new JPanel();
		GridBagConstraints gbc_energyTypeLabelPanel = new GridBagConstraints();
		gbc_energyTypeLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_energyTypeLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_energyTypeLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_energyTypeLabelPanel.gridx = 0;
		gbc_energyTypeLabelPanel.gridy = 3;
		add(energyTypeLabelPanel, gbc_energyTypeLabelPanel,
				DetailPanelMode.BASICS);
		energyTypeLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		energyTypeLabelPanel.add(energyTypeLabel_);

		JPanel energyTypeComboBoxPanel = new JPanel();
		GridBagConstraints gbc_energyTypeComboBoxPanel = new GridBagConstraints();
		gbc_energyTypeComboBoxPanel.insets = new Insets(0, 0, 5, 0);
		gbc_energyTypeComboBoxPanel.fill = GridBagConstraints.BOTH;
		gbc_energyTypeComboBoxPanel.gridx = 1;
		gbc_energyTypeComboBoxPanel.gridy = 3;
		add(energyTypeComboBoxPanel, gbc_energyTypeComboBoxPanel,
				DetailPanelMode.BASICS);
		energyTypeComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));

		energyTypeComboBoxPanel.add(energyTypeComboBox_);
		energyTypeComboBox_.setModel(new DefaultComboBoxModel(
				energyTypeStrings_));
		energyTypeComboBox_.setSelectedIndex(1);

		//Alpha parameter
		JPanel alphaLabelPanel = new JPanel();
		GridBagConstraints gbc_alphaLabelPanel = new GridBagConstraints();
		gbc_alphaLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_alphaLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_alphaLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_alphaLabelPanel.gridx = 0;
		gbc_alphaLabelPanel.gridy = 4;
		add(alphaLabelPanel, gbc_alphaLabelPanel, DetailPanelMode.BASICS);
		alphaLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		alphaLabelPanel.add(alphaLabel_);

		JPanel alphaSpinnerPanel = new JPanel();
		GridBagConstraints gbc_alphaSpinnerPanel = new GridBagConstraints();
		gbc_alphaSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbc_alphaSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbc_alphaSpinnerPanel.gridx = 1;
		gbc_alphaSpinnerPanel.gridy = 4;
		add(alphaSpinnerPanel, gbc_alphaSpinnerPanel, DetailPanelMode.BASICS);
		alphaSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));

		alphaSpinnerPanel.add(alphaSpinner_);
		alphaSpinner_.setModel(new SpinnerNumberModel(SettingsSD.ALPHA_DEFAULT,
				0.0, 1.0, 0.01));
		alphaSpinner_.setEnabled(energyTypeComboBox_.getSelectedItem()
				.toString().compareTo(energyTypeStrings_[2]) == 0);

		//Prior shape
		JPanel priorShapeLabelPanel = new JPanel();
		GridBagConstraints gbc_priorShapeLabelPanel = new GridBagConstraints();
		gbc_priorShapeLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_priorShapeLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_priorShapeLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_priorShapeLabelPanel.gridx = 0;
		gbc_priorShapeLabelPanel.gridy = 5;
		add(priorShapeLabelPanel, gbc_priorShapeLabelPanel,
				DetailPanelMode.BASICS);
		priorShapeLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		priorShapeLabelPanel.add(priorShapeLabel_);

		JPanel priorShapeComboBoxPanel = new JPanel();
		GridBagConstraints gbc_priorShapeComboBoxPanel = new GridBagConstraints();
		gbc_priorShapeComboBoxPanel.insets = new Insets(0, 0, 5, 0);
		gbc_priorShapeComboBoxPanel.fill = GridBagConstraints.BOTH;
		gbc_priorShapeComboBoxPanel.gridx = 1;
		gbc_priorShapeComboBoxPanel.gridy = 5;
		add(priorShapeComboBoxPanel, gbc_priorShapeComboBoxPanel,
				DetailPanelMode.BASICS);
		priorShapeComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));

		priorShapeComboBoxPanel.add(priorShapeComboBox_);
		priorShapeComboBox_.setModel(new DefaultComboBoxModel(
				priorShapeStrings_));
		priorShapeComboBox_.setSelectedIndex(0);
		
		//Shape space 
//		JPanel shapeSpaceLabelPanel = new JPanel();
//		GridBagConstraints gbc_shapeSpaceLabelPanel = new GridBagConstraints();
//		gbc_shapeSpaceLabelPanel.anchor = GridBagConstraints.EAST;
//		gbc_shapeSpaceLabelPanel.insets = new Insets(0, 0, 5, 5);
//		gbc_shapeSpaceLabelPanel.fill = GridBagConstraints.VERTICAL;
//		gbc_shapeSpaceLabelPanel.gridx = 0;
//		gbc_shapeSpaceLabelPanel.gridy = 6;
//		add(shapeSpaceLabelPanel, gbc_shapeSpaceLabelPanel,
//				DetailPanelMode.ADVANCED);
//		shapeSpaceLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));
//
//		shapeSpaceLabelPanel.add(shapeSpaceLabel_);
//		
//		JPanel shapeSpaceComboBoxPanel = new JPanel();
//		GridBagConstraints gbc_shapeSpaceComboBoxPanel = new GridBagConstraints();
//		gbc_shapeSpaceComboBoxPanel.insets = new Insets(0, 0, 5, 0);
//		gbc_shapeSpaceComboBoxPanel.fill = GridBagConstraints.BOTH;
//		gbc_shapeSpaceComboBoxPanel.gridx = 1;
//		gbc_shapeSpaceComboBoxPanel.gridy = 6;
//		add(shapeSpaceComboBoxPanel, gbc_shapeSpaceComboBoxPanel,
//				DetailPanelMode.ADVANCED);
//		shapeSpaceComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));
//
//		shapeSpaceComboBoxPanel.add(shapeSpaceComboBox_);
//		shapeSpaceComboBox_.setModel(new DefaultComboBoxModel(
//				shapeSpaceStrings_));
//		shapeSpaceComboBox_.setSelectedIndex(0);

		//Beta parameter
//		JPanel priorBetaPanel = new JPanel();
//		GridBagConstraints gbc_betaLabelPanel = new GridBagConstraints();
//		gbc_betaLabelPanel.anchor = GridBagConstraints.EAST;
//		gbc_betaLabelPanel.insets = new Insets(0, 0, 5, 5);
//		gbc_betaLabelPanel.fill = GridBagConstraints.VERTICAL;
//		gbc_betaLabelPanel.gridx = 0;
//		gbc_betaLabelPanel.gridy = 7;
//		add(priorBetaPanel, gbc_betaLabelPanel, DetailPanelMode.ADVANCED);
//		priorBetaPanel.setLayout(new GridLayout(0, 1, 0, 0));
//
//		priorBetaPanel.add(betaLabel_);
//
//		JPanel betaSpinnerPanel = new JPanel();
//		GridBagConstraints gbc_betaSpinnerPanel = new GridBagConstraints();
//		gbc_betaSpinnerPanel.insets = new Insets(0, 0, 5, 0);
//		gbc_betaSpinnerPanel.fill = GridBagConstraints.BOTH;
//		gbc_betaSpinnerPanel.gridx = 1;
//		gbc_betaSpinnerPanel.gridy = 7;
//		add(betaSpinnerPanel, gbc_betaSpinnerPanel, DetailPanelMode.ADVANCED);
//		betaSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));
//
//		betaSpinnerPanel.add(betaSpinner_);
//		betaSpinner_.setModel(new SpinnerNumberModel(SettingsSD.BETA_DEFAULT,
//				0.0, Double.MAX_VALUE, 0.5));
//		betaSpinner_.setEnabled(priorShapeComboBox_.getSelectedIndex() != 0);

		//Optimizer iterations
		JPanel maxIterationsLabelPanel = new JPanel();
		GridBagConstraints gbc_maxIterationsLabelPanel = new GridBagConstraints();
		gbc_maxIterationsLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_maxIterationsLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_maxIterationsLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_maxIterationsLabelPanel.gridx = 0;
		gbc_maxIterationsLabelPanel.gridy = 6;
		add(maxIterationsLabelPanel, gbc_maxIterationsLabelPanel,
				DetailPanelMode.BASICS);
		maxIterationsLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		maxIterationsLabelPanel.add(maxIterationsLabel_);

		JPanel maxIterationsSpinnerPanel = new JPanel();
		GridBagConstraints gbc_maxIterationsSpinnerPanel = new GridBagConstraints();
		gbc_maxIterationsSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbc_maxIterationsSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbc_maxIterationsSpinnerPanel.gridx = 1;
		gbc_maxIterationsSpinnerPanel.gridy = 6;
		add(maxIterationsSpinnerPanel, gbc_maxIterationsSpinnerPanel,
				DetailPanelMode.BASICS);
		maxIterationsSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));

		maxIterationsSpinnerPanel.add(maxIterationsSpinner_);
		maxIterationsSpinner_
				.setModel(new SpinnerNumberModel(SettingsSD.MAX_LIFE_DEFAULT,
						new Integer(0), null, new Integer(1)));

		//Immortal
		JPanel immortalCheckboxPanel = new JPanel();
		immortalCheckboxPanel.setBorder(null);
		GridBagConstraints gbc_immortalCheckboxPanel = new GridBagConstraints();
		gbc_immortalCheckboxPanel.fill = GridBagConstraints.BOTH;
		gbc_immortalCheckboxPanel.gridx = 1;
		gbc_immortalCheckboxPanel.gridy = 7;
		add(immortalCheckboxPanel, gbc_immortalCheckboxPanel,
				DetailPanelMode.BASICS);
		immortalCheckboxPanel.setLayout(new GridLayout(0, 1, 0, 0));

		immortalCheckbox_.setSelected(false);
		maxIterationsSpinner_.setEnabled(!immortalCheckbox_.isSelected());

		immortalCheckboxPanel.add(immortalCheckbox_);
		
		//Title: Subdivision parameters
		JPanel subdivisionParametersTitleLabelPanel = new JPanel();
		GridBagConstraints gbc_subdivisionParametersTitleLabelPanel = new GridBagConstraints();
		gbc_subdivisionParametersTitleLabelPanel.anchor = GridBagConstraints.WEST;
		gbc_subdivisionParametersTitleLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_subdivisionParametersTitleLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_subdivisionParametersTitleLabelPanel.gridx = 0;
		gbc_subdivisionParametersTitleLabelPanel.gridy = 8;
		add(subdivisionParametersTitleLabelPanel, gbc_subdivisionParametersTitleLabelPanel,
				DetailPanelMode.BASICS);
		subdivisionParametersTitleLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		subdivisionParametersTitleLabelPanel.add(subdivisionParametersTitleLabel_);
		subdivisionParametersTitleLabel_.setFont(new Font("", Font.BOLD, 13));
		
		// Subdivision level
		JPanel numSubdivLevelLabelPanel = new JPanel();
		GridBagConstraints gbc_numSubdivLevelLabelPanel = new GridBagConstraints();
		gbc_numSubdivLevelLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_numSubdivLevelLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_numSubdivLevelLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_numSubdivLevelLabelPanel.gridx = 0;
		gbc_numSubdivLevelLabelPanel.gridy = 9;
		add(numSubdivLevelLabelPanel, gbc_numSubdivLevelLabelPanel,
				DetailPanelMode.BASICS);
		numSubdivLevelLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		numSubdivLevelLabelPanel.add(numSubdivLevelLabel_);
		
		JPanel numSubdivLevelSpinnerPanel = new JPanel();
		GridBagConstraints gbc_numSubdivLevelSpinnerPanel = new GridBagConstraints();
		gbc_numSubdivLevelSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbc_numSubdivLevelSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbc_numSubdivLevelSpinnerPanel.gridx = 1;
		gbc_numSubdivLevelSpinnerPanel.gridy = 9;
		add(numSubdivLevelSpinnerPanel, gbc_numSubdivLevelSpinnerPanel,
				DetailPanelMode.BASICS);
		numSubdivLevelSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));

		numSubdivLevelSpinnerPanel.add(numSubdivLevelSpinner_);
		numSubdivLevelSpinner_.setModel(new SpinnerNumberModel(new Integer(
				SettingsSD.SDLEVEL_DEFAULT), new Integer(0), new Integer(10),
				new Integer(1)));
		
		//Subdivision refinement factor
		JPanel mScalingLabelPanel = new JPanel();
		GridBagConstraints gbc_mScalingLabelPanel = new GridBagConstraints();
		gbc_mScalingLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_mScalingLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_mScalingLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_mScalingLabelPanel.gridx = 0;
		gbc_mScalingLabelPanel.gridy = 10;
		add(mScalingLabelPanel, gbc_mScalingLabelPanel,
				DetailPanelMode.BASICS);
		mScalingLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		mScalingLabelPanel.add(mScalingLabel_);
		
		JPanel mScalingSpinnerPanel = new JPanel();
		GridBagConstraints gbc_mScalingSpinnerPanel = new GridBagConstraints();
		gbc_mScalingSpinnerPanel.insets = new Insets(0, 0, 5, 0);
		gbc_mScalingSpinnerPanel.fill = GridBagConstraints.BOTH;
		gbc_mScalingSpinnerPanel.gridx = 1;
		gbc_mScalingSpinnerPanel.gridy = 10;
		add(mScalingSpinnerPanel, gbc_mScalingSpinnerPanel,
				DetailPanelMode.BASICS);
		mScalingSpinnerPanel.setLayout(new GridLayout(0, 1, 0, 0));

		mScalingSpinnerPanel.add(mScalingSpinner_);
		mScalingSpinner_.setModel(new SpinnerNumberModel(new Integer(
				SettingsSD.SDMSCALING_DEFAULT), new Integer(2), new Integer(10),
				new Integer(1)));
		
		// Basis function 
		JPanel basisFunctionLabelPanel = new JPanel();
		GridBagConstraints gbc_basisFunctionLabelPanel = new GridBagConstraints();
		gbc_basisFunctionLabelPanel.anchor = GridBagConstraints.EAST;
		gbc_basisFunctionLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_basisFunctionLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_basisFunctionLabelPanel.gridx = 0;
		gbc_basisFunctionLabelPanel.gridy = 11;
		add(basisFunctionLabelPanel, gbc_basisFunctionLabelPanel,
				DetailPanelMode.BASICS);
		basisFunctionLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		basisFunctionLabelPanel.add(basisFunctionLabel_);
		

		JPanel basisFunctionComboBoxPanel = new JPanel();
		GridBagConstraints gbc_basisFunctionComboBoxPanel = new GridBagConstraints();
		gbc_basisFunctionComboBoxPanel.insets = new Insets(0, 0, 5, 0);
		gbc_basisFunctionComboBoxPanel.fill = GridBagConstraints.BOTH;
		gbc_basisFunctionComboBoxPanel.gridx = 1;
		gbc_basisFunctionComboBoxPanel.gridy = 11;
		add(basisFunctionComboBoxPanel, gbc_basisFunctionComboBoxPanel,
				DetailPanelMode.BASICS);
		basisFunctionComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));

		basisFunctionComboBoxPanel.add(basisFunctionComboBox_);
		basisFunctionComboBox_.setModel(new DefaultComboBoxModel(
				basisFunctionStrings_));
		basisFunctionComboBox_.setSelectedIndex(2);
		
		mScalingSpinner_.setEnabled(basisFunctionComboBox_.getSelectedItem()
				.toString().compareTo(basisFunctionStrings_[0]) == 0 || basisFunctionComboBox_.getSelectedItem()
				.toString().compareTo(basisFunctionStrings_[1]) == 0 );
		numControlPointsSpinner_.setEnabled(basisFunctionComboBox_.getSelectedItem()
				.toString().compareTo(basisFunctionStrings_[0]) == 0 || basisFunctionComboBox_.getSelectedItem()
				.toString().compareTo(basisFunctionStrings_[1]) == 0 || basisFunctionComboBox_.getSelectedItem()
				.toString().compareTo(basisFunctionStrings_[2]) == 0 || basisFunctionComboBox_.getSelectedItem()
				.toString().compareTo(basisFunctionStrings_[3]) == 0);
		numSubdivLevelSpinner_.setEnabled(basisFunctionComboBox_.getSelectedItem()
				.toString().compareTo(basisFunctionStrings_[0]) == 0 || basisFunctionComboBox_.getSelectedItem()
				.toString().compareTo(basisFunctionStrings_[1]) == 0 || basisFunctionComboBox_.getSelectedItem()
				.toString().compareTo(basisFunctionStrings_[2]) == 0 || basisFunctionComboBox_.getSelectedItem()
				.toString().compareTo(basisFunctionStrings_[3]) == 0);
		
		if(basisFunctionComboBox_.getSelectedItem()
				.toString().compareTo(basisFunctionStrings_[4]) == 0 || basisFunctionComboBox_.getSelectedItem()
				.toString().compareTo(basisFunctionStrings_[5]) == 0){
			numControlPointsSpinner_.setValue(4);
			numSubdivLevelSpinner_.setValue(6);
		}

		//Title: Multiresolution
		JPanel multiresolutionTitleLabelPanel = new JPanel();
		GridBagConstraints gbc_multiresolutionTitleLabelPanel = new GridBagConstraints();
		gbc_multiresolutionTitleLabelPanel.anchor = GridBagConstraints.WEST;
		gbc_multiresolutionTitleLabelPanel.insets = new Insets(0, 0, 5, 5);
		gbc_multiresolutionTitleLabelPanel.fill = GridBagConstraints.VERTICAL;
		gbc_multiresolutionTitleLabelPanel.gridx = 0;
		gbc_multiresolutionTitleLabelPanel.gridy = 12;
		add(multiresolutionTitleLabelPanel, gbc_multiresolutionTitleLabelPanel,
				DetailPanelMode.BASICS);
		multiresolutionTitleLabelPanel.setLayout(new GridLayout(0, 1, 0, 0));

		multiresolutionTitleLabelPanel.add(multiresolutionTitleLabel_);
		multiresolutionTitleLabel_.setFont(new Font("", Font.BOLD, 13));
		
		JPanel multiresolutionComboBoxPanel = new JPanel();
		GridBagConstraints gbc_multiresolutionComboBoxPanel = new GridBagConstraints();
		gbc_multiresolutionComboBoxPanel.insets = new Insets(0, 0, 5, 0);
		gbc_multiresolutionComboBoxPanel.fill = GridBagConstraints.BOTH;
		gbc_multiresolutionComboBoxPanel.gridx = 1;
		gbc_multiresolutionComboBoxPanel.gridy = 13;
		add(multiresolutionComboBoxPanel, gbc_multiresolutionComboBoxPanel,
				DetailPanelMode.BASICS);
		multiresolutionComboBoxPanel.setLayout(new GridLayout(0, 1, 0, 0));

		multiresolutionComboBoxPanel.add(multiresolutionComboBox_);
		multiresolutionComboBox_.setModel(new DefaultComboBoxModel(
				multiresolutionStrings_));
		multiresolutionComboBox_.setSelectedIndex(0);
				
		//Listener		

		setVisibility(activeMode_);

		targetBrightnessComboBox_.addItemListener(this);
		numControlPointsSpinner_.addChangeListener(this);
		numSubdivLevelSpinner_.addChangeListener(this);
		mScalingSpinner_.addChangeListener(this);
		energyTypeComboBox_.addItemListener(this);
		alphaSpinner_.addChangeListener(this);
		priorShapeComboBox_.addItemListener(this);
		shapeSpaceComboBox_.addItemListener(this);
		betaSpinner_.addChangeListener(this);
		maxIterationsSpinner_.addChangeListener(this);
		immortalCheckbox_.addItemListener(this);
		basisFunctionComboBox_.addItemListener(this);
		multiresolutionComboBox_.addItemListener(this);
	}

	// ----------------------------------------------------------------------------

	/** Returns the energy tradeoff parameter introduced by the user. */
	public Double getAlpha() {
		return (Double) alphaSpinner_.getValue();
	}

	// ----------------------------------------------------------------------------

	/** Returns the energy weight associated to the prior-shape energy. */
	public Double getBeta() {
		return (Double) betaSpinner_.getValue();
	}

	// ----------------------------------------------------------------------------

	public Custom getCustomPriorShape() {
		return customPriorShape_;
	}

	// ----------------------------------------------------------------------------

	/** Returns the energy type selected by the user. */
	public ESnakeEnergyTypeSD getEnergyType() {
		switch (energyTypeComboBox_.getSelectedIndex()) {
		case 0:
			return ESnakeEnergyTypeSD.CONTOUR;
		case 1:
			return ESnakeEnergyTypeSD.REGION;
		case 2:
			return ESnakeEnergyTypeSD.MIXTURE;
		}
		return null;
	}

	// ----------------------------------------------------------------------------

	/** Returns the maximum number of iterations introduced by the user. */
	public Integer getMaxIterations() {
		return (Integer) maxIterationsSpinner_.getValue();
	}

	// ----------------------------------------------------------------------------

	/** Returns the number of control points introduced by the user. */
	public Integer getNumControlPoints() {
		return (Integer) numControlPointsSpinner_.getValue();
	}

	// ----------------------------------------------------------------------------

	/** Returns the number of subdivision introduced by the user. */
	public Integer getNumSubdivLevel() {
		return (Integer) numSubdivLevelSpinner_.getValue();
	}

	// ----------------------------------------------------------------------------

	/** Set the number of subdivision. */
	public void setNumSubdivLevelSpinner(int level) {
		numSubdivLevelSpinner_.setValue(level);
	}
	// ----------------------------------------------------------------------------
	
	/** Returns the m scaling factor introduced by the user. */
	public Integer getMScalingFactor() {
		return (Integer) mScalingSpinner_.getValue();
	}

	// ----------------------------------------------------------------------------
	
	/** Returns the energy type selected by the user. */
	public ESnakePriorShapeTypeSD getPriorShapeType() {
		switch (priorShapeComboBox_.getSelectedIndex()) {
		case 0:
			return ESnakePriorShapeTypeSD.NONE;
		case 1:
			return ESnakePriorShapeTypeSD.ROD;
		case 2:
			return ESnakePriorShapeTypeSD.BANANA;
		case 3:
			return ESnakePriorShapeTypeSD.BOOMERANG;
		case 4:
			return ESnakePriorShapeTypeSD.BRAINWHITE;
		case 5:
			return ESnakePriorShapeTypeSD.CIRCLE;
		case 6:
			return ESnakePriorShapeTypeSD.CORPUSCALLOSUM;
		case 7:
			return ESnakePriorShapeTypeSD.CROSS;
		case 8:
			return ESnakePriorShapeTypeSD.ELECTRICGUITAR;
		case 9:
			return ESnakePriorShapeTypeSD.FEMUR;
		case 10:
			return ESnakePriorShapeTypeSD.FLY;
		case 11:
			return ESnakePriorShapeTypeSD.MOUSEORGAN;
		case 12:
			return ESnakePriorShapeTypeSD.SEMICIRCLE;
		case 13:
			return ESnakePriorShapeTypeSD.SQUARE;
		case 14:
			return ESnakePriorShapeTypeSD.UPPERLIP;
		case 15:
			return ESnakePriorShapeTypeSD.CUSTOM;
		}
		return null;
	}
	
	
	// ----------------------------------------------------------------------------

		/** Returns the shape space type selected by the user. */
		public ShapeSpaceTypeSD getShapeSpaceType() {
			switch (shapeSpaceComboBox_.getSelectedIndex()) {
			case 0:
				return ShapeSpaceTypeSD.SIMILARITY;
			case 1:
				return ShapeSpaceTypeSD.AFFINE;			
			}
			return null;
		}

	// ----------------------------------------------------------------------------
		/** Returns the basis function selected by the user. */
		public BSplineBasisType getBasisFunctionType() {
			switch (basisFunctionComboBox_.getSelectedIndex()) {
			case 0:
				return BSplineBasisType.QUADRATICBSPLINE;
			case 1:
				return BSplineBasisType.CUBICBSPLINE;	
			case 2:
				return BSplineBasisType.ESPLINE3;		
			case 3:
				return BSplineBasisType.DESLAURIERSDUBUC;	
			case 4:
				return BSplineBasisType.GDESLAURIERSDUBUC;	
			case 5:
				return BSplineBasisType.GDESLAURIERSDUBUC2;	
			}
			return null;
		}

	// ----------------------------------------------------------------------------
		/** Returns the multiresolution type selected by the user. */
		public MultiresolutionTypeSD getMultiresolutionType() {
			switch (multiresolutionComboBox_.getSelectedIndex()) {
			case 0:
				return MultiresolutionTypeSD.MANUAL;
			case 1:
				return MultiresolutionTypeSD.AUTOMATIC;	
			}
			return null;
		}

	// ----------------------------------------------------------------------------

	/** Returns the brightness detection mode selected by the user. */
	public ESnakeTargetTypeSD getTargetBrightness() {
		switch (targetBrightnessComboBox_.getSelectedIndex()) {
		case 0:
			return ESnakeTargetTypeSD.DARK;
		case 1:
			return ESnakeTargetTypeSD.BRIGHT;
		}
		return null;
	}

	// ----------------------------------------------------------------------------

	/**
	 * Returns <code>true</code> if the user wants to ignore the maximum number
	 * of iterations, and evolve the snake till convergence. Returns
	 * <code>false</code> otherwise.
	 */
	public boolean isImmortal() {
		return immortalCheckbox_.isSelected();
	}

	// ----------------------------------------------------------------------------

	/** Sets the snake parameters to the interface. */
	public void setSnakeParameters(ESnakeParametersSD snakeParameters) {
		numControlPointsSpinner_.setValue(snakeParameters.getM());
		numSubdivLevelSpinner_.setValue(snakeParameters.getSubdivisionLevel());
		mScalingSpinner_.setValue(snakeParameters.getMScalingFactor());
		alphaSpinner_.setValue(snakeParameters.getAlpha());
		betaSpinner_.setValue(snakeParameters.getBeta());
		maxIterationsSpinner_.setValue(snakeParameters.getMaxLife());
		immortalCheckbox_.setSelected(snakeParameters.isImmortal());
		switch (snakeParameters.getEnergyType()) {
		case CONTOUR:
			energyTypeComboBox_.setSelectedIndex(0);
			break;
		case REGION:
			energyTypeComboBox_.setSelectedIndex(1);
			break;
		case MIXTURE:
			energyTypeComboBox_.setSelectedIndex(2);
			break;
		}
		switch (snakeParameters.getDetectType()) {
		case DARK:
			targetBrightnessComboBox_.setSelectedIndex(0);
			break;
		case BRIGHT:
			targetBrightnessComboBox_.setSelectedIndex(1);
			break;
		}
		switch (snakeParameters.getPriorShapeType()) {
		case NONE:
			priorShapeComboBox_.setSelectedIndex(0);
			break;
		case ROD:
			priorShapeComboBox_.setSelectedIndex(1);
			break;
		case BANANA:
			priorShapeComboBox_.setSelectedIndex(2);
			break;
		case BOOMERANG:
			priorShapeComboBox_.setSelectedIndex(3);
			break;
		case BRAINWHITE:
			priorShapeComboBox_.setSelectedIndex(4);
			break;
		case CIRCLE:
			priorShapeComboBox_.setSelectedIndex(5);
			break;
		case CORPUSCALLOSUM:
			priorShapeComboBox_.setSelectedIndex(6);
			break;
		case CROSS:
			priorShapeComboBox_.setSelectedIndex(7);
			break;
		case ELECTRICGUITAR:
			priorShapeComboBox_.setSelectedIndex(8);
			break;
		case FEMUR:
			priorShapeComboBox_.setSelectedIndex(9);
			break;
		case FLY:
			priorShapeComboBox_.setSelectedIndex(10);
			break;
		case MOUSEORGAN:
			priorShapeComboBox_.setSelectedIndex(11);
			break;
		case SEMICIRCLE:
			priorShapeComboBox_.setSelectedIndex(12);
			break;
		case SQUARE:
			priorShapeComboBox_.setSelectedIndex(13);
			break;
		case UPPERLIP:
			priorShapeComboBox_.setSelectedIndex(14);
			break;
		case CUSTOM:
			priorShapeComboBox_.setSelectedIndex(15);
			break;
		default:
			break;
		}
		switch (snakeParameters.getMultiresolutionType()) {
		case MANUAL:
			multiresolutionComboBox_.setSelectedIndex(0);
			break;
		case AUTOMATIC:
			multiresolutionComboBox_.setSelectedIndex(1);
			break;
		default:
			break;
		}
//		switch (snakeParameters.getShapeSpaceType()) {
//		case SIMILARITY:
//			shapeSpaceComboBox_.setSelectedIndex(0);
//			break;
//		case AFFINE:
//			shapeSpaceComboBox_.setSelectedIndex(1);
//			break;
//		default:
//			break;
//		}
		switch (snakeParameters.getBasisFunctionType()) {
		case QUADRATICBSPLINE:
			basisFunctionComboBox_.setSelectedIndex(0);
			break;
		case CUBICBSPLINE:
			basisFunctionComboBox_.setSelectedIndex(1);
			break;
		case ESPLINE3:
			basisFunctionComboBox_.setSelectedIndex(2);
			break;
		case DESLAURIERSDUBUC:
			basisFunctionComboBox_.setSelectedIndex(3);
			break;
		case GDESLAURIERSDUBUC:
			basisFunctionComboBox_.setSelectedIndex(4);
			break;
		case GDESLAURIERSDUBUC2:
			basisFunctionComboBox_.setSelectedIndex(5);
			break;
		default:
			break;
		}
	}

	// ============================================================================
	// PRIVATE METHODS

	private void updateNumControlPointsSpinner() {
		ESnakePriorShapeTypeSD shape = getPriorShapeType();
		switch (shape) {
		case NONE:
			break;
		case ROD:
			numControlPointsSpinner_.setValue(new Rod().getMinNumberNodes());
			break;
		case BANANA:
			numControlPointsSpinner_.setValue(new Banana().getMinNumberNodes());
			break;
		case BOOMERANG:
			numControlPointsSpinner_.setValue(new Boomerang()
					.getMinNumberNodes());
			break;
		case BRAINWHITE:
			numControlPointsSpinner_.setValue(new BrainWhite()
					.getMinNumberNodes());
			break;
		case CIRCLE:
			break;
		case CORPUSCALLOSUM:
			numControlPointsSpinner_.setValue(new CorpusCallosum()
					.getMinNumberNodes());
			break;
		case CROSS:
			numControlPointsSpinner_.setValue(new Cross().getMinNumberNodes());
			break;
		case ELECTRICGUITAR:
			numControlPointsSpinner_.setValue(new ElectricGuitar()
					.getMinNumberNodes());
			break;
		case FEMUR:
			numControlPointsSpinner_.setValue(new Femur().getMinNumberNodes());
			break;
		case FLY:
			numControlPointsSpinner_.setValue(new Fly().getMinNumberNodes());
			break;
		case MOUSEORGAN:
			numControlPointsSpinner_.setValue(new MouseOrgan()
					.getMinNumberNodes());
			break;
		case SEMICIRCLE:
			numControlPointsSpinner_.setValue(new Semicircle()
					.getMinNumberNodes());
			break;
		case SQUARE:
			numControlPointsSpinner_.setValue(new Square().getMinNumberNodes());
			break;
		case UPPERLIP:
			numControlPointsSpinner_.setValue(new UpperLip()
					.getMinNumberNodes());
			break;
		case CUSTOM:
			numControlPointsSpinner_.setValue(customPriorShape_
					.getMinNumberNodes());
			break;
		}

	}

	// ----------------------------------------------------------------------------
	// LISTENER METHODS

	/** Handles the events of the spinners. */
	@Override
	public void stateChanged(ChangeEvent e) {
		Object source = e.getSource();
		if (source == numControlPointsSpinner_ || source == alphaSpinner_
				|| source == betaSpinner_ || source == maxIterationsSpinner_ || source==numSubdivLevelSpinner_ || source==mScalingSpinner_) {
//			bigSnake_.loadSnakeParametersFromInterface();
		}
	}

	// ----------------------------------------------------------------------------

	/**
	 * Handles the events of the combo boxes and the checkbox. It fires an
	 * update of the active snake.
	 */
	@Override
	public void itemStateChanged(ItemEvent e) {
//		if (e.getSource() == targetBrightnessComboBox_) {
//			if (e.getStateChange() == ItemEvent.SELECTED) {
//				bigSnake_.loadSnakeParametersFromInterface();
//			}
//		} else if (e.getSource() == energyTypeComboBox_) {
//			if (e.getStateChange() == ItemEvent.SELECTED) {
//				alphaSpinner_.setEnabled(energyTypeComboBox_.getSelectedItem()
//						.toString().compareTo(energyTypeStrings_[2]) == 0);
//				bigSnake_.loadSnakeParametersFromInterface();
//			}
//		} else if (e.getSource() == priorShapeComboBox_) {
//			if (e.getStateChange() == ItemEvent.SELECTED) {
//				betaSpinner_
//						.setEnabled(priorShapeComboBox_.getSelectedIndex() != 0);
//				if (priorShapeComboBox_.getSelectedIndex() == 14) {
//					try {
//						customPriorShape_ = IOXMLSD.loadPriorShapeFromXML();
//					} catch (Exception e1) {
//						priorShapeComboBox_.setSelectedIndex(0);
//						betaSpinner_.setEnabled(false);
//						new AnnounceFrame("File not loaded.");
//						return;
//					}
//					if (customPriorShape_ != null) {
//						updateNumControlPointsSpinner();
//						bigSnake_.loadSnakeParametersFromInterface();
//					} else {
//						priorShapeComboBox_.setSelectedIndex(0);
//						betaSpinner_.setEnabled(false);
//						new AnnounceFrame("File not loaded.");
//						return;
//					}
//				} else {
//					updateNumControlPointsSpinner();
//					bigSnake_.loadSnakeParametersFromInterface();
//				}
//			}
//		} 
//		else if (e.getSource() == shapeSpaceComboBox_) {
//			if (e.getStateChange() == ItemEvent.SELECTED) {
//				bigSnake_.loadSnakeParametersFromInterface();
//			}			
//		}
//		else if (e.getSource() == multiresolutionComboBox_) {
//			if (e.getStateChange() == ItemEvent.SELECTED) {
//				bigSnake_.loadSnakeParametersFromInterface();
//			}
//		}
//		else if (e.getSource() == immortalCheckbox_) {
//			maxIterationsSpinner_.setEnabled(!immortalCheckbox_.isSelected());
//			bigSnake_.loadSnakeParametersFromInterface();
//		}
//		else if (e.getSource() == basisFunctionComboBox_) {
//			if (e.getStateChange() == ItemEvent.SELECTED) {
//				mScalingSpinner_.setEnabled(basisFunctionComboBox_.getSelectedItem()
//						.toString().compareTo(basisFunctionStrings_[0]) == 0 || basisFunctionComboBox_.getSelectedItem()
//						.toString().compareTo(basisFunctionStrings_[1]) == 0 );	
//				numControlPointsSpinner_.setEnabled(basisFunctionComboBox_.getSelectedItem()
//						.toString().compareTo(basisFunctionStrings_[0]) == 0 || basisFunctionComboBox_.getSelectedItem()
//						.toString().compareTo(basisFunctionStrings_[1]) == 0 || basisFunctionComboBox_.getSelectedItem()
//						.toString().compareTo(basisFunctionStrings_[2]) == 0 || basisFunctionComboBox_.getSelectedItem()
//						.toString().compareTo(basisFunctionStrings_[3]) == 0);
//				numSubdivLevelSpinner_.setEnabled(basisFunctionComboBox_.getSelectedItem()
//						.toString().compareTo(basisFunctionStrings_[0]) == 0 || basisFunctionComboBox_.getSelectedItem()
//						.toString().compareTo(basisFunctionStrings_[1]) == 0 || basisFunctionComboBox_.getSelectedItem()
//						.toString().compareTo(basisFunctionStrings_[2]) == 0 || basisFunctionComboBox_.getSelectedItem()
//						.toString().compareTo(basisFunctionStrings_[3]) == 0);
//				
//				if(basisFunctionComboBox_.getSelectedItem()
//						.toString().compareTo(basisFunctionStrings_[4]) == 0 || basisFunctionComboBox_.getSelectedItem()
//						.toString().compareTo(basisFunctionStrings_[5]) == 0){
//					numControlPointsSpinner_.setValue(4);
//					numSubdivLevelSpinner_.setValue(6);
//
//				}
//				
//				bigSnake_.loadSnakeParametersFromInterface();
//			}			
//		}
	}
}
