package plugins.big.bigsnake.jaccard;

import ij.ImagePlus;
import ij.gui.PolygonRoi;
import ij.gui.Roi;
import ij.io.Opener;
import ij.process.ByteProcessor;
import plugins.big.bigsnakeutils.icy.snake2D.Snake2DScale;


/**
 * This class handle the computation of the Jaccard index
 * @author Yann Perret (yann.perret@epfl.ch)
 * 2016-2017
 */
public class JaccardSnakes{
    
    //Groundtruth (mask)
    private double[][] mask = null;
    
    //Skin of the snake
    private Snake2DScale[] skin1 = null;
    
    
    public JaccardSnakes(String name, int init, Snake2DScale[] skin, double[][] mask) {
        this.skin1 = skin;
        this.mask = mask;
    }
    
    public double run() {
        double jacc = computeJaccard(skin1[0], mask);
        //		System.out.println("jaccard index = " + jacc);
        return jacc;
        
    }
    
    
    /**
     * Compute the jaccard index between a snake skin and a binary mask
     * @param skin1 skin of the snake
     * @param mask binary mask as 2D double image
     * @return jaccard index
     */
    public double computeJaccard(Snake2DScale skin1, double[][] mask) {
        double jaccard = 0.0;
        double and = 0.0;
        double or = 0.0;
        
        PolygonRoi roi1 = new PolygonRoi((skin1.xpoints).clone(),
                                         (skin1.ypoints).clone(), skin1.npoints, Roi.TRACED_ROI);
        if (roi1.getMask() == null /*|| roi2.getMask() == null*/) {
            return 0.0;
        }
        
        ByteProcessor bp1 = new ByteProcessor(mask.length, mask[0].length);
        bp1.insert(roi1.getMask(), roi1.getBounds().x, roi1.getBounds().y);
        
        String path="/Users/anais/Documents/workspaceAymeric/ssnakebig/test_images/ideal_emb1_z36_t22/components/membrane.png";
        double[][] boundaries = loadImage(path,mask.length, mask[0].length);
        
        //		//To display the binary mask of the snake
        //		Sequence sequence2 = new Sequence();
        //
        //		IcyBufferedImage tempImage2 = new IcyBufferedImage(mask.length, mask[0].length, 1, DataType.DOUBLE);
        //		double[] temp2 = tempImage2.getDataXYAsDouble(0);
        //
        //		for (int y = 0; y < mask[0].length; y++) {
        //			for (int x = 0; x < mask.length; x++) {
        //				temp2[x + y * mask.length] = bp1.getPixel(x, y);
        //			}
        //		}
        //
        //		tempImage2.dataChanged();
        //		sequence2.addImage(tempImage2);
        //
        //		Icy.getMainInterface().addSequence(sequence2);
        //		//
        
        //		//To display the binary mask of the snake
        //		Sequence sequence = new Sequence();
        //
        //		IcyBufferedImage tempImage = new IcyBufferedImage(mask.length, mask[0].length, 1, DataType.DOUBLE);
        //		double[] temp = tempImage.getDataXYAsDouble(0);
        //
        //		for (int y = 0; y < mask[0].length; y++) {
        //			for (int x = 0; x < mask.length; x++) {
        //				if(boundaries[x][y]==0){
        //				temp[x + y * mask.length] = bp1.getPixel(x, y);
        //				}
        //				else{
        //					temp[x + y * mask.length] =0.0;
        //				}
        //			}
        //		}
        //
        //		tempImage.dataChanged();
        //		sequence.addImage(tempImage);
        //
        //		Icy.getMainInterface().addSequence(sequence);
        //		//
        
        double a = 0.0, b = 0.0;
        for (int x = 0; x < mask.length; x++) {
            for (int y = 0; y < mask[0].length; y++) {
                a = bp1.getPixel(x, y);
                b = mask[x][y];
                
                if(boundaries[x][y]==0){//Pour le Jaccard on ne veut garder que l'intérieur de la cellule. Pas la membrane
                    if (a != 0 || b != 0) {
                        or += 1.0;
                    }
                    if (a != 0 && b != 0) {
                        and += 1.0;
                    }
                }
            }
        }
        
        jaccard = and / or;
        return jaccard;
    }
    
    public static double[][] loadImage(String filename, int width, int height){
        
        Opener opener = new Opener();
        
        double[][] imageMask = new double[width][height];
        
        //String filename=folder +"/"+ file.getName();
        
        ImagePlus image = opener.openImage(filename);
        imageMask= ImagePlus2Array2D(image);
        
        return imageMask;
    }
    
    /**
     * Convert an ImagePlus to a 2D array
     * @param imp image as ImagePlus
     * @return 2D array
     */
    public static double[][] ImagePlus2Array2D(ImagePlus imp) {
        
        int nx = imp.getWidth();
        int ny = imp.getHeight();
        
        double[][] output = new double[nx][ny];
        
        for(int y = 0; y < ny; y++) {
            for(int x = 0; x < nx; x++) {
                output[x][y] = imp.getProcessor().getPixelValue(x, y);
                
            }
        }
        
        return output;
    }
}
