package plugins.big.bigsnake.roi;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Path2D;
import java.util.ArrayList;
import java.util.List;

import icy.canvas.IcyCanvas;
import icy.painter.Anchor2D;
import plugins.big.bigsnake.keeper.SnakeKeeperSD;
import plugins.big.bigsnake.snake.ESnakeSD;
import plugins.big.bigsnake.snake.GlobalSnakeSD;
import plugins.big.bigsnake.snake.SubsnakeSD;
import plugins.big.bigsnakeutils.icy.snake2D.Snake2DNode;
import plugins.big.bigsnakeutils.icy.snake2D.Snake2DScale;

/**
 * Class that adapts certain aspects of the ESnakeSD interaction/display
 * to be adapted to GlobalSnake (marker sizes, colors, etc...).
 * 
 * @version January 13, 2018
 * 
 * @author Aymeric Galan (aymeric.galan@gmail.com)
 */
public class ROI2DGlobalSnakeSD extends ROI2DSnakeSD {
	
	/** GlobalSnake associated to the ROI. */
	private GlobalSnakeSD globalSnake_ = null;
	
	private List<Snake2DScale[]> subsnakeScales_ = null;

	/** Number of Subsnakes in that GlobalSnake */
	private int nSubsnakes_ = 1;
	
	/**
	 * List of groups of all polylines 
	 * forming the skin of all Subsnakes.
	 */
	private ArrayList<Snake2DScale[]> allScales_ = null;
	
	
	// ===============
	// PUBLIC METHODS
	
	/** Default constructor. */
	public ROI2DGlobalSnakeSD() {
		super();
		setColorsAndSizes();
	}
	
	
	/** Constructor. */
	public ROI2DGlobalSnakeSD(ESnakeSD snake, SnakeKeeperSD keeper) {
		super(snake, keeper);
		
		setColorsAndSizes();
		
		globalSnake_ = (GlobalSnakeSD) snake;  // same than snake_, but as a GlobalSnake
		nSubsnakes_ = globalSnake_.getNumSubsnakes();
		
		subsnakeScales_ = new ArrayList<Snake2DScale[]>();
		for (int i = 0 ; i < nSubsnakes_ ; i++) {
			SubsnakeSD subsnake = globalSnake_.getSubsnake(i);
			subsnakeScales_.add(subsnake.getScales());
		}
	}
	
	
	// ----------------------------------------------------------------------------
	
	
	@Override
	public void refreshScales() {
		super.refreshScales();
		refreshSubsnakeScales();
	}


	@Override
	public void changePosition(double[] xPosition, double[] yPosition,
			Snake2DScale[] scales) {
		
		super.changePosition(xPosition, yPosition, scales);
		
		refreshSubsnakeScales();
	}

	
	/** Draw the smooth red curve representing the snake skin */
	@Override
	protected void drawScales(Graphics2D g2, IcyCanvas canvas) {
		
		if (subsnakeScales_ != null) {
			
			double stroke = getAdjustedStroke(canvas);
			g2.setStroke(new BasicStroke((float) (smoothSkinStrokeWidth_ * stroke)));
			
			for (int id = 0 ; id < nSubsnakes_ ; id++) {
				
				Snake2DScale[] scales = subsnakeScales_.get(id);
				Snake2DScale smoothCurve = scales[0];
				
				g2.setColor(smoothCurve.getColor());
				g2.draw(smoothCurve);

			}
			
		}
	}
	
	
	/** Draw the IDs of the snake close to the contour. */
	@Override
	protected void drawSnakeIds(Graphics2D g) {
		// do not draw IDs for a globalSnake
	}
	
	
	private void refreshSubsnakeScales() {
		for (int i = 0 ; i < nSubsnakes_ ; i++) {
			SubsnakeSD subsnake = globalSnake_.getSubsnake(i);
			subsnakeScales_.set(i, subsnake.getScales());
		}
	}
	
	
	/** Subsnake visual parameters */
	private void setColorsAndSizes() {
		DEFAULT_NORMAL_COLOR = Color.RED;
		DEFAULT_SUBDIVISION_NORMAL_COLOR = new Color(30, 144, 255, 255);  // light blue
		smoothSkinStrokeWidth_ = 0.5;
		anchorCrossSize_ = 2. / 3;
		subdivAnchorCrossSize_ = 1. / 4;
	}
	
}
