package plugins.big.bigsnake.roi;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;

import icy.canvas.IcyCanvas;
import plugins.big.bigsnake.keeper.SnakeKeeperSD;
import plugins.big.bigsnake.snake.ESnakeSD;
import plugins.big.bigsnake.snake.GlobalSnakeSD;
import plugins.big.bigsnake.snake.SubsnakeSD;

/**
 * Class that adapts certain aspects of the ESnakeSD interaction/display
 * to be adapted to Subsnake (marker sizes, colors, etc...).
 * 
 * @version January 13, 2018
 * 
 * @author Aymeric Galan (aymeric.galan@gmail.com)
 */
public class ROI2DSubsnakeCurveSD extends ROI2DSnakeSD {

	/** Default constructor. */
	public ROI2DSubsnakeCurveSD() {
		super();
		setColorsAndSizes();
		hideHiddenNodes_ = false;
		useSkinForSubdivMarkers_ = true;
	}
	
	
	/** Constructor. */
	public ROI2DSubsnakeCurveSD(ESnakeSD snake, SnakeKeeperSD keeper) {
		super(snake, keeper);
		setColorsAndSizes();
		hideHiddenNodes_ = false;
		useSkinForSubdivMarkers_ = true;
	}

	
	/** draw (i.e. display) the IDs of the snake close to the contour. */
	@Override
	protected void drawSnakeIds(Graphics2D g) {
		if (keeper_ != null) {
			// (font name, style, size)
			Font fontPlain = new Font("Helvetica", Font.PLAIN, 10);
			Font fontBold = new Font("Helvetica", Font.BOLD, 14);
			
			String keeperId = keeper_.getID();
			int subsnakeId = ((SubsnakeSD) keeper_.snake_).getId();
			
			String textKeeper = "";
			if (keeperId != null) {
				textKeeper = String.format(" (%s)", keeperId);
			}
			String textSubsnake = String.format("%d", subsnakeId);
			
			// all the following stuff to align properly text strings
			FontMetrics metricsB = g.getFontMetrics(fontBold);
//			FontMetrics metricsP = g.getFontMetrics(fontPlain);
			
			float centroidX = (float) keeper_.snake_.getCentroid().getX();
			float centroidY = (float) keeper_.snake_.getCentroid().getY();
			
			// will be left aligned (default)
			float textKeeperPosX = centroidX;
			// will centered vertically
			float textKeeperPosY = centroidY;
			
			float textSubsnakePosX;
			if (keeperId != null) {
				// will be right aligned
				textSubsnakePosX = centroidX - metricsB.stringWidth(textSubsnake);
			} else {
				// will be centered
				textSubsnakePosX = centroidX - metricsB.stringWidth(textSubsnake) / 2;
			}
			float textSubsnakePosY = centroidY;
			
			// visually better to add a small constant offset
			boolean applyOffset = true;
			if (applyOffset) {
				float offsetX = 5;
				textKeeperPosX -= offsetX;
				textSubsnakePosX -= offsetX;
			}
			
			// draw both texts
			g.setFont(fontBold);
			g.drawString(textSubsnake, textSubsnakePosX, textSubsnakePosY);
			g.setFont(fontPlain);
			g.drawString(textKeeper, textKeeperPosX, textKeeperPosY);
		}
	}
	
	
	/** Subsnake visual parameters */
	private void setColorsAndSizes() {
		DEFAULT_NORMAL_COLOR = Color.RED;
		DEFAULT_SUBDIVISION_NORMAL_COLOR = Color.LIGHT_GRAY;
		smoothSkinStrokeWidth_ = 1.;
		anchorCrossSize_ = 1. / 2;
		subdivAnchorCrossSize_ = 1. / 3;
	}
	
}
