/*******************************************************************************
 * Copyright (c) 2012-2013 Biomedical Image Group (BIG), EPFL, Switzerland.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Ricard Delgado-Gonzalo (ricard.delgado@gmail.com)
 *     Nicolas Chenouard (nicolas.chenouard@gmail.com)
 *     Philippe Th&#233;venaz (philippe.thevenaz@epfl.ch)
 *     Emrah Bostan (emrah.bostan@gmail.com)
 *     Ulugbek S. Kamilov (kamilov@gmail.com)
 *     Ramtin Madani (ramtin_madani@yahoo.com)
 *     Masih Nilchian (masih_n85@yahoo.com)
 *     C&#233;dric Vonesch (cedric.vonesch@epfl.ch)
 *     Virginie Uhlmann (virginie.uhlmann@epfl.ch)
 *     Cl&#233;ment Marti (clement.marti@epfl.ch)
 *     Julien Jacquemot (julien.jacquemot@epfl.ch)
 *     Daniel Schmitter (daniel.schmitter@epfl.ch)
 *     Ana�s Badoual (anais.badoual@gmail.com)
 ******************************************************************************/
package plugins.big.bigsnake.snake;

import icy.gui.frame.progress.AnnounceFrame;
import icy.imagej.ImageJUtil;
import icy.main.Icy;
import icy.sequence.Sequence;
import icy.util.XMLUtil;
import ij.ImagePlus;
import ij.gui.PolygonRoi;
import ij.gui.Roi;
import ij.process.ByteProcessor;

import java.awt.Color;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.w3c.dom.Element;

import plugins.big.bigsnake.core.ImageLUTContainerSD;
import plugins.big.bigsnake.io.IOXMLSD;
import plugins.big.bigsnake.snake.Enums.ESnakeEnergyTypeSD;
import plugins.big.bigsnake.snake.Enums.ESnakeOrientationSD;
import plugins.big.bigsnake.snake.Enums.ESnakePriorShapeTypeSD;
import plugins.big.bigsnake.snake.Enums.ESnakeTargetTypeSD;
import plugins.big.bigsnake.snake.Enums.MultiresolutionTypeSD;
import plugins.big.bigsnake.snake.Enums.ShapeSpaceTypeSD;
import plugins.big.bigsnakeutils.icy.snake2D.LinkCurveControlPoints;
import plugins.big.bigsnakeutils.icy.snake2D.Snake2D;
import plugins.big.bigsnakeutils.icy.snake2D.Snake2DNode;
import plugins.big.bigsnakeutils.icy.snake2D.Snake2DScale;
import plugins.big.bigsnakeutils.process.process1D.BSplineBasis;
import plugins.big.bigsnakeutils.process.process1D.BSplineBasis.BSplineBasisType;
import plugins.big.bigsnakeutils.process.process1D.Convolver1D;
import plugins.big.bigsnakeutils.process.process1D.Filters;
import plugins.big.bigsnakeutils.shape.priorshapes.ShapeProjector;
import plugins.big.bigsnakeutils.shape.priorshapes.ShapeProjectorBuilder;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Banana;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Boomerang;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.BrainWhite;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Circle;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.CorpusCallosum;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Cross;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Custom;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.ElectricGuitar;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Femur;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Fly;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.MouseOrgan;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.PriorShape;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Rod;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Semicircle;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.Square;
import plugins.big.bigsnakeutils.shape.priorshapes.shapes.UpperLip;
import plugins.big.bigsnakeutils.shape.utils.Geometry2D;
import plugins.kernel.roi.roi2d.ROI2DEllipse;
import plugins.kernel.roi.roi2d.ROI2DPolygon;
import plugins.kernel.roi.roi2d.ROI2DRectangle;

/**
 * Exponential spline snake (E-Snake).
 * 
 * @version January 13, 2018
 * 
 * @author Ricard Delgado-Gonzalo (ricard.delgado@gmail.com)
 * @author Ramtin Madani (ramtin_madani@yahoo.com)
 * @author Masih Nilchian (masih_n85@yahoo.com)
 * @author Daniel Schmitter (daniel.schmitter@epfl.ch)
 * @author Ana�s Badoual (anais.badoual@gmail.com)
 * @author Aymeric Galan (aymeric.galan@gmail.com)
 */
public class ESnakeSD implements Snake2D {

	/** Snake defining nodes. */
	protected Snake2DNode[] coef_ = null;

	/** Snake subdivision points. */
	protected Snake2DNode[] subdivCoef_ = null; // 'protected' may be temporary

	/** Snake derivative subdivision points. COOL */
	protected Snake2DNode[] dersubdivCoef_ = null;

	// ----------------------------------------------------------------------------
	// SNAKE CONTOUR

	/** Samples of the x coordinates of the snake contour. */
	protected double[] xSnakeContour_ = null;
	/** Samples of the y coordinates of the snake contour. */
	protected double[] ySnakeContour_ = null;

	/**
	 * Samples of the x coordinates of the snake contour obtained by
	 * subdivision.
	 */
	protected double[] xSnakeContourSubdivision_ = null;
	/**
	 * Samples of the y coordinates of the snake contour obtained by
	 * subdivision.
	 */
	protected double[] ySnakeContourSubdivision_ = null;

	/** Samples of the x coordinates of the derivative of the snake contour. */
	protected double[] xSnakeTangentVector_ = null;
	/** Samples of the y coordinates of the derivative of the snake contour. */
	protected double[] ySnakeTangentVector_ = null;

	/**
	 * Samples of the x coordinates of the derivative of the snake contour
	 * obtained by subdivision.
	 */
	protected double[] xSnakeTangentVectorSubdivision_ = null;
	/**
	 * Samples of the y coordinates of the derivative of the snake contour
	 * obtained by subdivision.
	 */
	protected double[] ySnakeTangentVectorSubdivision_ = null;

	/** Bounding box that encloses the outline of the snake. */
	protected Rectangle snakeContourBoundingBox_ = null;

	/**
	 * Samples of the x coordinates of the shell that surrounds the snake
	 * contour.
	 */
	private double[] xSnakeShellContour_ = null;
	/**
	 * Samples of the y coordinates of the shell that surrounds the snake
	 * contour.
	 */
	private double[] ySnakeShellContour_ = null;

	/** Bounding box that encloses the outline of the shell of the snake. */
	protected Rectangle snakeShellBoundingBox_ = null;

	/** Number of samples at which each segment of the contour is discretized. */
	protected final int nSamplesPerSegment_ = 50;

	// ----------------------------------------------------------------------------
	// SNAKE STATUS FIELDS

	/**
	 * If <code>true</code> indicates that the snake is able to keep being
	 * optimized.
	 */
	protected boolean alive_ = true;
	/** Signed area of the region enclosed by the snake. */
	protected double area_ = 0;
	/**
	 * If <code>true</code>, the snake has gone through the initialization
	 * process.
	 */
	protected boolean isInitialized_ = false;
	/**
	 * Number of iterations left when <code>immortal_</code> is
	 * <code>false</code>.
	 */
	protected int life_ = 0;
	/**
	 * If <code>false</code> indicates that the snake takes an ill conditioned
	 * shape.
	 */
	private boolean valid_ = true;

	// ----------------------------------------------------------------------------
	// SNAKE OPTION FIELDS

	/** Indicates the type of features to detect (bright or dark). **/
	protected ESnakeTargetTypeSD detectType_ = null;
	/** Number of spline vector coefficients. */
	protected int M_ = 0;
	/** Number of subdivision coefficients. */
	protected int MSub_ = 0;
	/** Number of subdivision level. */
	protected int Level_ = 0;
	/** m scaling factor. */
	protected int mScaling_ = 0;
	/** Snake energy used during the optimization process. */
	protected ESnakeEnergyTypeSD energyType_ = null;
	/** Energy trade-off factor. */
	private double alpha_ = 0;
	/** Prior-shape information. */
	private ESnakePriorShapeTypeSD priorShapeType_ = null;
	/** Shape space information. */
	private ShapeSpaceTypeSD shapeSpaceType_ = null;
	/** Basis function information. */
	protected BSplineBasisType basisFunctionType_ = null;
	/** Weight contribution of the prior-shape energy to the total energy. */
	private double beta_ = 0;
	/** Multiresolution information. */
	private MultiresolutionTypeSD multiresolutionType_ = null;
	/**
	 * Maximum number of iterations allowed when the <code>immortal_</code> is
	 * <code>false</code>.
	 */
	protected int maxLife_ = 0;
	/**
	 * If <code>true</code> indicates that the snake will keep iterating till
	 * the optimizer decides so.
	 */
	protected boolean immortal_ = true;

	/** Initial contour. */
	private Polygon initialContour_ = null;

	// ----------------------------------------------------------------------------
	// SPLINE LUTS

	/** Basis function of the curve. */
	//private static final BSplineBasis.BSplineBasisType BASIS_FUNCTION =BSplineBasisType.ESPLINE3;
	protected BSplineBasis.BSplineBasisType BASIS_FUNCTION = null;

	protected double[] bSplineLUT_ = null;
	/** LUT with the samples of the derivative of the basis function. */
	protected double[] bSplineDerivativeLUT_ = null;
	/** LUT with the samples of the autocorrelation function. */
	private double[] bSplineAutocorrelationLUT_ = null;
	/**
	 * LUT with the values of
	 * $Q(i,j)=\int_{0}^{M}\,\phi_M(t-j)\,\phi'_M(t-i)\,\mathrm{d}t$.
	 */
	protected double[][] qLUT_ = null;

	// ----------------------------------------------------------------------------
	// IMAGE FIELDS

	/** Container of the LUTs. */
	protected ImageLUTContainerSD imageLUTs_ = null;

	// ----------------------------------------------------------------------------
	// SELF-INTERSECTION FIELDS

	/** The intersected node in the polygon. */
	private int badNodeIndex_ = 0;
	/** The intersected edge of the polygon. */
	private int badEdgeIndex_ = 0;
	/** Describing the self intersection of the snake polygon. */
	private boolean controlPolygonSelfIntersects_ = false;
	/** Condition of the snake polygon edges with respect to each other. */
	private int[][] edgeVsEdge_ = null;

	// ----------------------------------------------------------------------------
	// PRIOR-SHAPE FIELDS

	/** Encodes the shape-prior that the snake uses within its energy. */
	private PriorShape priorShape_ = null;
	/** Shape-projectors factory. */
	private ShapeProjectorBuilder shapeProjectorBuilder_ = null;
	/** Projector that projects a particular spline curve to a shape space. */
	private ShapeProjector shapeProjectorAffine_ = null;
	private ShapeProjector shapeProjectorSimilarity_ = null;
	private Custom customPriorShape_ = null;

	// ----------------------------------------------------------------------------
	// DISPLAY

	/** Color of the scale that represents the contour of the snake. */
	protected Color SNAKE_CONTOUR_COLOR = new Color(255, 0, 0, 255);
	/** Color of the scale that represents the control polygon of the snake. */
	protected Color CONTROL_POLYGON_COLOR = new Color(255, 255, 0, 128);

	// ----------------------------------------------------------------------------
	// AUXILIARY FIELDS AND LUTS

	/** PI/M. */
	protected double PIM_ = 0;
	/** 2*PI/M. */
	protected double PI2M_ = 0;
	/**
	 * Support of the basis function multiplied by
	 * <code>nSamplesPerSegment_</code>.
	 */
	protected int NR_ = 0;
	/** Number of control points multiplied by <code>nSamplesPerSegment_</code>. */
	protected int MR_ = 0;

	/** LUT with samples of one period of a sine. */
	private double[] sinLUT_ = null;
	/** LUT with samples of one period of a cosine. */
	private double[] cosLUT_ = null;

	// ----------------------------------------------------------------------------
	// CONSTANTS

	/** Square root of two. */
	private static final double SQRT2 = Math.sqrt(2.0);

	// ----------------------------------------------------------------------------
	// XML TAGS

	/**
	 * Label of the XML tag containing the list of snake-defining control
	 * points.
	 */
	public static final String ID_CONTROL_POINTS = "control_points";
	/**
	 * Label of the XML tag containing the a single snake-defining control
	 * point.
	 */
	public static final String ID_CONTROL_POINT = "control_point";

	// ============================================================================
	// PUBLIC METHODS

	/** Constructor. */
	public ESnakeSD(ImageLUTContainerSD imageLUTs,
			ESnakeParametersSD parameters, Object initialContour,
			Custom customPriorShape) {
		
		setCustomPriorShape(customPriorShape);

		if (imageLUTs == null) {
			System.err.println("Image not properly loaded.");
			return;
		}
		imageLUTs_ = imageLUTs;
		if (parameters == null) {
			System.err.println("E-Snake parameters not properly loaded.");
			return;
		}
		M_ = parameters.getM();
		if (M_ < 3) {
			System.err
					.println("The minimum number of knots for this basis function is three.");
			return;
		}

		Level_ = parameters.getSubdivisionLevel();
		mScaling_ = parameters.getMScalingFactor();

		MSub_ = (int) Math.pow(mScaling_, Level_) * M_;

		shapeProjectorBuilder_ = new ShapeProjectorBuilder(M_);

		if (initialContour instanceof Polygon) {
			initialContour_ = (Polygon) initialContour;
		} else if (initialContour instanceof ROI2DPolygon) {
			initialContour_ = ((ROI2DPolygon) initialContour).getPolygon();
		} else if (initialContour instanceof ROI2DEllipse) {
			double x = ((ROI2DEllipse) initialContour).getEllipse()
					.getCenterX();
			double y = ((ROI2DEllipse) initialContour).getEllipse()
					.getCenterY();
			double h = ((ROI2DEllipse) initialContour).getEllipse().getHeight();
			double w = ((ROI2DEllipse) initialContour).getEllipse().getWidth();

			Polygon p = new Polygon();
			for (int i = 0; i < 100; i++) {
				p.addPoint((int) (x + w * Math.cos(2.0 * Math.PI * i / 100.0)
						/ 2.0),
						(int) (y + h * Math.sin(2.0 * Math.PI * i / 100.0)
								/ 2.0));
			}
			initialContour_ = p;
		} else if (initialContour instanceof ROI2DRectangle) {
			Polygon p = new Polygon();
			p.addPoint((int) ((ROI2DRectangle) initialContour).getBounds()
					.getMinX(), (int) ((ROI2DRectangle) initialContour)
					.getBounds().getMinY());
			p.addPoint((int) ((ROI2DRectangle) initialContour).getBounds()
					.getMinX(), (int) ((ROI2DRectangle) initialContour)
					.getBounds().getMaxY());
			p.addPoint((int) ((ROI2DRectangle) initialContour).getBounds()
					.getMaxX(), (int) ((ROI2DRectangle) initialContour)
					.getBounds().getMaxY());
			p.addPoint((int) ((ROI2DRectangle) initialContour).getBounds()
					.getMaxX(), (int) ((ROI2DRectangle) initialContour)
					.getBounds().getMinY());
		} else {
			initialContour_ = null;
		}

		maxLife_ = parameters.getMaxLife();
		alpha_ = parameters.getAlpha();
		beta_ = parameters.getBeta();
		immortal_ = parameters.isImmortal();
		energyType_ = parameters.getEnergyType();
		detectType_ = parameters.getDetectType();
		shapeSpaceType_ = parameters.getShapeSpaceType();
		
		basisFunctionType_=parameters.getBasisFunctionType();
		BASIS_FUNCTION=basisFunctionType_;
		
		multiresolutionType_=parameters.getMultiresolutionType();
		
		setPriorShape(parameters.getPriorShapeType());
		initialize(true);
	}
	

	// ----------------------------------------------------------------------------
	// SNAKE2D METHODS

	/**
	 * The purpose of this method is to compute the energy of the snake. This
	 * energy is usually made of three additive terms: 1) the image energy,
	 * which gives the driving force associated to the data; 2) the internal
	 * energy, which favors smoothness of the snake; and 3) the constraint
	 * energy, which incorporates a priori knowledge. This method is called
	 * repeatedly during the optimization of the snake, but only as long as the
	 * method <code>isAlive()</code> returns <code>true</code>. It is imperative
	 * that this function be everywhere differentiable with respect to the
	 * snake-defining nodes.
	 */
	@Override
	public double energy() {
		
		// true to deactivate prior and self-intersection energies
		final boolean simpleEnergy = true;
		
		// debug :
		final boolean verboseEnergy = false;

		if (!immortal_) {
			life_--;
			
			if (life_ == 0) {
				alive_ = false;
			}
		}

		if (!isSnakeInBoundingBox()) {
			return Double.MAX_VALUE; // 100000;//Double.MAX_VALUE*;
		}
		
		double energyRegularization = 0; // temp.
		energyRegularization = Math.pow(computeSelfIntersectionEnergy(), 6);
		if (verboseEnergy) System.out.println("E_regul = " + Math.pow(computeSelfIntersectionEnergy(), 6));

		double E = 0;
		switch (energyType_) {
		case CONTOUR:
			E = computeContourEnergy();
			break;
		case RIDGE:
			E = computeRidgeEnergy();
			break;
		case REGION:
			E = computeRegionEnergy();
			break;
		case MIXTURE: {
			E = alpha_ * computeContourEnergy() + (1 - alpha_)
					* computeRegionEnergy();
		}
			break;
		}
		if (verboseEnergy) System.out.println("E = " + E);

		double priorShape = 0;
		if (!(priorShapeType_ == null || priorShapeType_ == ESnakePriorShapeTypeSD.NONE)) {
			priorShape = beta_ * computePriorShapeEnergy();
		}
		if (verboseEnergy) System.out.println("E_prior = " + priorShape);

		
		if (simpleEnergy) {
			energyRegularization = 0;
			priorShape = 0;
		}
		if (detectType_ == ESnakeTargetTypeSD.DARK) {
			return -E + energyRegularization + priorShape;
		} else if (detectType_ == ESnakeTargetTypeSD.BRIGHT) {
			return E + energyRegularization + priorShape;
		} else {
			return Double.MAX_VALUE;
		}
	}

	// ----------------------------------------------------------------------------

	/**
	 * Returns a point with the position of the center of gravity of the scales.
	 */
	@Override
	public Point2D.Double getCentroid() {
		Point2D.Double centroid = new Point2D.Double();
		for (Snake2DNode element : coef_) {
			centroid.x += element.x;
			centroid.y += element.y;
		}
		centroid.x /= coef_.length;
		centroid.y /= coef_.length;
		return centroid;
	}

	// ----------------------------------------------------------------------------

	/**
	 * Returns a point with the position of the center of gravity of the scales
	 * obtained with subdivision.
	 */
	public Point2D.Double getCentroidSubdivision() {
		Point2D.Double centroid = new Point2D.Double();
		for (Snake2DNode element : subdivCoef_) {
			centroid.x += element.x;
			centroid.y += element.y;
		}
		centroid.x /= subdivCoef_.length;
		centroid.y /= subdivCoef_.length;
		return centroid;
	}

	// ----------------------------------------------------------------------------
	/**
	 * The purpose of this method is to compute the gradient of the snake energy
	 * with respect to the snake-defining nodes.
	 */
	@Override
	public Point2D.Double[] getEnergyGradient() {
		return null;
	}

	// ----------------------------------------------------------------------------

	/** This method provides an accessor to the snake-defining nodes. */
	@Override
	public Snake2DNode[] getNodes() {
		return coef_;
	}

	// ----------------------------------------------------------------------------

	/** This method returns the quantity of snake-defining nodes. */
	@Override
	public int getNumNodes() {
		return coef_.length;
	}

	// ----------------------------------------------------------------------------

	/** This method returns the number of subdivision. */
	public int getLevelSubdivision() {
		return Level_;
	}
	
	// ----------------------------------------------------------------------------

	/** This method provides an accessor to the snake scaling factor. */
	public int getScalingFactor() {
		 return mScaling_;
	}
	
	// ----------------------------------------------------------------------------

	/** This method uploads the number of subdivision. */
	public void setLevelSubdivision(int level) {
		 Level_=level;
	}

	// ----------------------------------------------------------------------------

	/** This method provides an accessor to the snake subdivision point. */
	public Snake2DNode[] getSubdivisionPoints() {
		return subdivCoef_;
	}

	// ----------------------------------------------------------------------------

	/**
	 * The purpose of this method is to determine what to draw on screen, given
	 * the current configuration of nodes. Collectively, the array of scales
	 * forms the skin of the snake.
	 */
	@Override
	public Snake2DScale[] getScales() {
		int numScales = getNumScales();
		Snake2DScale[] skin = new Snake2DScale[numScales];

		double[] xpoints = new double[M_];
		double[] ypoints = new double[M_];
		for (int k = 0; k < M_; k++) {
			xpoints[k] = coef_[k].x;
			ypoints[k] = coef_[k].y;
		}
//		skin[0] = new Snake2DScale(xSnakeContour_, ySnakeContour_, MR_,
//				SNAKE_CONTOUR_COLOR, true);
		skin[0] = new Snake2DScale(xSnakeContourSubdivision_, ySnakeContourSubdivision_, 
				xSnakeContourSubdivision_.length, SNAKE_CONTOUR_COLOR, true);
		
		skin[1] = new Snake2DScale(xpoints, ypoints, M_, CONTROL_POLYGON_COLOR,
				true);

		return skin;
	}
	
	// ----------------------------------------------------------------------------
	
	public Snake2DNode[] getSmoothSkin() {
		Snake2DNode[] smoothSkin = new Snake2DNode[MSub_];
		for (int i = 0 ; i < MSub_ ; i++) {
			smoothSkin[i] = new Snake2DNode(
					xSnakeContourSubdivision_[i], 
					ySnakeContourSubdivision_[i]);
		}
		return smoothSkin;
	}

	// ----------------------------------------------------------------------------

	@Override
	public void initialize(boolean initShape) {

		switch (BASIS_FUNCTION) {
		case ESPLINE3:
			NR_ = BSplineBasis.ESPLINE3SUPPORT * nSamplesPerSegment_;
			break;
		case ESPLINE4:
			NR_ = BSplineBasis.ESPLINE4SUPPORT * nSamplesPerSegment_;
			break;
		case LINEARBSPLINE:
			NR_ = BSplineBasis.LINEARBSPLINESUPPORT * nSamplesPerSegment_;
			break;
		case QUADRATICBSPLINE:
			NR_ = BSplineBasis.QUADRATICBSPLINESUPPORT * nSamplesPerSegment_;
			break;
		case CUBICBSPLINE:
			NR_ = BSplineBasis.CUBICBSPLINESUPPORT * nSamplesPerSegment_;
			break;
		case MSPLINE:
			NR_ = BSplineBasis.MSPLINESUPPORT * nSamplesPerSegment_;
			break;
		case DESLAURIERSDUBUC:
			NR_ = BSplineBasis.QUADRATICBSPLINESUPPORT * nSamplesPerSegment_;
			break;
		case GDESLAURIERSDUBUC:
			NR_ = BSplineBasis.ESPLINE3SUPPORT * nSamplesPerSegment_;
			break;
		case GDESLAURIERSDUBUC2:
			NR_ = BSplineBasis.ESPLINE3SUPPORT * nSamplesPerSegment_;
			break;
		}
		
		MR_ = M_ * nSamplesPerSegment_;
		PIM_ = Math.PI / M_;
		PI2M_ = 2 * PIM_;

		MSub_ = (int) Math.pow(mScaling_, Level_) * M_;
		
		initializeContoursAndTangents();

		qLUT_ = new double[M_][M_];

		snakeContourBoundingBox_ = new Rectangle();
		snakeShellBoundingBox_ = new Rectangle();
		
		life_ = maxLife_;
		buildLUTs();
		if (initShape) {
			initializeDefaultShape();
		}
		updateSnakeSkin();
		updateAreaSubdivision();

		if (energyType_ == ESnakeEnergyTypeSD.REGION
				|| energyType_ == ESnakeEnergyTypeSD.MIXTURE) {
			updateSnakeShell();
		}
		isInitialized_ = true;
	}

	// ----------------------------------------------------------------------------

	/**
	 * The purpose of this method is to compute the subdivision points.
	 */
	// example to compute the contour using subdivision points for a m scaling
	// refinement and a number of subdivision iteration equal to Level_
	public void subdivision() {
		Snake2DNode[] pointsToSubdivide;
		pointsToSubdivide = coef_;
		int order;

		if (BASIS_FUNCTION == BSplineBasisType.LINEARBSPLINE)
			order = 2;
		else if (BASIS_FUNCTION == BSplineBasisType.QUADRATICBSPLINE)
			order = 3;
		else if (BASIS_FUNCTION == BSplineBasisType.CUBICBSPLINE)
			order = 4;
		else {
			System.out
					.println("The order of the basis function has to be between 2 and 4");
			order = 4; // Default value
		}
		
		//Computation of the subdivision points
		for (int i = 0; i < Level_; i++) {
			computeSubdivPoints(mScaling_, pointsToSubdivide, order);
			pointsToSubdivide = subdivCoef_;

//			 System.out.println("Subdivision points at iteration " + (i+1));
//			 for (int k=0; k<subdivCoef_.length; k++){
//			 System.out.println("i="+k+"subdivCoefX[k]"+ subdivCoef_[k].x +
//			 "subdivCoefY[k] "+ subdivCoef_[k].y); } 
		}

		
		if(Level_==0) {
			subdivCoef_ = new Snake2DNode[MSub_];
			
			for(int i=0; i<MSub_; i++){
				subdivCoef_[i]=new Snake2DNode(coef_[i].x, coef_[i].y);
			}
		}
		
		// Computation of the tangents at each subdivision points
		computeDerivativeSubdivPoints(order);
		
		// We interpole the subdivision points

		// Convolution by the post refinement filter to put points on the curve
		double[] B;
		//int nb = (order - 1) / 2; // integer part
		//int Bsize = 2 * nb;
		B = new double[order];

		int nbSubdivPoints;
		nbSubdivPoints = subdivCoef_.length;

		double[] subdivCoefX, subdivCoefY;
		subdivCoefX = new double[nbSubdivPoints];
		subdivCoefY = new double[nbSubdivPoints];

		for (int i = 0; i < nbSubdivPoints; i++) {
			subdivCoefX[i] = subdivCoef_[i].x;
			subdivCoefY[i] = subdivCoef_[i].y;
		}

		for (int i = 0; i < B.length; i++) B[i] = 0;

		B = BSplineBasis.computeBSplineFilter(order - 1);
		/*for (int i = 0; i < B.length; i++){
			//B[i] = BSplineBasis.CubicBSpline(i);//Autre solution
			System.out.println("B[i]="+B[i]);
		}*/

		double[] subdivCoefX2, subdivCoefY2;
		subdivCoefX2 = new double[nbSubdivPoints];
		subdivCoefY2 = new double[nbSubdivPoints];

		subdivCoefX2 = Convolver1D.subdivConvolve(subdivCoefX, subdivCoefX2, B,
				nbSubdivPoints);
		subdivCoefY2 = Convolver1D.subdivConvolve(subdivCoefY, subdivCoefY2, B,
				nbSubdivPoints);

		for (int i = 0; i < nbSubdivPoints; i++) {
			subdivCoef_[i] = new Snake2DNode(subdivCoefX2[i], subdivCoefY2[i]);
		}
		
//		System.out.println("Interpolated subdivision points"); for (int i=0;
//		 i<nbSubdivPoints; i++){ System.out.println("i="+i+"subdivCoefX[i]"+
//		 subdivCoef_[i].x + "subdivCoefY[i] "+ subdivCoef_[i].y); }
	}
	
	// ----------------------------------------------------------------------------

		/**
		 * The purpose of this method is to compute the subdivision points for a non stationary scheme only for ESpline3.
		 */
		// example to compute the contour using subdivision points for a m scaling
		// refinement and a number of subdivision iteration equal to Level_
		public void nonStationnarySubdivision() {
			Snake2DNode[] pointsToSubdivide;
			
			pointsToSubdivide = coef_;
			int order=3;
			double alphaSub=PI2M_;

			// System.out.println("Control points");

			/*
			 * if(life_ < 400) for(int i=0; i<M_; i++){
			 * System.out.println("coefx_[i]="+coef_[i].x+"coef_[i]="+coef_[i].y); }
			 */

			if (BASIS_FUNCTION == BSplineBasisType.ESPLINE3) order = 3;
			else System.out.println("The order of the ESpline basis function has to be 3");		
			
			//Computation of the subdivision points
			for (int i = 0; i < Level_; i++) {
				alphaSub/=mScaling_;
				computeNSSubdivPoints(mScaling_, pointsToSubdivide, order, alphaSub);
				pointsToSubdivide = subdivCoef_;

				 /*System.out.println("Subdivision points at iteration " + (i+1));
				 for (int k=0; k<subdivCoef_.length; k++){
				 System.out.println("i="+k+"subdivCoefX[k]"+ subdivCoef_[k].x +
				 "subdivCoefY[k] "+ subdivCoef_[k].y); }*/
				 
			}

			if(Level_==0) {
				subdivCoef_ = new Snake2DNode[MSub_];
				
				for(int i=0; i<MSub_; i++){
					subdivCoef_[i]=new Snake2DNode(coef_[i].x, coef_[i].y);
				}
			}

			
			// Computation of the tangents at each subdivision points
			computeDerivativeNSSubdivPoints(order, alphaSub);
			
			// We interpole the subdivision points

			// Convolution by the post refinement filter to put points on the curve
			double[] B;
			B = new double[order];

			int nbSubdivPoints;
			nbSubdivPoints = subdivCoef_.length;

			double[] subdivCoefX, subdivCoefY;
			subdivCoefX = new double[nbSubdivPoints];
			subdivCoefY = new double[nbSubdivPoints];

			for (int i = 0; i < nbSubdivPoints; i++) {
				subdivCoefX[i] = subdivCoef_[i].x;
				subdivCoefY[i] = subdivCoef_[i].y;
			}
			
			for (int i = 0; i < B.length; i++){
				B[i] = BSplineBasis.ESpline3Bis(i,alphaSub, M_);
			}
			
			double[] subdivCoefX2, subdivCoefY2;
			subdivCoefX2 = new double[nbSubdivPoints];
			subdivCoefY2 = new double[nbSubdivPoints];

			subdivCoefX2 = Convolver1D.subdivConvolve(subdivCoefX, subdivCoefX2, B,
					nbSubdivPoints);
			subdivCoefY2 = Convolver1D.subdivConvolve(subdivCoefY, subdivCoefY2, B,
					nbSubdivPoints);
			
			for (int i = 0; i < nbSubdivPoints; i++) {
				subdivCoef_[i] = new Snake2DNode(subdivCoefX2[i], subdivCoefY2[i]);
			}
			
			/*System.out.println("Interpolated subdivision points"); 
			 for (int i=0;i<nbSubdivPoints; i++){ 
				 System.out.println("i="+i+"subdivCoefX[i]"+subdivCoef_[i].x + "subdivCoefY[i] "+ subdivCoef_[i].y); 
			 }*/ 

		}
		
		// ----------------------------------------------------------------------------

		/**
		 * The purpose of this method is to compute the subdivision points for a stationary scheme with the Deslaurier-Dubuc function: m=2.
		 */
		public void subdivisionDeslaurierDubuc() {
			Snake2DNode[] pointsToSubdivide;
			pointsToSubdivide = coef_;

			//Computation of the subdivision points
			for (int i = 0; i < Level_; i++) {
				computeSubdivPointsDeslaurierDubuc(mScaling_, pointsToSubdivide);
				pointsToSubdivide = subdivCoef_;

				/*
				 * System.out.println("Subdivision points at iteration " + (i+1));
				 * for (int k=0; k<subdivCoef_.length; k++){
				 * System.out.println("i="+k+"subdivCoefX[k]"+ subdivCoef_[k].x +
				 * "subdivCoefY[k] "+ subdivCoef_[k].y); }
				 */
			}
			
			if(Level_==0) {
				subdivCoef_ = new Snake2DNode[MSub_];
				
				for(int i=0; i<MSub_; i++){
					subdivCoef_[i]=new Snake2DNode(coef_[i].x, coef_[i].y);
				}
			}
			
			// Computation of the tangents at each subdivision points
			computeDerivativeSubdivPointsDeslaurierDubuc();
			
			// We interpole the subdivision points

			// Convolution by the post refinement filter to put points on the curve
			double[] B;
			//int size=M_* (int) Math.pow(2, Level_);//size for calculError()
			B = new double[7];

			int nbSubdivPoints;
			nbSubdivPoints = subdivCoef_.length;

			double[] subdivCoefX, subdivCoefY;
			subdivCoefX = new double[nbSubdivPoints];
			subdivCoefY = new double[nbSubdivPoints];

			for (int i = 0; i < nbSubdivPoints; i++) {
				subdivCoefX[i] = subdivCoef_[i].x;
				subdivCoefY[i] = subdivCoef_[i].y;
			}

			for (int i = 0; i < B.length; i++) B[i] = 0;
			
			//In comment should be use only when you want to use calculError();
			/*int index=0;
			if(Level_==0) index=(int)Math.pow(2, Level_+1);
			else {
				for(int j=1; j<Level_; j++){
					index+=(int)Math.pow(2, j);
				}
				index+=(int)Math.pow(2, Level_+1);
			}
			B[size+1-index]=1;*/
			
			B[3]=1;

			double[] subdivCoefX2, subdivCoefY2;
			subdivCoefX2 = new double[nbSubdivPoints];
			subdivCoefY2 = new double[nbSubdivPoints];

			subdivCoefX2 = Convolver1D.subdivConvolve(subdivCoefX, subdivCoefX2, B,
					nbSubdivPoints);
			subdivCoefY2 = Convolver1D.subdivConvolve(subdivCoefY, subdivCoefY2, B,
					nbSubdivPoints);

			for (int i = 0; i < nbSubdivPoints; i++) {
				subdivCoef_[i] = new Snake2DNode(subdivCoefX2[i], subdivCoefY2[i]);
			}
			
			 /*System.out.println("Interpolated subdivision points"); for (int i=0;
			 i<nbSubdivPoints; i++){ System.out.println("i="+i+"subdivCoefX[i]"+
			 subdivCoef_[i].x + "subdivCoefY[i] "+ subdivCoef_[i].y); }*/
		}
		
		// ----------------------------------------------------------------------------
		
		/**
		 * The purpose of this method is to compute the subdivision points for a non stationary scheme with the generalized Deslaurier-Dubuc function: m=2, M=4, level_=6.
		 */
		public void subdivisionNSDeslaurierDubuc() {
			Snake2DNode[] pointsToSubdivide;
			pointsToSubdivide = coef_;

			//Computation of the subdivision points
			for (int i = 0; i < 6; i++) {
				computeSubdivPointsNSDeslaurierDubuc(mScaling_, pointsToSubdivide, i+1);
				pointsToSubdivide = subdivCoef_;

				/*
				 * System.out.println("Subdivision points at iteration " + (i+1));
				 * for (int k=0; k<subdivCoef_.length; k++){
				 * System.out.println("i="+k+"subdivCoefX[k]"+ subdivCoef_[k].x +
				 * "subdivCoefY[k] "+ subdivCoef_[k].y); }
				 */
			}
			
			// Computation of the tangents at each subdivision points
			computeDerivativeNSSubdivPointsDeslaurierDubuc();
			
			// We interpole the subdivision points

			// Convolution by the post refinement filter to put points on the curve
			double[] B;
			B = new double[11];

			int nbSubdivPoints;
			nbSubdivPoints = subdivCoef_.length;

			double[] subdivCoefX, subdivCoefY;
			subdivCoefX = new double[nbSubdivPoints];
			subdivCoefY = new double[nbSubdivPoints];

			for (int i = 0; i < nbSubdivPoints; i++) {
				subdivCoefX[i] = subdivCoef_[i].x;
				subdivCoefY[i] = subdivCoef_[i].y;
			}

			for (int i = 0; i < B.length; i++) B[i] = 0;
			B[5]=1;
			
			double[] subdivCoefX2, subdivCoefY2;
			subdivCoefX2 = new double[nbSubdivPoints];
			subdivCoefY2 = new double[nbSubdivPoints];

			subdivCoefX2 = Convolver1D.subdivConvolve(subdivCoefX, subdivCoefX2, B,
					nbSubdivPoints);
			subdivCoefY2 = Convolver1D.subdivConvolve(subdivCoefY, subdivCoefY2, B,
					nbSubdivPoints);

			for (int i = 0; i < nbSubdivPoints; i++) {
				subdivCoef_[i] = new Snake2DNode(subdivCoefX2[i], subdivCoefY2[i]);
			}
			
			 /*System.out.println("Interpolated subdivision points"); for (int i=0;
			 i<nbSubdivPoints; i++){ System.out.println("i="+i+"subdivCoefX[i]"+
			 subdivCoef_[i].x + "subdivCoefY[i] "+ subdivCoef_[i].y); }*/
		}

	// ----------------------------------------------------------------------------
		
		// ----------------------------------------------------------------------------
		
		/**
		 * The purpose of this method is to compute the subdivision points for a non stationary scheme with the generalized Deslaurier-Dubuc function: m=2, M=4, level_=6, gamma=(0,0,2jPi/M,-2jPi/M).
		 */
		public void subdivisionNSDeslaurierDubuc2() {
			Snake2DNode[] pointsToSubdivide;
			pointsToSubdivide = coef_;

			//Computation of the subdivision points
			for (int i = 0; i < 6; i++) {
				computeSubdivPointsNSDeslaurierDubuc2(mScaling_, pointsToSubdivide, i+1);
				pointsToSubdivide = subdivCoef_;

				/*
				 * System.out.println("Subdivision points at iteration " + (i+1));
				 * for (int k=0; k<subdivCoef_.length; k++){
				 * System.out.println("i="+k+"subdivCoefX[k]"+ subdivCoef_[k].x +
				 * "subdivCoefY[k] "+ subdivCoef_[k].y); }
				 */
			}
			
			// Computation of the tangents at each subdivision points
			computeDerivativeNSSubdivPointsDeslaurierDubuc2();
			
			// We interpole the subdivision points

			// Convolution by the post refinement filter to put points on the curve
			double[] B;
			B = new double[7];

			int nbSubdivPoints;
			nbSubdivPoints = subdivCoef_.length;

			double[] subdivCoefX, subdivCoefY;
			subdivCoefX = new double[nbSubdivPoints];
			subdivCoefY = new double[nbSubdivPoints];

			for (int i = 0; i < nbSubdivPoints; i++) {
				subdivCoefX[i] = subdivCoef_[i].x;
				subdivCoefY[i] = subdivCoef_[i].y;
			}

			for (int i = 0; i < B.length; i++) B[i] = 0;
			B[3]=1;
			
			double[] subdivCoefX2, subdivCoefY2;
			subdivCoefX2 = new double[nbSubdivPoints];
			subdivCoefY2 = new double[nbSubdivPoints];

			subdivCoefX2 = Convolver1D.subdivConvolve(subdivCoefX, subdivCoefX2, B,
					nbSubdivPoints);
			subdivCoefY2 = Convolver1D.subdivConvolve(subdivCoefY, subdivCoefY2, B,
					nbSubdivPoints);

			for (int i = 0; i < nbSubdivPoints; i++) {
				subdivCoef_[i] = new Snake2DNode(subdivCoefX2[i], subdivCoefY2[i]);
			}
			
			 /*System.out.println("Interpolated subdivision points"); for (int i=0;
			 i<nbSubdivPoints; i++){ System.out.println("i="+i+"subdivCoefX[i]"+
			 subdivCoef_[i].x + "subdivCoefY[i] "+ subdivCoef_[i].y); }*/
		}

	// ----------------------------------------------------------------------------
		
	/**
	 * Initially part of {@link ESnakeSD#initialize(boolean)}.
	 * Put it in an external method so it can be easily overridden (see SubsnakeSD class).
	 */
	protected void initializeContoursAndTangents() {
		xSnakeContour_ = new double[MR_];
		ySnakeContour_ = new double[MR_];
		
		xSnakeTangentVector_ = new double[MR_];
		ySnakeTangentVector_ = new double[MR_];
		
		xSnakeShellContour_ = new double[MSub_];
		ySnakeShellContour_ = new double[MSub_];

		xSnakeContourSubdivision_ = new double[MSub_];
		ySnakeContourSubdivision_ = new double[MSub_];
		
		xSnakeTangentVectorSubdivision_ = new double[MSub_];
		ySnakeTangentVectorSubdivision_ = new double[MSub_];
	}
	
	// ----------------------------------------------------------------------------

	/**
	 * The purpose of this method is to compute the subdivision points for a m
	 * scaling refinement for a stationary scheme (with bSpline)
	 */
	private void computeSubdivPoints(int m, Snake2DNode[] pointsToSubdivide,
			int order) {
		double[] h, SamplingCoefX, SamplingCoefY, subdivCoefX, subdivCoefY;
		int Msub, MinputCoef_;
		int SamplingCoefSize, filterSize;
		MinputCoef_ = pointsToSubdivide.length;

		filterSize = order * (m - 1) + 1;
		h = new double[filterSize];

		SamplingCoefSize = m * MinputCoef_;
		SamplingCoefX = new double[SamplingCoefSize];
		SamplingCoefY = new double[SamplingCoefSize];

		Msub = m * MinputCoef_;
		subdivCoefX = new double[Msub];
		subdivCoefY = new double[Msub];
		subdivCoef_ = new Snake2DNode[Msub];

		// compute the mask
		h = BSplineBasis.MaskPolynomialSpline(m, order);

		// m sampling of the coefficients (control points)
		for (int i = 0; i < SamplingCoefSize; i++) {
			SamplingCoefX[i] = 0;
			SamplingCoefY[i] = 0;
		}

		for (int i = 0; i < MinputCoef_; i++) {
			SamplingCoefX[m * i] = pointsToSubdivide[i].x;
			SamplingCoefY[m * i] = pointsToSubdivide[i].y;
		}

		// Convolution
		subdivCoefX = Convolver1D.subdivConvolve(SamplingCoefX, subdivCoefX, h,
				SamplingCoefSize);
		subdivCoefY = Convolver1D.subdivConvolve(SamplingCoefY, subdivCoefY, h,
				SamplingCoefSize);

		for (int i = 0; i < Msub; i++) {
			subdivCoef_[i] = new Snake2DNode(subdivCoefX[i], subdivCoefY[i]);
		}

		
//		  for (int i=0; i<Msub; i++){
//		  System.out.println("i="+i+"subdivCoefX[i]"+ subdivCoef_[i].x +
//		  "subdivCoefY[i] "+ subdivCoef_[i].y); }
		
	}
	
	// ----------------------------------------------------------------------------
		/**
		 * The purpose of this method is to compute the subdivision points for a m
		 * scaling refinement for a stationary scheme using the Deslaurier and Dubuc function which reproduces polynomials of degree 3. For m=2;
		 */
		private void computeSubdivPointsDeslaurierDubuc(int m, Snake2DNode[] pointsToSubdivide) {
			double[] h, SamplingCoefX, SamplingCoefY, subdivCoefX, subdivCoefY;
			int Msub, MinputCoef_;
			int SamplingCoefSize;
			MinputCoef_ = pointsToSubdivide.length;

			h = new double[7];

			SamplingCoefSize = m * MinputCoef_;
			SamplingCoefX = new double[SamplingCoefSize];
			SamplingCoefY = new double[SamplingCoefSize];

			Msub = m * MinputCoef_;
			subdivCoefX = new double[Msub];
			subdivCoefY = new double[Msub];
			subdivCoef_ = new Snake2DNode[Msub];
			
			if(m!=2) System.out.println("Works only for a refinement factor equal to 2");

			// compute the mask
			h[0]=-0.1250/2.0;
			h[1]=0;
			h[2]=1.125/2.0;
			h[3]=2/2.0;
			h[4]=h[2];
			h[5]=h[1];
			h[6]=h[0];

			// m sampling of the coefficients (control points)
			for (int i = 0; i < SamplingCoefSize; i++) {
				SamplingCoefX[i] = 0;
				SamplingCoefY[i] = 0;
			}

			for (int i = 0; i < MinputCoef_; i++) {
				SamplingCoefX[m * i] = pointsToSubdivide[i].x;
				SamplingCoefY[m * i] = pointsToSubdivide[i].y;
			}

			// Convolution
			subdivCoefX = Convolver1D.subdivConvolve(SamplingCoefX, subdivCoefX, h,
					SamplingCoefSize);
			subdivCoefY = Convolver1D.subdivConvolve(SamplingCoefY, subdivCoefY, h,
					SamplingCoefSize);

			for (int i = 0; i < Msub; i++) {
				subdivCoef_[i] = new Snake2DNode(subdivCoefX[i], subdivCoefY[i]);
//				subdivCoef_[i] = new Snake2DNode(Math.round(subdivCoefX[i]), 
//						Math.round(subdivCoefY[i]));
				// TODO Math.round coordinates to easier common points suppression in GlobalSnake ?
			}

			/*
			 * for (int i=0; i<Msub; i++){
			 * System.out.println("i="+i+"subdivCoefX[i]"+ subdivCoef_[i].x +
			 * "subdivCoefY[i] "+ subdivCoef_[i].y); }
			 */
		}
		
		// ----------------------------------------------------------------------------

		/**
		 * The purpose of this method is to compute the subdivision points for a m
		 * scaling refinement for a non stationary scheme using the Deslaurier and Dubuc function M=4, level_=6, m=2 and gamma=(0,0,2jPi/M,2jPi/M,-2jPi/M,-2jPi/M);
		 */
		private void computeSubdivPointsNSDeslaurierDubuc(int m, Snake2DNode[] pointsToSubdivide, int iteration) {
			double[] h, SamplingCoefX, SamplingCoefY, subdivCoefX, subdivCoefY, hScalingFunction;
			int Msub, MinputCoef_;
			int SamplingCoefSize;
			MinputCoef_ = pointsToSubdivide.length;

			h = new double[11];
			hScalingFunction= new double[6];

			SamplingCoefSize = m * MinputCoef_;
			SamplingCoefX = new double[SamplingCoefSize];
			SamplingCoefY = new double[SamplingCoefSize];

			Msub = m * MinputCoef_;
			subdivCoefX = new double[Msub];
			subdivCoefY = new double[Msub];
			subdivCoef_ = new Snake2DNode[Msub];
			
			if(m!=2) System.out.println("Works only for a refinement factor equal to 2");

			// compute the mask
			if(iteration==1){
				hScalingFunction[0]=0.6454;
				hScalingFunction[1]=1.0921;
				hScalingFunction[2]=0.5466;
				hScalingFunction[3]=-0.2056;
				hScalingFunction[4]=-0.1920;
				hScalingFunction[5]=0.1135;
			}		
			else if(iteration==2){
				hScalingFunction[0]=0.5062;
				hScalingFunction[1]=1.1350;
				hScalingFunction[2]=0.6288;
				hScalingFunction[3]=-0.1953;
				hScalingFunction[4]=-0.1350;
				hScalingFunction[5]=0.0602;

			}
			else if(iteration==3){
				hScalingFunction[0]=0.4790;
				hScalingFunction[1]=1.1398;
				hScalingFunction[2]=0.6452;
				hScalingFunction[3]=-0.1920;
				hScalingFunction[4]=-0.1242;
				hScalingFunction[5]=0.0522;
			}
			else if(iteration==4){
				hScalingFunction[0]=0.4726;
				hScalingFunction[1]=1.1408;
				hScalingFunction[2]=0.6491;
				hScalingFunction[3]=-0.1912;
				hScalingFunction[4]=-0.1217;
				hScalingFunction[5]=0.0504;
			}
			else if(iteration==5){
				hScalingFunction[0]=0.4710;
				hScalingFunction[1]=1.1410;
				hScalingFunction[2]=0.6500;
				hScalingFunction[3]=-0.1910;
				hScalingFunction[4]=-0.1210;
				hScalingFunction[5]=0.0500;
			}
			else if(iteration==6){
				hScalingFunction[0]=0.4706;
				hScalingFunction[1]=1.1411;
				hScalingFunction[2]=0.6503;
				hScalingFunction[3]=-0.1910;
				hScalingFunction[4]=-0.1209;
				hScalingFunction[5]=0.0499;
			}
			
			h[0]=0.5*hScalingFunction[0]*hScalingFunction[5];
			h[1]=0.5*(hScalingFunction[0]*hScalingFunction[4]+hScalingFunction[1]*hScalingFunction[5]);
			h[2]=0.5*(hScalingFunction[0]*hScalingFunction[3]+hScalingFunction[1]*hScalingFunction[4]+hScalingFunction[2]*hScalingFunction[5]);
			h[3]=0.5*(hScalingFunction[0]*hScalingFunction[2]+hScalingFunction[1]*hScalingFunction[3]+hScalingFunction[2]*hScalingFunction[4]+hScalingFunction[3]*hScalingFunction[5]);
			h[4]=0.5*(hScalingFunction[0]*hScalingFunction[1]+hScalingFunction[1]*hScalingFunction[2]+hScalingFunction[2]*hScalingFunction[3]+hScalingFunction[3]*hScalingFunction[4]+hScalingFunction[4]*hScalingFunction[5]);
			h[5]=0.5*(hScalingFunction[0]*hScalingFunction[0]+hScalingFunction[1]*hScalingFunction[1]+hScalingFunction[2]*hScalingFunction[2]+hScalingFunction[3]*hScalingFunction[3]+hScalingFunction[4]*hScalingFunction[4]+hScalingFunction[5]*hScalingFunction[5]);
			h[6]=h[4];
			h[7]=h[3];
			h[8]=h[2];
			h[9]=h[1];
			h[10]=h[0];

			// m sampling of the coefficients (control points)
			for (int i = 0; i < SamplingCoefSize; i++) {
				SamplingCoefX[i] = 0;
				SamplingCoefY[i] = 0;
			}

			for (int i = 0; i < MinputCoef_; i++) {
				SamplingCoefX[m * i] = pointsToSubdivide[i].x;
				SamplingCoefY[m * i] = pointsToSubdivide[i].y;
			}

			// Convolution
			subdivCoefX = Convolver1D.subdivConvolve(SamplingCoefX, subdivCoefX, h,
					SamplingCoefSize);
			subdivCoefY = Convolver1D.subdivConvolve(SamplingCoefY, subdivCoefY, h,
					SamplingCoefSize);

			for (int i = 0; i < Msub; i++) {
				subdivCoef_[i] = new Snake2DNode(subdivCoefX[i], subdivCoefY[i]);
			}

			/*
			 * for (int i=0; i<Msub; i++){
			 * System.out.println("i="+i+"subdivCoefX[i]"+ subdivCoef_[i].x +
			 * "subdivCoefY[i] "+ subdivCoef_[i].y); }
			 */
		}
	
	// ----------------------------------------------------------------------------
		/**
		 * The purpose of this method is to compute the subdivision points for a m
		 * scaling refinement for a non stationary scheme using the Deslaurier and Dubuc function M=4, level_=6, m=2 and gamma=(0,0,2jPi/M,-2jPi/M);
		 */
		private void computeSubdivPointsNSDeslaurierDubuc2(int m, Snake2DNode[] pointsToSubdivide, int iteration) {
			double[] h, SamplingCoefX, SamplingCoefY, subdivCoefX, subdivCoefY, hScalingFunction;
			int Msub, MinputCoef_;
			int SamplingCoefSize;
			MinputCoef_ = pointsToSubdivide.length;

			h = new double[7];
			hScalingFunction= new double[6];

			SamplingCoefSize = m * MinputCoef_;
			SamplingCoefX = new double[SamplingCoefSize];
			SamplingCoefY = new double[SamplingCoefSize];

			Msub = m * MinputCoef_;
			subdivCoefX = new double[Msub];
			subdivCoefY = new double[Msub];
			subdivCoef_ = new Snake2DNode[Msub];
			
			if(m!=2) System.out.println("Works only for a refinement factor equal to 2");

			// compute the mask
			double ReDC=2+4*Math.cos(Math.PI/Math.pow(2, iteration+1))+2*Math.cos(Math.PI/Math.pow(2, iteration-1))+4*Math.cos(Math.PI/Math.pow(2, iteration))+4*Math.cos(3*Math.PI/Math.pow(2, iteration+1));
			double ImDC=4*Math.sin(Math.PI/Math.pow(2, iteration+1))+2*Math.sin(Math.PI/Math.pow(2, iteration-1))+4*Math.sin(Math.PI/Math.pow(2, iteration))+4*Math.sin(3*Math.PI/Math.pow(2, iteration+1));

			double ReNum1=1+2*Math.cos(Math.PI/Math.pow(2, iteration+1))+Math.cos(Math.PI/Math.pow(2, iteration-1))+3*Math.cos(Math.PI/Math.pow(2, iteration))+2*Math.cos(3*Math.PI/Math.pow(2, iteration+1));
			double ImNum1=2*Math.sin(Math.PI/Math.pow(2, iteration+1))+Math.sin(Math.PI/Math.pow(2, iteration-1))+3*Math.sin(Math.PI/Math.pow(2, iteration))+2*Math.sin(3*Math.PI/Math.pow(2, iteration+1));
			
			h[0]=-(Math.cos(Math.PI/Math.pow(2, iteration))*ReDC+Math.sin(Math.PI/Math.pow(2, iteration))*ImDC)/(ReDC*ReDC+ImDC*ImDC);
			h[1]=0;
			h[2]=(ReNum1*ReDC+ImNum1*ImDC)/(ReDC*ReDC+ImDC*ImDC);
			h[3]=1;
			h[4]=h[2];
			h[5]=h[1];
			h[6]=h[0];

			// m sampling of the coefficients (control points)
			for (int i = 0; i < SamplingCoefSize; i++) {
				SamplingCoefX[i] = 0;
				SamplingCoefY[i] = 0;
			}

			for (int i = 0; i < MinputCoef_; i++) {
				SamplingCoefX[m * i] = pointsToSubdivide[i].x;
				SamplingCoefY[m * i] = pointsToSubdivide[i].y;
			}

			// Convolution
			subdivCoefX = Convolver1D.subdivConvolve(SamplingCoefX, subdivCoefX, h,
					SamplingCoefSize);
			subdivCoefY = Convolver1D.subdivConvolve(SamplingCoefY, subdivCoefY, h,
					SamplingCoefSize);

			for (int i = 0; i < Msub; i++) {
				subdivCoef_[i] = new Snake2DNode(subdivCoefX[i], subdivCoefY[i]);
			}

			/*
			 * for (int i=0; i<Msub; i++){
			 * System.out.println("i="+i+"subdivCoefX[i]"+ subdivCoef_[i].x +
			 * "subdivCoefY[i] "+ subdivCoef_[i].y); }
			 */
		}
	
	// ----------------------------------------------------------------------------

	/**
	 * The purpose of this method is to compute the subdivision points for a m
	 * scaling.
	 */
	private void computeNSSubdivPoints(int m, Snake2DNode[] pointsToSubdivide,
			int order, double alphaSub) {
		double[] h, SamplingCoefX, SamplingCoefY, subdivCoefX, subdivCoefY;
		int Msub, MinputCoef_;
		int SamplingCoefSize, filterSize;
		MinputCoef_ = pointsToSubdivide.length;

		filterSize = order * (m - 1) + 1;
		h = new double[filterSize];

		SamplingCoefSize = m * MinputCoef_;
		SamplingCoefX = new double[SamplingCoefSize];
		SamplingCoefY = new double[SamplingCoefSize];

		Msub = m * MinputCoef_;
		subdivCoefX = new double[Msub];
		subdivCoefY = new double[Msub];
		subdivCoef_ = new Snake2DNode[Msub];

		// compute the mask
		h = BSplineBasis.MaskESpline(m, order, alphaSub);

		// m sampling of the coefficients (control points)
		for (int i = 0; i < SamplingCoefSize; i++) {
			SamplingCoefX[i] = 0;
			SamplingCoefY[i] = 0;
		}

		for (int i = 0; i < MinputCoef_; i++) {
			SamplingCoefX[m * i] = pointsToSubdivide[i].x;
			SamplingCoefY[m * i] = pointsToSubdivide[i].y;
		}

		// Convolution
		subdivCoefX = Convolver1D.subdivConvolve(SamplingCoefX, subdivCoefX, h,
				SamplingCoefSize);
		subdivCoefY = Convolver1D.subdivConvolve(SamplingCoefY, subdivCoefY, h,
				SamplingCoefSize);

		for (int i = 0; i < Msub; i++) {
			subdivCoef_[i] = new Snake2DNode(subdivCoefX[i], subdivCoefY[i]);
		}

		/*
		 * for (int i=0; i<Msub; i++){
		 * System.out.println("i="+i+"subdivCoefX[i]"+ subdivCoef_[i].x +
		 * "subdivCoefY[i] "+ subdivCoef_[i].y); }
		 */
	}

	// ----------------------------------------------------------------------------

	/**
	 * The purpose of this method is to compute the derivative subdivision
	 * points for a m scaling.
	 */
	protected void computeDerivativeSubdivPoints(int N) {
		int size = subdivCoef_.length;

		dersubdivCoef_ = new Snake2DNode[size];

		double[] Der;
		Der = new double[N + 1];

		Der = BSplineBasis.computePolynomialDerivativeMask(N);

		
		 //for(int i=0; i<N+1; i++){ System.out.println(Der[i]); }
		 

		double[] subdivCoefX, subdivCoefY;
		subdivCoefX = new double[size];
		subdivCoefY = new double[size];

		for (int i = 0; i < size; i++) {
			subdivCoefX[i] = subdivCoef_[i].x;
			subdivCoefY[i] = subdivCoef_[i].y;
		}

		double[] DersubdivCoefX, DersubdivCoefY;
		DersubdivCoefX = new double[size];
		DersubdivCoefY = new double[size];
		
		//Method 1: with the post filter

		DersubdivCoefX = Convolver1D.subdivConvolve(subdivCoefX,
				DersubdivCoefX, Der, size);
		DersubdivCoefY = Convolver1D.subdivConvolve(subdivCoefY,
				DersubdivCoefY, Der, size);

		for (int i = 0; i < size; i++) {
			dersubdivCoef_[i] = new Snake2DNode(Math.pow(mScaling_, Level_)*DersubdivCoefX[i],
					Math.pow(2, Level_)*DersubdivCoefY[i]);
		}

		
//		  System.out.println("Derivative subdivision points"); for (int i=0;
//		  i<size; i++){ System.out.println("i="+i+"DersubdivCoefX[i]"+
//		  dersubdivCoef_[i].x + "DersubdivCoefY[i] "+ dersubdivCoef_[i].y); }
		 
		
			//Method 2: with the formule of the subfivision : (1/2^k)(Pk[m+1]-Pk[m])
			
//			for (int i = 0; i < size; i++) {
//				if(i==size-1){
//					DersubdivCoefX[i]=Math.pow(2, Level_)*(subdivCoef_[0].x-subdivCoef_[i].x);
//					DersubdivCoefY[i]=Math.pow(2, Level_)*(subdivCoef_[0].y-subdivCoef_[i].y);
//				}
//				else {
//				DersubdivCoefX[i]=Math.pow(2, Level_)*(subdivCoef_[i+1].x-subdivCoef_[i].x);
//				DersubdivCoefY[i]=Math.pow(2, Level_)*(subdivCoef_[i+1].y-subdivCoef_[i].y);
//				}
//			}
//			
//			for (int i = 0; i < size; i++) {
//				dersubdivCoef_[i] = new Snake2DNode(DersubdivCoefX[i],
//						DersubdivCoefY[i]);
//			}
		
//		 System.out.println("Derivative subdivision points"); for (int i=0;
//				  i<size; i++){ System.out.println("i="+i+"DersubdivCoefX[i]"+
//				  dersubdivCoef_[i].x + "DersubdivCoefY[i] "+ dersubdivCoef_[i].y); }

	}
	// ----------------------------------------------------------------------------

		/**
		 * The purpose of this method is to compute the derivative subdivision
		 * points for a m scaling.
		 */
		private void computeDerivativeNSSubdivPoints(int N, double alphaSub) {
			int size = subdivCoef_.length;

			dersubdivCoef_ = new Snake2DNode[size];

			double[] Der;
			Der = new double[N + 1];

			Der = BSplineBasis.computeESplineDerivativeMask(N, alphaSub, M_);

			/*
			 * for(int i=0; i<N+1; i++){ System.out.println(Der[i]); }
			 */

			double[] subdivCoefX, subdivCoefY;
			subdivCoefX = new double[size];
			subdivCoefY = new double[size];
			
			//Method 1: with the post filter

			for (int i = 0; i < size; i++) {
				subdivCoefX[i] = subdivCoef_[i].x;
				subdivCoefY[i] = subdivCoef_[i].y;
			}

			double[] DersubdivCoefX, DersubdivCoefY;
			DersubdivCoefX = new double[size];
			DersubdivCoefY = new double[size];

			DersubdivCoefX = Convolver1D.subdivConvolve(subdivCoefX,
					DersubdivCoefX, Der, size);
			DersubdivCoefY = Convolver1D.subdivConvolve(subdivCoefY,
					DersubdivCoefY, Der, size);

			for (int i = 0; i < size; i++) {
				dersubdivCoef_[i] = new Snake2DNode(Math.pow(mScaling_, Level_)*DersubdivCoefX[i],
						Math.pow(2, Level_)*DersubdivCoefY[i]);
			}
			
			//Method 2: with the formule of the subfivision : (1/2^k)(Pk[m+1]-Pk[m])
			
//			for (int i = 0; i < size; i++) {
//				if(i==size-1){
//					DersubdivCoefX[i]=Math.pow(2, Level_)*(subdivCoef_[0].x-subdivCoef_[i].x);
//					DersubdivCoefY[i]=Math.pow(2, Level_)*(subdivCoef_[0].y-subdivCoef_[i].y);
//				}
//				else {
//				DersubdivCoefX[i]=Math.pow(2, Level_)*(subdivCoef_[i+1].x-subdivCoef_[i].x);
//				DersubdivCoefY[i]=Math.pow(2, Level_)*(subdivCoef_[i+1].y-subdivCoef_[i].y);
//				}
//			}
//			
//			for (int i = 0; i < size; i++) {
//				dersubdivCoef_[i] = new Snake2DNode(DersubdivCoefX[i],
//						DersubdivCoefY[i]);
//			}

			/*
			 * System.out.println("Derivative subdivision points"); for (int i=0;
			 * i<size; i++){ System.out.println("i="+i+"DersubdivCoefX[i]"+
			 * dersubdivCoef_[i].x + "DersubdivCoefY[i] "+ dersubdivCoef_[i].y); }
			 */

		}
		
		// ----------------------------------------------------------------------------
		/**
		 * The purpose of this method is to compute the derivative subdivision
		 * points for a 2 scaling for the stationary Deslaurier Dubuc scheme.
		 */
		protected void computeDerivativeSubdivPointsDeslaurierDubuc() {
			int size = subdivCoef_.length;

			dersubdivCoef_ = new Snake2DNode[size];

			double[] Der;
			Der = new double[7];

			Der[0]=0;
			Der[1]=-0.0833;
			Der[2]=0.6667;
			Der[3]=0;
			Der[4]=-0.6667;
			Der[5]=0.0833;
			Der[6]=0;

			double[] subdivCoefX, subdivCoefY;
			subdivCoefX = new double[size];
			subdivCoefY = new double[size];
			
			//Method 1: with the post filter

			for (int i = 0; i < size; i++) {
				subdivCoefX[i] = subdivCoef_[i].x;
				subdivCoefY[i] = subdivCoef_[i].y;
			}

			double[] DersubdivCoefX, DersubdivCoefY;
			DersubdivCoefX = new double[size];
			DersubdivCoefY = new double[size];

			DersubdivCoefX = Convolver1D.subdivConvolve(subdivCoefX,
					DersubdivCoefX, Der, size);
			DersubdivCoefY = Convolver1D.subdivConvolve(subdivCoefY,
					DersubdivCoefY, Der, size);

			for (int i = 0; i < size; i++) {
				dersubdivCoef_[i] = new Snake2DNode(Math.pow(mScaling_, Level_)*DersubdivCoefX[i],
						Math.pow(2, Level_)*DersubdivCoefY[i]);
			}
			
			//Method 2: with the formule of the subfivision : (1/2^k)(Pk[m+1]-Pk[m])
			
//			for (int i = 0; i < size; i++) {
//				if(i==size-1){
//					DersubdivCoefX[i]=Math.pow(2, Level_)*(subdivCoef_[0].x-subdivCoef_[i].x);
//					DersubdivCoefY[i]=Math.pow(2, Level_)*(subdivCoef_[0].y-subdivCoef_[i].y);
//				}
//				else {
//				DersubdivCoefX[i]=Math.pow(2, Level_)*(subdivCoef_[i+1].x-subdivCoef_[i].x);
//				DersubdivCoefY[i]=Math.pow(2, Level_)*(subdivCoef_[i+1].y-subdivCoef_[i].y);
//				}
//			}
//			
//			for (int i = 0; i < size; i++) {
//				dersubdivCoef_[i] = new Snake2DNode(DersubdivCoefX[i],
//						DersubdivCoefY[i]);
//			}

			/*
			 * System.out.println("Derivative subdivision points"); for (int i=0;
			 * i<size; i++){ System.out.println("i="+i+"DersubdivCoefX[i]"+
			 * dersubdivCoef_[i].x + "DersubdivCoefY[i] "+ dersubdivCoef_[i].y); }
			 */
		}
		// ----------------------------------------------------------------------------
				/**
				 * The purpose of this method is to compute the derivative subdivision
				 * points for a 2 scaling for the non stationary Deslaurier Dubuc scheme.
				 */
				private void computeDerivativeNSSubdivPointsDeslaurierDubuc() {
					int size = subdivCoef_.length;

					dersubdivCoef_ = new Snake2DNode[size];

					double[] Der;
					Der = new double[11];

					Der[0]=0;
					Der[1]=0.0003;
					Der[2]=0.0146;
					Der[3]=-0.1451;
					Der[4]=0.745;
					Der[5]=0;
					Der[6]=-0.745;
					Der[7]=0.1451;
					Der[8]=-0.0146;
					Der[9]=-0.0003;
					Der[10]=0;

					double[] subdivCoefX, subdivCoefY;
					subdivCoefX = new double[size];
					subdivCoefY = new double[size];

					for (int i = 0; i < size; i++) {
						subdivCoefX[i] = subdivCoef_[i].x;
						subdivCoefY[i] = subdivCoef_[i].y;
					}

					double[] DersubdivCoefX, DersubdivCoefY;
					DersubdivCoefX = new double[size];
					DersubdivCoefY = new double[size];
					
					//Method 1: with the post filter

					DersubdivCoefX = Convolver1D.subdivConvolve(subdivCoefX,
							DersubdivCoefX, Der, size);
					DersubdivCoefY = Convolver1D.subdivConvolve(subdivCoefY,
							DersubdivCoefY, Der, size);

					for (int i = 0; i < size; i++) {
						dersubdivCoef_[i] = new Snake2DNode(Math.pow(mScaling_, Level_)*DersubdivCoefX[i],
								Math.pow(2, Level_)*DersubdivCoefY[i]);
					}
					
					//Method 2: with the formule of the subfivision : (1/2^k)(Pk[m+1]-Pk[m])
					
//					for (int i = 0; i < size; i++) {
//						if(i==size-1){
//							DersubdivCoefX[i]=Math.pow(2, Level_)*(subdivCoef_[0].x-subdivCoef_[i].x);
//							DersubdivCoefY[i]=Math.pow(2, Level_)*(subdivCoef_[0].y-subdivCoef_[i].y);
//						}
//						else {
//						DersubdivCoefX[i]=Math.pow(2, Level_)*(subdivCoef_[i+1].x-subdivCoef_[i].x);
//						DersubdivCoefY[i]=Math.pow(2, Level_)*(subdivCoef_[i+1].y-subdivCoef_[i].y);
//						}
//					}
//					
//					for (int i = 0; i < size; i++) {
//						dersubdivCoef_[i] = new Snake2DNode(DersubdivCoefX[i],
//								DersubdivCoefY[i]);
//					}

					/*
					 * System.out.println("Derivative subdivision points"); for (int i=0;
					 * i<size; i++){ System.out.println("i="+i+"DersubdivCoefX[i]"+
					 * dersubdivCoef_[i].x + "DersubdivCoefY[i] "+ dersubdivCoef_[i].y); }
					 */

				}

	// ----------------------------------------------------------------------------
				// ----------------------------------------------------------------------------
				/**
				 * The purpose of this method is to compute the derivative subdivision
				 * points for a 2 scaling for the non stationary Deslaurier Dubuc scheme, gamma=(0,0,2jPi/M,-2jPi/M).
				 */
				private void computeDerivativeNSSubdivPointsDeslaurierDubuc2() {
					int size = subdivCoef_.length;

					dersubdivCoef_ = new Snake2DNode[size];

					double[] Der;
					Der = new double[7];

					Der[0]=0;
					Der[1]=-0.0833;
					Der[2]=0.6667;
					Der[3]=0;
					Der[4]=-0.6667;
					Der[5]=0.0833;
					Der[6]=0;

					double[] subdivCoefX, subdivCoefY;
					subdivCoefX = new double[size];
					subdivCoefY = new double[size];

					for (int i = 0; i < size; i++) {
						subdivCoefX[i] = subdivCoef_[i].x;
						subdivCoefY[i] = subdivCoef_[i].y;
					}

					double[] DersubdivCoefX, DersubdivCoefY;
					DersubdivCoefX = new double[size];
					DersubdivCoefY = new double[size];
					
					//Method 1: with the post filter

					DersubdivCoefX = Convolver1D.subdivConvolve(subdivCoefX,
							DersubdivCoefX, Der, size);
					DersubdivCoefY = Convolver1D.subdivConvolve(subdivCoefY,
							DersubdivCoefY, Der, size);

					for (int i = 0; i < size; i++) {
						dersubdivCoef_[i] = new Snake2DNode(Math.pow(mScaling_, Level_)*DersubdivCoefX[i],
								Math.pow(2, Level_)*DersubdivCoefY[i]);
					}
					 
					//Method 2: with the formule of the subfivision : (1/2^k)(Pk[m+1]-Pk[m])
					
//					for (int i = 0; i < size; i++) {
//						if(i==size-1){
//							DersubdivCoefX[i]=Math.pow(2, Level_)*(subdivCoef_[0].x-subdivCoef_[i].x);
//							DersubdivCoefY[i]=Math.pow(2, Level_)*(subdivCoef_[0].y-subdivCoef_[i].y);
//						}
//						else {
//						DersubdivCoefX[i]=Math.pow(2, Level_)*(subdivCoef_[i+1].x-subdivCoef_[i].x);
//						DersubdivCoefY[i]=Math.pow(2, Level_)*(subdivCoef_[i+1].y-subdivCoef_[i].y);
//						}
//					}
//					
//					for (int i = 0; i < size; i++) {
//						dersubdivCoef_[i] = new Snake2DNode(DersubdivCoefX[i],
//								DersubdivCoefY[i]);
//					}
//					
//					 System.out.println("Derivative subdivision points"); for (int i=0;
//							  i<size; i++){ System.out.println("i="+i+"DersubdivCoefX[i]"+
//							  dersubdivCoef_[i].x + "DersubdivCoefY[i] "+ dersubdivCoef_[i].y); }
					

				}

	// ----------------------------------------------------------------------------

	/**
	 * The purpose of this method is to monitor the status of the snake.
	 */
	@Override
	public boolean isAlive() {
		return alive_;
	}

	// ----------------------------------------------------------------------------

	/** Returns <code>true</code> if the snake has been initialized. */
	@Override
	public boolean isInitialized() {
		return isInitialized_;
	}

	// ----------------------------------------------------------------------------

	/**
	 * Sets the status of the snake to alive, and restores the maximum number
	 * iterations to the original one.
	 */
	@Override
	public void reviveSnake() {
		alive_ = true;
		life_ = maxLife_;
	}

	// ----------------------------------------------------------------------------

	/** Saves the snake-defining parameters in an XML file. */
	@Override
	public void saveToXML(Element node) {
		getSnakeParameters().saveToXML(node);
		Element controlPointsElement = XMLUtil.addElement(node,
				ID_CONTROL_POINTS);
		for (Snake2DNode controlPoint : coef_) {
			Element controlPointElement = XMLUtil.addElement(
					controlPointsElement, ID_CONTROL_POINT);
			controlPoint.saveToXML(controlPointElement);
		}
	}

	// ----------------------------------------------------------------------------

	/** This method provides a mutator to the snake-defining nodes. */
	@Override
	public void setNodes(Snake2DNode[] node) {
		for (int i = 0; i < M_; i++) {
			coef_[i].x = node[i].x;
			coef_[i].y = node[i].y;
		}

		updateSnakeSkin();
		updateAreaSubdivision();
		// updateArea();

		if (energyType_ == ESnakeEnergyTypeSD.REGION
				|| energyType_ == ESnakeEnergyTypeSD.MIXTURE) {
			updateSnakeShell();
		}
		validateSnake();
	}

	// ----------------------------------------------------------------------------
	// ESNAKE METHODS

	/** Retrieves the area under the curve determined by the snake. */
	public double getArea() {
		return Math.abs(area_);
	}

	// ----------------------------------------------------------------------------
	public BSplineBasisType getBasisFunctionType() {
		return basisFunctionType_;
	}
	
	// ----------------------------------------------------------------------------
	public MultiresolutionTypeSD getMultiresolutionType() {
		return multiresolutionType_;
	}

	// ----------------------------------------------------------------------------


	/** Returns the bounding box of the snake curve. */
	public Rectangle getBounds() {
		return snakeContourBoundingBox_;
	}

	// ----------------------------------------------------------------------------

	/** Returns the custom shape prior (if any). */
	public Custom getCustomPriorShape() {
		return customPriorShape_;
	}

	// ----------------------------------------------------------------------------

	/**
	 * Returns the number of scales provided by the method
	 * <code>getScales()</code>.
	 */
	public int getNumScales() {
		return 2;
	}

	// ----------------------------------------------------------------------------

	/**
	 * Returns a new container with the information of the execution parameters
	 * of the snake.
	 */
	public ESnakeParametersSD getSnakeParameters() {
		return new ESnakeParametersSD(detectType_, M_, Level_, basisFunctionType_, mScaling_,
				energyType_, alpha_, priorShapeType_,
				maxLife_, immortal_, multiresolutionType_);
	}

	// ----------------------------------------------------------------------------

	/** Retrieves the orientation of the curve determined by the snake. */
	public ESnakeOrientationSD getOrientation() {
		if (area_ >= 0) {
			return ESnakeOrientationSD.CLOCKWISE;
		} else {
			return ESnakeOrientationSD.COUNTERCLOCKWISE;
		}
	}

	// ----------------------------------------------------------------------------

	/**
	 * Returns <code>false</code> if the snake reaches an invalid configuration.
	 * It can be related to ill-posed configuration of the points or related to
	 * the image boundaries.
	 */
	public boolean isValid() {
		return valid_;
	}

	// ----------------------------------------------------------------------------

	/** Sets a custom prior-shape to the snake. */
	public void setCustomPriorShape(Custom customPriorShape) {
		customPriorShape_ = customPriorShape;
	}

	// ----------------------------------------------------------------------------

	/** Sets the E-Snake execution parameters. */
	public void setSnakeParameters(ESnakeParametersSD parameters) {
		detectType_ = parameters.getDetectType();
		int M = parameters.getM();
		if (M != M_) {
			setNumNodes(1.5, M);
		}

		int Level = parameters.getSubdivisionLevel();
		if (Level != Level_) {
			setLevel(Level);
		}

		int m = parameters.getMScalingFactor();
		if (m != mScaling_) {
			setmScaling(m);
		}

		MSub_ = (int) Math.pow(mScaling_, Level_) * M;

		energyType_ = parameters.getEnergyType();
		alpha_ = parameters.getAlpha();
		ESnakePriorShapeTypeSD priorShapeType = parameters.getPriorShapeType();
		if (priorShapeType != priorShapeType_) {
			setPriorShape(priorShapeType);
		}
//		ShapeSpaceTypeSD shapeSpaceType = parameters.getShapeSpaceType();
//		if (shapeSpaceType != shapeSpaceType_) {
//			shapeSpaceType_ = shapeSpaceType;
			// TODO: check wheather if-condition can be removed,
			// i.e. directly call shapeSpaceType_ =
			// parameters.getShapeSpaceType();
		//}
		BSplineBasisType basisFunctionType = parameters.getBasisFunctionType();
		if (basisFunctionType != basisFunctionType_) {
			basisFunctionType_ = basisFunctionType;
			}
		
		BASIS_FUNCTION=basisFunctionType_;
		
		MultiresolutionTypeSD multiresolutionType = parameters.getMultiresolutionType();
		if (multiresolutionType != multiresolutionType_) {
			multiresolutionType_ = multiresolutionType;
			}
		
		//beta_ = parameters.getBeta();
		maxLife_ = parameters.getMaxLife();
		immortal_ = parameters.isImmortal();
		reviveSnake();
	}

	// ----------------------------------------------------------------------------
	// GEOMETRIC TRANSFORMATION METHODS

	/**
	 * Modifies the control points of a snake in order to scale the snake
	 * horizontally by a given factor.
	 */
	public void dilateX(double a) {
		Point2D.Double g = getCentroid();
		Snake2DNode[] coef = getNodes();
		for (int i = 0; i < getNumNodes(); i++) {
			coef[i].x = a * coef[i].x + (1 - a) * g.x;
		}
		setNodes(coef);
	}

	// ----------------------------------------------------------------------------

	/**
	 * Modifies the control points of a snake in order to scale the snake
	 * vertically by a given factor.
	 */
	public void dilateY(double b) {
		Point2D.Double g = getCentroid();
		Snake2DNode[] coef = getNodes();
		for (int i = 0; i < getNumNodes(); i++) {
			coef[i].y = b * coef[i].y + (1 - b) * g.y;
		}
		setNodes(coef);
	}

	// ============================================================================
	// PRIVATE METHODS

	/** Initializes all look-up tables of the class. */
	protected void buildLUTs() {
		double currentVal;
		bSplineLUT_ = new double[NR_];
		bSplineDerivativeLUT_ = new double[NR_];
		sinLUT_ = new double[MR_];
		cosLUT_ = new double[MR_];

		double PI2MR = PI2M_ / nSamplesPerSegment_;
		for (int i = 0; i < MR_; i++) {
			sinLUT_[i] = Math.sin(PI2MR * i + 1.5);
			cosLUT_[i] = Math.cos(PI2MR * i + 1.5);
		}

		for (int i = 0; i < NR_; i++) {
			currentVal = (double) i / (double) nSamplesPerSegment_;
			switch (BASIS_FUNCTION) {
			case ESPLINE3:
				bSplineLUT_[i] = BSplineBasis.ESpline3(currentVal, PI2M_);
				break;
			case ESPLINE4:
				bSplineLUT_[i] = BSplineBasis.ESpline4(currentVal, PI2M_);
				break;
			case LINEARBSPLINE:
				bSplineLUT_[i] = BSplineBasis.LinearBSpline(currentVal);
				break;
			case QUADRATICBSPLINE:
				bSplineLUT_[i] = BSplineBasis.QuadraticSpline(currentVal);
				break;
			case CUBICBSPLINE:
				bSplineLUT_[i] = BSplineBasis.CubicBSpline(currentVal);
				break;
			case MSPLINE:
				bSplineLUT_[i] = BSplineBasis.MSpline(currentVal, PI2M_);
				break;
			case DESLAURIERSDUBUC:
				bSplineLUT_[i] = BSplineBasis.QuadraticSpline(currentVal);
				break;
			case GDESLAURIERSDUBUC:
				bSplineLUT_[i] = BSplineBasis.ESpline3(currentVal, PI2M_);
				break;
			case GDESLAURIERSDUBUC2:
				bSplineLUT_[i] = BSplineBasis.ESpline3(currentVal, PI2M_);
				break;
			}
			bSplineDerivativeLUT_[i] = BSplineBasis.ESpline3_Prime(currentVal,
					PI2M_);
		}

		int qSize = 2 * BSplineBasis.ESPLINE3SUPPORT - 1;
		bSplineAutocorrelationLUT_ = new double[qSize];
		for (int i = 0; i < qSize; i++) {
			bSplineAutocorrelationLUT_[i] = BSplineBasis.correlationESpline(i
					- BSplineBasis.ESPLINE3SUPPORT + 1, PI2M_);
		}

		for (int i = 0; i < M_; i++) {
			for (int j = 0; j < M_; j++) {
				qLUT_[i][j] = computeQ(i, j);
			}
		}
	}

	// ----------------------------------------------------------------------------

	/** Initializes the snake control points with a predefined shape. */
	private void initializeDefaultShape() {
		coef_ = new Snake2DNode[M_];
		if (initialContour_ != null) {
			if (initialContour_.npoints != 0) {
				Point2D.Double[] resampledContour = Geometry2D
						.arcLengthResampling(initialContour_, M_);
				coef_ = getSplineKnots(resampledContour, M_);
				return;
			}
		}

		coef_ = priorShape_.getNodes(M_);

		double xMin = Double.MAX_VALUE;
		double yMin = Double.MAX_VALUE;

		double xMax = -Double.MAX_VALUE;
		double yMax = -Double.MAX_VALUE;

		double xg = 0;
		double yg = 0;
		for (Snake2DNode element : coef_) {
			double x = element.x;
			double y = element.y;
			xg += x;
			yg += y;
			if (x > xMax) {
				xMax = x;
			}
			if (y > yMax) {
				yMax = y;
			}
			if (x < xMin) {
				xMin = x;
			}
			if (y < yMin) {
				yMin = y;
			}
		}
		xg /= M_;
		yg /= M_;

		double shapeWidth = xMax - xMin;
		double shapeHeight = yMax - yMin;

		double scale = 1.0 / Math.max(shapeWidth / imageLUTs_.getImageWidth(),
				shapeHeight / imageLUTs_.getImageHeight());

		for (int i = 0; i < coef_.length; i++) {
			coef_[i].x = 0.33 * scale * (coef_[i].x - xg)
					+ imageLUTs_.getImageWidth() / 2.0;
			coef_[i].y = 0.33 * scale * (coef_[i].y - yg)
					+ imageLUTs_.getImageHeight() / 2.0;
		}

	}

	// ----------------------------------------------------------------------------

	/**
	 * Computes the points of the contour of the snake from the control points
	 * and stores it in a LUT.
	 */
	protected void updateSnakeSkin() {
		int index;

		int xmin = imageLUTs_.getImageWidth() - 1;
		int xmax = 0;
		int ymin = imageLUTs_.getImageHeight() - 1;
		int ymax = 0;

		double aux, aux2, xPrimeVal, yPrimeVal, xPosVal, yPosVal;
		for (int i = 0; i < MR_; i++) {

			xPosVal = 0.0;
			yPosVal = 0.0;
			xPrimeVal = 0.0;
			yPrimeVal = 0.0;
			for (int k = 0; k < M_; k++) {
				index = i - k * nSamplesPerSegment_;

				while (index < 0) {
					index += MR_;
				}
				while (index >= MR_) {
					index -= MR_;
				}
				if (index >= NR_) {
					continue;
				} else {
					aux = bSplineLUT_[index];
					aux2 = bSplineDerivativeLUT_[index];
				}
				xPosVal += coef_[k].x * aux;
				yPosVal += coef_[k].y * aux;
				xPrimeVal += coef_[k].x * aux2;
				yPrimeVal += coef_[k].y * aux2;

			}
			xSnakeContour_[i] = xPosVal;
			ySnakeContour_[i] = yPosVal;

			xSnakeTangentVector_[i] = xPrimeVal;
			ySnakeTangentVector_[i] = yPrimeVal;
		}

		// Subdivision
		//stationary schemes
		if(BASIS_FUNCTION == BSplineBasisType.LINEARBSPLINE){
			subdivision();
		}
		else if(BASIS_FUNCTION == BSplineBasisType.QUADRATICBSPLINE){
			subdivision();
		}
		else if(BASIS_FUNCTION == BSplineBasisType.CUBICBSPLINE){
			subdivision();
		}
		else if(BASIS_FUNCTION == BSplineBasisType.DESLAURIERSDUBUC){
			subdivisionDeslaurierDubuc();
		}
		//non-stationary schemes
		else if(BASIS_FUNCTION == BSplineBasisType.GDESLAURIERSDUBUC){
			subdivisionNSDeslaurierDubuc();
		}
		else if(BASIS_FUNCTION == BSplineBasisType.GDESLAURIERSDUBUC2){
			subdivisionNSDeslaurierDubuc2();
		}
		else{//i.e. if ESPLINE3
			nonStationnarySubdivision();
		}
		
		for (int i = 0; i < MSub_; i++) {
			xSnakeContourSubdivision_[i] = subdivCoef_[i].x;
			ySnakeContourSubdivision_[i] = subdivCoef_[i].y;

			xSnakeTangentVectorSubdivision_[i] = dersubdivCoef_[i].x;
			ySnakeTangentVectorSubdivision_[i] = dersubdivCoef_[i].y;

			// update the bounding box
			if ((int) Math.floor(xSnakeContourSubdivision_[i]) < xmin) {
				xmin = (int) Math.floor(xSnakeContourSubdivision_[i]);
			}
			if ((int) Math.ceil(xSnakeContourSubdivision_[i]) > xmax) {
				xmax = (int) Math.ceil(xSnakeContourSubdivision_[i]);
			}
			if ((int) Math.floor(ySnakeContourSubdivision_[i]) < ymin) {
				ymin = (int) Math.floor(ySnakeContourSubdivision_[i]);
			}
			if ((int) Math.ceil(ySnakeContourSubdivision_[i]) > ymax) {
				ymax = (int) Math.ceil(ySnakeContourSubdivision_[i]);
			}

		}

		//saveSnakesTXT();
		
		snakeContourBoundingBox_.x = xmin;
		snakeContourBoundingBox_.y = ymin;
		snakeContourBoundingBox_.width = xmax - xmin + 1;
		snakeContourBoundingBox_.height = ymax - ymin + 1;
	}

	// ----------------------------------------------------------------------------

	/**
	 * Computes the points of the snake shell used in the computation of the
	 * region energy and stores it in a LUT. The shell is obtained by an
	 * isotropic scaling of the snake contour over the center of gravity of the
	 * snake such that the shell encloses the two times more area than the
	 * snake.
	 */
	private void updateSnakeShell() {

		Point2D.Double snakeCentroid = getCentroidSubdivision();

		int xmin = imageLUTs_.getImageWidth() - 1;
		int xmax = 0;
		int ymin = imageLUTs_.getImageHeight() - 1;
		int ymax = 0;

		for (int i = 0; i < MSub_; i++) {
			xSnakeShellContour_[i] = SQRT2
					* (xSnakeContourSubdivision_[i] - snakeCentroid.x)
					+ snakeCentroid.x;
			ySnakeShellContour_[i] = SQRT2
					* (ySnakeContourSubdivision_[i] - snakeCentroid.y)
					+ snakeCentroid.y;
			if ((int) Math.floor(xSnakeShellContour_[i]) < xmin) {
				xmin = (int) Math.floor(xSnakeShellContour_[i]);
			}
			if ((int) Math.ceil(xSnakeShellContour_[i]) > xmax) {
				xmax = (int) Math.ceil(xSnakeShellContour_[i]);
			}
			if ((int) Math.floor(ySnakeShellContour_[i]) < ymin) {
				ymin = (int) Math.floor(ySnakeShellContour_[i]);
			}
			if ((int) Math.ceil(ySnakeShellContour_[i]) > ymax) {
				ymax = (int) Math.ceil(ySnakeShellContour_[i]);
			}
		}
		snakeShellBoundingBox_.x = xmin;
		snakeShellBoundingBox_.y = ymin;
		snakeShellBoundingBox_.width = xmax - xmin + 1;
		snakeShellBoundingBox_.height = ymax - ymin + 1;
	}

	// ----------------------------------------------------------------------------

	/**
	 * Modifies the value of the <code>valid</code> flag depending if the snake
	 * is in a ill-conditioned configuration or not.
	 */
	protected void validateSnake() {

		for (int i = 0; i < coef_.length; i++) {
			if (Double.isNaN(coef_[i].x))
				alive_ = false;
		}

		controlPolygonSelfIntersects_ = false;
		if (!isSnakeInBoundingBox()) {
			valid_ = false;
		} else if (isSnakeIntersected()) {
			valid_ = false;
		} else {
			valid_ = true;
		}
	}

	// ----------------------------------------------------------------------------

	/**
	 * Computes the area of the curve defined by the snake, the enclosing
	 * ellipse and the area ratio.
	 */
	private void updateArea() {
		double area = 0.0;
		int l_p;
		for (int k = 0; k < M_; k++) {
			int kN = k + BSplineBasis.ESPLINE3SUPPORT;
			for (int l = k - BSplineBasis.ESPLINE3SUPPORT + 1; l < kN; l++) {
				l_p = l;
				while (l_p < 0) {
					l_p += M_;
				}

				while (l_p >= M_) {
					l_p -= M_;
				}

				area += coef_[k].y * coef_[l_p].x
						* bSplineAutocorrelationLUT_[kN - l - 1];
			}
		}

		area_ = area;
	}

	// ----------------------------------------------------------------------------
	// ENERGY METHODS

	/** Computes the contour energy. */
	private double computeContourEnergy() {
		double energy = 0.0;
		
		double[] gradientXImageDataArray = imageLUTs_.getGradientXImageDataArray();
		double[] gradientYImageDataArray = imageLUTs_.getGradientYImageDataArray();

		double fgradX_val, fgradY_val;
		
		for (int i = 0; i < MSub_; i++) {
			fgradX_val=getPixelInterpolation(gradientXImageDataArray, subdivCoef_[i]);
			
			fgradY_val=getPixelInterpolation(gradientYImageDataArray, subdivCoef_[i]);
			
			energy-=(fgradX_val*ySnakeTangentVectorSubdivision_[i]-fgradY_val*xSnakeTangentVectorSubdivision_[i])/Math.pow(mScaling_, Level_);
		}

		//System.out.println("energy2="+energy);

		return energy;
	}
	
	
	private double computeRidgeEnergy() {
		double energy = 0.0;

		double[] merit = imageLUTs_.getMerit();
		double[] ridgedx = imageLUTs_.getRidgedx();
		double[] ridgedy = imageLUTs_.getRidgedy();

		double ridgedx_val, ridgedy_val, merit_val;

		for (int i = 0; i < MSub_; i++) {
			ridgedx_val = getPixelInterpolation(ridgedx, subdivCoef_[i]);
			ridgedy_val = getPixelInterpolation(ridgedy, subdivCoef_[i]);
			merit_val = getPixelInterpolation(merit, subdivCoef_[i]);
	
//			 energy-=merit_val*1000*(Math.abs(ridgedx_val*ySnakeTangentVectorSubdivision_[i])
//					 +Math.abs(ridgedy_val*(-1)*xSnakeTangentVectorSubdivision_[i]))
//					 /Math.pow(mScaling_, Level_);
			
			double normTangentVector = 
					Math.sqrt(xSnakeTangentVectorSubdivision_[i] 
							* xSnakeTangentVectorSubdivision_[i] 
							+ ySnakeTangentVectorSubdivision_[i] 
							* ySnakeTangentVectorSubdivision_[i]);
	
			if (normTangentVector == 0)
				normTangentVector = 1.0;
	
			double cosValue = 
					Math.abs(ridgedx_val * ySnakeTangentVectorSubdivision_[i]
							+ (-1) * ridgedy_val 
							* xSnakeTangentVectorSubdivision_[i])
					/ normTangentVector;
			energy -= merit_val * 1000 * cosValue / Math.pow(mScaling_, Level_);
			
			//energy-=merit_val*1000; 
		}

//		System.out.println("Ridge energy = " + energy);

		return energy;
	}
	
	// ----------------------------------------------------------------------------     
	/* Return the value of the image f at the pixel point **/
	private double getPixelInterpolation(double[] f, Snake2DNode points){
		
		//bilinear interpolation (wikipedia)
		int imageWidth=imageLUTs_.getImageWidth();
		
		int x1= (int) Math.floor(points.x);
		int y1= (int) Math.floor(points.y);
		
		int x2=(int) (points.x+1);
		int y2=(int) (points.y+1);
		
		double c1= (x2-points.x)/(x2-x1);
		double c2= (points.x-x1)/(x2-x1);
		
		double c00=c1*f[x1+y1*imageWidth]+c2*f[x2+y1*imageWidth];
		double c01=c1*f[x1+y2*imageWidth]+c2*f[x2+y2*imageWidth];
		
		double c3= (y2-points.y)/(y2-y1);
		double c4= (points.y-y1)/(y2-y1);
		
		double c=c3*c00+c4*c01;
		
		return c;
	}
	
//	private double computeContourEnergy() {
//		double energy = 0.0;
//		double fuyLap_val;
//		int x1, x2, y1, y2;
//		double DeltaX1, DeltaX2, DeltaY1;
//
//		int sign = 0;
//		switch (getOrientation()) {
//		case CLOCKWISE:
////			System.out.println("(clockwise)");
//			sign = 1;
//			break;
//		case COUNTERCLOCKWISE:
////			System.out.println("(counter clockwise)");
//			sign = -1;
//			break;
//		}
//
//		int imageWidth = imageLUTs_.getImageWidth();
//		int imageHeight = imageLUTs_.getImageHeight();
//		double[] filteredImageDataArray = imageLUTs_
//				.getFilteredImageDataArray();
//		double[] preintegratedFilteredImageDataArray = imageLUTs_
//				.getPreintegratedFilteredImageDataArray();
//		
//		for (int i = 0; i < MSub_; i++) {
//			x1 = (int) Math.floor(xSnakeContourSubdivision_[i] - 0.5);
//			y1 = (int) Math.floor(ySnakeContourSubdivision_[i] - 0.5);
//
//			if (x1 < 1) {
//				x1 = 1;
//			} else if (x1 > imageWidth - 2) {
//				x1 = imageWidth - 2;
//			}
//
//			if (y1 < 1) {
//				y1 = 1;
//			} else if (y1 > imageHeight - 2) {
//				y1 = imageHeight - 2;
//			}
//
//			x2 = x1 + 1;
//			y2 = y1 + 1;
//
//			DeltaX1 = (xSnakeContourSubdivision_[i] - 0.5) - x1;
//			DeltaY1 = (ySnakeContourSubdivision_[i] - 0.5) - y1;
//			DeltaX2 = x2 - (xSnakeContourSubdivision_[i] - 0.5);
//
//			fuyLap_val = preintegratedFilteredImageDataArray[x1 + imageWidth
//					* (y1 - 1)]
//					* DeltaX2
//					+ preintegratedFilteredImageDataArray[x2 + imageWidth
//							* (y1 - 1)] * DeltaX1;
//			fuyLap_val += 0.5 * ((filteredImageDataArray[x1 + imageWidth * y1]
//					* DeltaX2 + filteredImageDataArray[x2 + imageWidth * y1]
//					* DeltaX1) + (DeltaY1 * (((filteredImageDataArray[x1
//					+ imageWidth * y1]
//					* DeltaX2 + filteredImageDataArray[x2 + imageWidth * y1]
//					* DeltaX1) * (2 - DeltaY1)) + ((filteredImageDataArray[x1
//					+ imageWidth * y2]
//					* DeltaX2 + filteredImageDataArray[x2 + imageWidth * y2]
//					* DeltaX1) * DeltaY1))));
//
//			energy += (1/(Math.pow(mScaling_, Level_)))*(fuyLap_val * xSnakeTangentVectorSubdivision_[i]);
////			System.out.println("computing energy = " + energy + " (" + i + ")");
////			System.out.println("xSnakeTangentVectorSubdivision_ = " + xSnakeTangentVectorSubdivision_[i] + " (" + i + ")");
//		}
//		energy = energy * sign;
//
//		return energy;
//	}

	// ----------------------------------------------------------------------------

	/** Computes the prior shape energy. */
	private double computePriorShapeEnergy() {

		if (shapeSpaceType_ == ShapeSpaceTypeSD.AFFINE) {
			return shapeProjectorAffine_.orthoProjectionDistanceAffine(coef_);
		} else if (shapeSpaceType_ == ShapeSpaceTypeSD.SIMILARITY) {
			return shapeProjectorSimilarity_
					.orthoProjectionDistanceSimilarity(coef_);
		}
		return 0;
	}

	// ----------------------------------------------------------------------------

	/** Computes the region energy. */
	private double computeRegionEnergy() {

		double[] imageDataArray = imageLUTs_.getImageDataArray();
		double[] preintegratedImageDataArray = imageLUTs_
				.getPreintegratedImageDataArray();

		double internalContribution_ = computeLineIntegral(imageDataArray,
				preintegratedImageDataArray, xSnakeContourSubdivision_,
				ySnakeContourSubdivision_, xSnakeTangentVectorSubdivision_);

		double externalContribution_ = SQRT2
				* computeLineIntegral(imageDataArray,
						preintegratedImageDataArray, xSnakeShellContour_,
						ySnakeShellContour_, xSnakeTangentVectorSubdivision_);

		double regionEnergy =(1/(Math.pow(mScaling_, Level_)))* ((externalContribution_ - internalContribution_) - internalContribution_)
				/ area_;

		// System.out.println("Region energy : "+regionEnergy +
		// "Area : "+area_);
		return regionEnergy;
	}

	// ----------------------------------------------------------------------------

	/**
	 * The purpose of this method is computing the distance of the intersected
	 * node in the snake polygon w.r.t corresponding edge.
	 */
	private double computeSelfIntersectionEnergy() {
		if (controlPolygonSelfIntersects_) {
			Point2D.Double p = coef_[badNodeIndex_];
			Point2D.Double v = coef_[badEdgeIndex_];
			Point2D.Double w = coef_[(badEdgeIndex_ + 1) % M_];

			double l2 = v.distanceSq(w);
			if (l2 == 0.0) {
				return p.distance(v);
			} else {
				double t = ((p.x - v.x) * (w.x - v.x) + (p.y - v.y)
						* (w.y - v.y))
						/ l2;
				if (t < 0.0) {
					return p.distance(v);
				} else if (t > 1.0) {
					return p.distance(w);
				} else {
					double projectionX = v.x + t * (w.x - v.x);
					double projectionY = v.y + t * (w.y - v.y); // Projection
																// falls
					return p.distance(projectionX, projectionY);
				}
			}
		} else {
			return 0;
		}
	}

	// ----------------------------------------------------------------------------

	/** Computes a line integral over a closed curve. */
	private double computeLineIntegral(double[] f, double[] fy, double[] x,
			double y[], double[] xp) {
		double fy_val;
		int x0, x1, y0, y1;
		double DeltaX1, DeltaX2, DeltaY1, DeltaY2;

		int imageWidth = imageLUTs_.getImageWidth();

		double lineIntegral = 0.0;
		for (int i = 0; i < MSub_; i++) {
			x0 = (int) Math.floor(x[i] - 0.5);
			y0 = (int) Math.floor(y[i] - 0.5);

			x1 = x0 + 1;
			y1 = y0 + 1;

			DeltaX1 = (x[i] - 0.5) - x0;
			DeltaY1 = (y[i] - 0.5) - y0;
			DeltaX2 = x1 - (x[i] - 0.5);

			DeltaX1 = (x[i] - 0.5) - x0;
			DeltaY1 = (y[i] - 0.5) - y0;
			DeltaX2 = 1 - DeltaX1;
			DeltaY2 = 1 - DeltaY1;

			fy_val = DeltaX2
					* (fy[x0 + imageWidth * y0] - 0.5 * f[x0 + imageWidth * y0]
							* DeltaY2 * DeltaY2 + 0.5 * f[x0 + imageWidth * y1]
							* DeltaY1 * DeltaY1)
					+ DeltaX1
					* (fy[x1 + imageWidth * y0] - 0.5 * f[x1 + imageWidth * y0]
							* DeltaY2 * DeltaY2 + 0.5 * f[x1 + imageWidth * y1]
							* DeltaY1 * DeltaY1);
			lineIntegral -= fy_val * xp[i];
		}

		return lineIntegral;
	}

	// ----------------------------------------------------------------------------

	/**
	 * $\int_{0}^{M}\,\varphi_{M}(t-k_2)\,\varphi'_{M}(t-k_1)\,\mathrm{d}t$.
	 */
	private double computeQ(int k1, int k2) {
		double val = 0;
		for (int i = 0; i < MR_; i++) {

			int index1 = i - k2 * nSamplesPerSegment_;

			while (index1 < 0) {
				index1 += MR_;
			}
			while (index1 >= MR_) {
				index1 -= MR_;
			}
			int index2 = i - k1 * nSamplesPerSegment_;

			while (index2 < 0) {
				index2 += MR_;
			}
			while (index2 >= MR_) {
				index2 -= MR_;
			}
			val += BSplineBasis.ESpline3((double) index1
					/ (double) nSamplesPerSegment_, PI2M_)
					* BSplineBasis.ESpline3_Prime((double) index2
							/ (double) nSamplesPerSegment_, PI2M_);
		}
		val /= nSamplesPerSegment_;
		return val;
	}

	// ----------------------------------------------------------------------------
	// SELF-INTERSECTION METHODS

	/**
	 * This method check if the snake has been intersected or not. It returns
	 * <code>false</code> when the snake is not bended.
	 */
	private boolean isSnakeIntersected() {
		boolean snakeIntersection = false;
		if (isPolygonIntersected(null)) {
			controlPolygonSelfIntersects_ = true;
			int count = 0;
			for (int i = 0; i < M_; i++) {
				if (!isPolygonIntersected(i)) {
					for (int[] element : edgeVsEdge_) {
						if (element[0] == i) {
							count++;
						}
						if (element[0] == presentPreviousNode(i, M_)) {
							count++;
						}
					}
				}
			}

			int[] IntersectedNode_ = new int[count];
			int[] IntersectedLine_ = new int[count];
			count = 0;
			for (int i = 0; i < M_; i++) {
				if (!isPolygonIntersected(i)) {
					for (int[] element : edgeVsEdge_) {
						if (element[0] == i) {
							IntersectedNode_[count] = i;
							IntersectedLine_[count] = element[1];
							count++;
						}
						if (element[0] == presentPreviousNode(i, M_)) {
							IntersectedNode_[count] = i;
							IntersectedLine_[count] = element[1];
							count++;
						}
					}
				}
			}
			for (int i = 0; i < count && !snakeIntersection; i++) {
				badNodeIndex_ = IntersectedNode_[i];
				badEdgeIndex_ = IntersectedLine_[i];
				double intersectedNodeCondvsLine = ((coef_[IntersectedNode_[i]].y - coef_[IntersectedLine_[i]].y)
						* (coef_[presentNextNode(IntersectedLine_[i], M_)].x - coef_[IntersectedLine_[i]].x) - (coef_[IntersectedNode_[i]].x - coef_[IntersectedLine_[i]].x)
						* (coef_[presentNextNode(IntersectedLine_[i], M_)].y - coef_[IntersectedLine_[i]].y));
				for (int k = 0; k < MR_; k++) {
					if (intersectedNodeCondvsLine
							* ((ySnakeContour_[k] - coef_[IntersectedLine_[i]].y)
									* (coef_[presentNextNode(
											IntersectedLine_[i], M_)].x - coef_[IntersectedLine_[i]].x) - (xSnakeContour_[k] - coef_[IntersectedLine_[i]].x)
									* (coef_[presentNextNode(
											IntersectedLine_[i], M_)].y - coef_[IntersectedLine_[i]].y)) > 0) {

						snakeIntersection = true;
					}
				}
			}
		}
		return snakeIntersection;
	}

	// ----------------------------------------------------------------------------

	/**
	 * The purpose of this method is to present the next node in the circular
	 * structure.
	 */
	private int presentNextNode(int a, int nodeNumber) {
		int NextNode_ = 0;
		if (a < nodeNumber - 1) {
			NextNode_ = a + 1;
		} else {
			NextNode_ = 0;
		}
		return NextNode_;
	}

	// ----------------------------------------------------------------------------
	@Override
	public boolean isLocalRefinement() {
		// TODO Auto-generated method stub
		return false;
	}
		
	// ----------------------------------------------------------------------------

	/**
	 * The purpose of this method is to present the previous node in the
	 * circular structure.
	 */
	private int presentPreviousNode(int a, int nodeNumber) {
		int nextNode = 0;
		if (a > 0) {
			nextNode = a - 1;
		} else {
			nextNode = nodeNumber - 1;
		}
		return nextNode;
	}

	// ----------------------------------------------------------------------------

	/**
	 * The purpose of this method is to check that there is any intersection or
	 * not in the polygon. It returns <code>false</code> whenever there is no
	 * line intersection in the polygon.
	 */
	private boolean isPolygonIntersected(Integer node) {
		double[][] vertices = removeNodePolygon(node);
		int[][] edgevsNode = new int[vertices.length][vertices.length];
		/* Compute the edgevsNode array */
		for (int i = 0; i < vertices.length; i++) {
			for (int j = 0; j < vertices.length; j++) {
				if ((vertices[i][1] - vertices[j][1])
						* (vertices[presentNextNode(j, vertices.length)][0] - vertices[j][0]) > (vertices[i][0] - vertices[j][0])
						* (vertices[presentNextNode(j, vertices.length)][1] - vertices[j][1])) {
					edgevsNode[j][i] = 1;
				} else if ((vertices[i][1] - vertices[j][1])
						* (vertices[presentNextNode(j, vertices.length)][0] - vertices[j][0]) < (vertices[i][0] - vertices[j][0])
						* (vertices[presentNextNode(j, vertices.length)][1] - vertices[j][1])) {
					edgevsNode[j][i] = -1;
				} else {
					edgevsNode[j][i] = 0;
				}
			}
		}

		/*
		 * 2D array which is applied to find the condition of the edges w .r.t.
		 * each other
		 */
		int[][] edgeCond = new int[vertices.length][vertices.length];
		for (int i = 0; i < vertices.length; i++) {
			for (int j = 0; j < vertices.length; j++) {
				edgeCond[i][j] = edgevsNode[i][j]
						* edgevsNode[i][presentNextNode(j, vertices.length)];
			}
		}

		int count = 0;
		for (int i = 0; i < vertices.length; i++) {
			for (int j = 0; j < vertices.length; j++) {
				if (edgeCond[i][j] == -1 && edgeCond[j][i] == -1) {
					count++;
				}
			}
		}
		int[][] edgevsEdge = new int[count][2];
		if (node == null) {
			edgeVsEdge_ = new int[count][2];
		}

		count = 0;
		for (int i = 0; i < vertices.length; i++) {
			for (int j = 0; j < vertices.length; j++) {
				if (edgeCond[i][j] == -1 && edgeCond[j][i] == -1) {
					edgevsEdge[count][0] = i;
					edgevsEdge[count][1] = j;
					if (node == null) {
						edgeVsEdge_[count][0] = i;
						edgeVsEdge_[count][1] = j;
					}
					count++;
				}
			}
		}
		return count != 0;
	}

	// ----------------------------------------------------------------------------

	/**
	 * The purpose of this method is to remove the specific node of the polygon
	 * and return the others as a new polygon
	 */
	private double[][] removeNodePolygon(Integer Indx) {
		double[][] outputPolygon;
		if (Indx == null) {
			outputPolygon = new double[M_][2];
			for (int i = 0; i < M_; i++) {
				outputPolygon[i][0] = coef_[i].x;
				outputPolygon[i][1] = coef_[i].y;
			}
		} else {
			outputPolygon = new double[M_ - 1][2];
			int count = 0;
			int NextNode_ = Indx;
			while (count < M_ - 1) {
				NextNode_ = presentNextNode(NextNode_, M_);
				outputPolygon[count][0] = coef_[NextNode_].x;
				outputPolygon[count][1] = coef_[NextNode_].y;
				count++;
			}
		}
		return outputPolygon;
	}

	// ----------------------------------------------------------------------------

	/**
	 * This method check if the snake is in the bounding box or not. It returns
	 * <code>false</code> when the snake touches the boundaries.
	 */
	private boolean isSnakeInBoundingBox() {
		if (snakeContourBoundingBox_.x < 1) {
			return false;
		} else if (snakeContourBoundingBox_.x + snakeContourBoundingBox_.width > imageLUTs_
				.getImageWidth() - 1) {
			return false;
		} else if (snakeContourBoundingBox_.y < 1) {
			return false;
		} else if (snakeContourBoundingBox_.y + snakeContourBoundingBox_.height > imageLUTs_
				.getImageHeight() - 1) {
			return false;
		} else if (energyType_ == ESnakeEnergyTypeSD.REGION
				|| energyType_ == ESnakeEnergyTypeSD.MIXTURE) {
			if (snakeShellBoundingBox_.x < 1) {
				return false;
			} else if (snakeShellBoundingBox_.x + snakeShellBoundingBox_.width > imageLUTs_
					.getImageWidth() - 1) {
				return false;
			} else if (snakeShellBoundingBox_.y < 1) {
				return false;
			} else if (snakeShellBoundingBox_.y + snakeShellBoundingBox_.height > imageLUTs_
					.getImageHeight() - 1) {
				return false;
			}
		}
		return true;
	}

	// ----------------------------------------------------------------------------
	// RESAMPLING METHODS

	private Snake2DNode[] getSplineKnots(Point2D.Double[] contour, int M) {
		double[] knotsX = new double[M];
		double[] knotsY = new double[M];

		double b = 2.0 / 3.0;

		for (int i = 0; i < M; i++) {
			knotsX[i] = contour[i].x;
			knotsY[i] = contour[i].y;
		}

		double[] pole = { (-b + Math.sqrt(2 * b - 1)) / (1 - b) };
		knotsX = Filters.prescaledPeriodic(knotsX, pole);
		knotsY = Filters.prescaledPeriodic(knotsY, pole);

		Snake2DNode[] newCoeff = new Snake2DNode[M];
		for (int i = 0; i < M; i++) {
			newCoeff[i] = new Snake2DNode(knotsX[i], knotsY[i]);
		}

		return newCoeff;
	}

	// ----------------------------------------------------------------------------

	/** Assigns a prior-shape to the snake and builds the necessary projectors. */
	private void setPriorShape(ESnakePriorShapeTypeSD priorShapeType) {
		priorShapeType_ = priorShapeType;
		switch (priorShapeType_) {
		case NONE:
			priorShape_ = new Circle();
			break;
		case ROD:
			priorShape_ = new Rod();
			break;
		case BANANA:
			priorShape_ = new Banana();
			break;
		case BOOMERANG:
			priorShape_ = new Boomerang();
			break;
		case BRAINWHITE:
			priorShape_ = new BrainWhite();
			break;
		case CIRCLE:
			priorShape_ = new Circle();
			break;
		case CORPUSCALLOSUM:
			priorShape_ = new CorpusCallosum();
			break;
		case CROSS:
			priorShape_ = new Cross();
			break;
		case ELECTRICGUITAR:
			priorShape_ = new ElectricGuitar();
			break;
		case FEMUR:
			priorShape_ = new Femur();
			break;
		case FLY:
			priorShape_ = new Fly();
			break;
		case MOUSEORGAN:
			priorShape_ = new MouseOrgan();
			break;
		case SEMICIRCLE:
			priorShape_ = new Semicircle();
			break;
		case SQUARE:
			priorShape_ = new Square();
			break;
		case UPPERLIP:
			priorShape_ = new UpperLip();
			break;
		case CUSTOM:
			priorShape_ = customPriorShape_;
			break;
		default:
			break;
		}
		shapeProjectorAffine_ = shapeProjectorBuilder_
				.buildShapeProjectorAffine(priorShape_);
		shapeProjectorSimilarity_ = shapeProjectorBuilder_
				.buildShapeProjectorSimilarity(priorShape_);
	}

	// ----------------------------------------------------------------------------

	/** Updates the number of control points. */
	protected void setNumNodes(double initialSample, int newM) {
		double c = BSplineBasis.correlationOfTwoESpline(3.0 / newM,
				(2 * Math.PI) / newM, (2 * Math.PI) / newM, newM, newM);
		double b = BSplineBasis.correlationOfTwoESpline(4.0 / newM,
				(2 * Math.PI) / newM, (2 * Math.PI) / newM, newM, newM);
		double a = BSplineBasis.correlationOfTwoESpline(5.0 / newM,
				(2 * Math.PI) / newM, (2 * Math.PI) / newM, newM, newM);

		double alpha1 = (b + Math.sqrt(b * b - 4 * a * c + 8 * a * a))
				/ (2 * a);
		double alpha2 = (b - Math.sqrt(b * b - 4 * a * c + 8 * a * a))
				/ (2 * a);

		double z1 = (-alpha1 + Math.sqrt(alpha1 * alpha1 - 4)) / 2;
		if (Math.abs(z1) > 1) {
			z1 = 1 / z1;
		}

		double z2 = (-alpha2 + Math.sqrt(alpha2 * alpha2 - 4)) / 2;
		if (Math.abs(z2) > 1) {
			z2 = 1 / z2;
		}

		double[] invMatrix = new double[newM];
		double cz1 = 1 / ((1 - Math.pow(z1, newM)) * (z1 - 1 / z1) * a * (z1
				+ 1 / z1 - z2 - 1 / z2));
		double cz1i = -1
				/ ((1 - Math.pow(z1, -newM)) * (z1 - 1 / z1) * a * (z1 + 1 / z1
						- z2 - 1 / z2));
		double cz2 = -1
				/ ((1 - Math.pow(z2, newM)) * (z2 - 1 / z2) * a * (z1 + 1 / z1
						- z2 - 1 / z2));
		double cz2i = 1 / ((1 - Math.pow(z2, -newM)) * (z2 - 1 / z2) * a * (z1
				+ 1 / z1 - z2 - 1 / z2));
		double z1k = 1;
		double z2k = 1;
		for (int k = 0; k < newM; k++) {
			invMatrix[k] = cz1 * z1k + cz1i * (1 / z1k) + cz2 * z2k + cz2i
					* (1 / z2k);
			z1k *= z1;
			z2k *= z2;
		}

		Point2D.Double[] vec = new Point2D.Double[newM];
		double x, y;
		for (int k = 0; k < newM; k++) {
			x = 0.0;
			y = 0.0;
			for (int i = -BSplineBasis.ESPLINE3SUPPORT; i < M_
					+ BSplineBasis.ESPLINE3SUPPORT; i++) {
				x += coef_[(M_ + i) % M_].x
						* BSplineBasis.correlationOfTwoESpline(
								((3.0 + i) / M_ - (double) k / (double) newM),
								(2 * Math.PI) / M_, (2 * Math.PI) / newM, M_,
								newM);
				y += coef_[(M_ + i) % M_].y
						* BSplineBasis.correlationOfTwoESpline(
								((3.0 + i) / M_ - (double) k / (double) newM),
								(2 * Math.PI) / M_, (2 * Math.PI) / newM, M_,
								newM);
			}
			vec[k] = new Point2D.Double(x, y);
		}

		Snake2DNode[] newCoef = new Snake2DNode[newM];
		for (int k = 0; k < newM; k++) {
			x = 0.0;
			y = 0.0;
			for (int i = 0; i < newM; i++) {
				x += vec[i].x * invMatrix[(newM + i - k) % newM];
				y += vec[i].y * invMatrix[(newM + i - k) % newM];
			}
			newCoef[k] = new Snake2DNode(x, y);
		}

		coef_ = newCoef;
		M_ = newM;
		shapeProjectorBuilder_ = new ShapeProjectorBuilder(M_);
		initialize(false);
		alive_ = true;
	}

	// ----------------------------------------------------------------------------

	/** Updates the number of subdivision level. */
	private void setLevel(int newLevel) {
		Level_ = newLevel;
		initialize(false);
	}

	// ----------------------------------------------------------------------------

	/** Updates the number of subdivision level. */
	private void setmScaling(int new_m) {
		mScaling_ = new_m;
		initialize(false);
	}

	// ----------------------------------------------------------------------------

	/** Updates the number of subdivision level. */
	private void testDerivative(int N, double index) {
		double[] value;
		value = new double[2];
		value[0] = 0;
		value[1] = 0;

		int j = 0;

		for (int i = -N + 1; i < M_; i++) {
			if (i < 0)
				j = i + M_;
			else
				j = i;
			value[0] += coef_[j].x
					* BSplineBasis.computePolynomialDerivative(index - i, N);
			value[1] += coef_[j].y
					* BSplineBasis.computePolynomialDerivative(index - i, N);
		}

		System.out.println(value[0] + " " + value[1]);
	}

	// ----------------------------------------------------------------------------

	/** Computes the area of the curve defined by the snake. */
	protected void updateAreaSubdivision() {
		double area = 0.0, integral = 0.0;
		int index = 0, div = 1;

		for (int k = 0; k < M_; k++) {
			for (int n = 0; n < M_; n++) {
				div = (int) Math.ceil((n - k) / M_);
				index = (n - k) - div * M_;

				if (BASIS_FUNCTION == BSplineBasisType.QUADRATICBSPLINE) {
					integral = BSplineBasis.Order5BSpline(index + 3)
							- BSplineBasis.Order5BSpline(index - 1 + 3)
							+ BSplineBasis.Order5BSpline(index - M_ + 3)
							- BSplineBasis.Order5BSpline(index - M_ - 1 + 3)
							+ BSplineBasis.Order5BSpline(index + M_ + 3)
							- BSplineBasis.Order5BSpline(index + M_ - 1 + 3);
					
					area += coef_[k].x * coef_[n].y * integral;
				} 
				else if(BASIS_FUNCTION == BSplineBasisType.CUBICBSPLINE) {
					integral = BSplineBasis.Order7BSpline(index + 4)
							- BSplineBasis.Order7BSpline(index - 1 + 4)
							+ BSplineBasis.Order7BSpline(index - M_ + 4)
							- BSplineBasis.Order7BSpline(index - M_ - 1 + 4)
							+ BSplineBasis.Order7BSpline(index + M_ + 4)
							- BSplineBasis.Order7BSpline(index + M_ - 1 + 4);
					
					area += coef_[k].x * coef_[n].y * integral;
				}
				else if (BASIS_FUNCTION == BSplineBasisType.DESLAURIERSDUBUC) {
					//Method 1
//					double[] Der;
//					Der = new double[7];
//
//					Der[0]=0;
//					Der[1]=-0.0833;
//					Der[2]=0.6667;
//					Der[3]=0;
//					Der[4]=-0.6667;
//					Der[5]=0.0833;
//					Der[6]=0;
//					
//					double a;
//					if((index+3)<0 || (index+3)>6) a=0;
//					else a=Der[index+3];
//					
//					double b;
//					if((index+3+M_)<0 || (index+3+M_)>6) b=0;
//					else b=Der[index+3+M_];
//					
//					double c;
//					if((index+3-M_)<0 || (index+3-M_)>6) c=0;
//					else c=Der[index+3-M_];
//					
//					integral=a+b+c;
					
					//Method 2
					area=0;
					for(int l=0; l<MSub_;l++){ area+=Math.pow(mScaling_,Level_)*subdivCoef_[l].y*dersubdivCoef_[l].x; }
					
				}
				else if (BASIS_FUNCTION == BSplineBasisType.GDESLAURIERSDUBUC) {
					//Method 1
//					double[] Der;
//					Der = new double[11];
//
//					Der[0]=0;
//					Der[1]=0.0003;
//					Der[2]=0.0146;
//					Der[3]=-0.1451;
//					Der[4]=0.745;
//					Der[5]=0;
//					Der[6]=-0.745;
//					Der[7]=0.1451;
//					Der[8]=-0.0146;
//					Der[9]=-0.0003;
//					Der[10]=0;
//					
//					double a;
//					if((index+5)<0 || (index+5)>10) a=0;
//					else a=Der[index+5];
//					
//					double b;
//					if((index+5+M_)<0 || (index+5+M_)>10) b=0;
//					else b=Der[index+5+M_];
//					
//					double c;
//					if((index+5-M_)<0 || (index+5-M_)>10) c=0;
//					else c=Der[index+5-M_];
//					
//					integral=a+b+c;
					
					//Method 2
					area=0;
					for(int l=0; l<MSub_;l++){ area+=Math.pow(mScaling_,Level_)* subdivCoef_[l].y*dersubdivCoef_[l].x; }
				}
				else if (BASIS_FUNCTION == BSplineBasisType.GDESLAURIERSDUBUC2) {
					area=0;
					for(int l=0; l<MSub_;l++){ area+=Math.pow(mScaling_,Level_)*subdivCoef_[l].y*dersubdivCoef_[l].x; }
				}
				else{//i.e if ESPLINE3
					double lambda=0;
					double nb=(double) Math.PI/M_;
					double nb2=Math.sin(nb)/nb;
					
					lambda=Math.pow(nb2, -2);
					
					integral = BSplineBasis.ESpline5(index + 3,PI2M_,M_)
							- BSplineBasis.ESpline5(index - 1 + 3,PI2M_,M_)
							+ BSplineBasis.ESpline5(index - M_ + 3,PI2M_,M_)
							- BSplineBasis.ESpline5(index - M_ - 1 + 3,PI2M_,M_)
							+ BSplineBasis.ESpline5(index + M_ + 3,PI2M_,M_)
							- BSplineBasis.ESpline5(index + M_ - 1 + 3,PI2M_,M_);
					integral=integral*lambda*lambda;
					
					area += coef_[k].x * coef_[n].y * integral;
				}
			}
		}

		area_ = -area;
		//System.out.println("areaSubdiv="+area_);

		
//		 double area2=0.0;
//		  
//		 for(int k=0; k<MSub_;k++){
//		  area2+=subdivCoef_[k].y*dersubdivCoef_[k].x; }
//		 System.out.println("areaSubdivision=" + area_ + "areaSubdivision2=" +
//		 area2);
		 
	}
	
	@Override
	public void localRefinementCoef(int index){
		// TODO Auto-generated method stub
	}
	
	@Override
	public int getRefinementFactor() {
		return 0;
	}

	public int getIndexRefinement() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public void setIndexRefinement(int index) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public boolean isLocalRefinementAction() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public int getFilterSize() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public java.awt.geom.Point2D.Double getCentroidRefinement() {
		// TODO Auto-generated method stub
		return null;
	}
	
	// ----------------------------------------------------------------------------

	/** Computes the Jaccard between a goldstandard (spline snake) (1) and the result of the subdivision snake (2) */
	public double computeJaccard() {
		
		Custom customShape1_ = null;
		
		double[] xSnakeContour1_ = null;
		double[] ySnakeContour1_ = null;
		double[] xSnakeContour2_ = null;
		double[] ySnakeContour2_ = null;
		
		Snake2DNode[] coef1_ = null;
		Snake2DNode[] coef2_ = null;
		
		int M1_ = 0;
		int M2_ = 0;
		
		int MR1_ = 0;
		
		//Initialization
		try {		
			customShape1_ = IOXMLSD.loadPriorShapeFromXML();			
			
			coef1_ = customShape1_.getDefaultNodes();
			coef2_ = coef_;

		} catch (Exception e1) {
			new AnnounceFrame("FAIL: File not loaded.");
			return 0;
		}

		//NR_ = BSplineBasis.MSPLINESUPPORT * nSamplesPerSegment_;		
		M1_ = customShape1_.getMinNumberNodes();	
		M2_ = M_;

		MR1_ = M1_ * nSamplesPerSegment_;
		PIM_ = Math.PI / M1_;
		PI2M_ = 2 * PIM_;

		xSnakeContour1_ = new double[MR1_];
		ySnakeContour1_ = new double[MR1_];
		xSnakeContour2_ = new double[xSnakeContourSubdivision_.length];
		ySnakeContour2_ = new double[ySnakeContourSubdivision_.length];
		
		//UpdateSnakeSkin1 (gold standard)
		int index;
		double aux, xPosVal1, yPosVal1;
		
		for (int i = 0; i < MR1_; i++) {
			xPosVal1 = 0.0;
			yPosVal1 = 0.0;
			
			for (int k = 0; k < M1_; k++) {
				index = i - k * nSamplesPerSegment_;				
				
				while (index < 0) {
					index += MR1_;
				}
				while (index >= MR1_) {
					index -= MR1_;
				}
				if (index >= NR_) {
					continue;
				} else {
					aux = bSplineLUT_[index];		
				}
				xPosVal1 += coef1_[k].x * aux;
				yPosVal1 += coef1_[k].y * aux;	
			}
			xSnakeContour1_[i] = xPosVal1;
			ySnakeContour1_[i] = yPosVal1;					
		}	
		
		//UpdateSnakeSkin2 
		xSnakeContour2_ = xSnakeContourSubdivision_;
		ySnakeContour2_ = ySnakeContourSubdivision_;
		
		//Getscale1
		int numScales = 2;
		Snake2DScale[] skin1 = new Snake2DScale[numScales];

		double[] xpoints1 = new double[M1_];
		double[] ypoints1 = new double[M1_];

		for (int k = 0; k < M1_; k++) {
			xpoints1[k] = coef1_[k].x;
			ypoints1[k] = coef1_[k].y;
		}
		skin1[0] = new Snake2DScale(xSnakeContour1_, ySnakeContour1_, MR1_,
				SNAKE_CONTOUR_COLOR, true);

		skin1[1] = new Snake2DScale(xpoints1, ypoints1, M1_,
				CONTROL_POLYGON_COLOR, true);
		
		//Getscale2
		Snake2DScale[] skin2 = new Snake2DScale[numScales];

		double[] xpoints2 = new double[M2_];
		double[] ypoints2 = new double[M2_];

		for (int k = 0; k < M2_; k++) {
			xpoints2[k] = coef2_[k].x;
			ypoints2[k] = coef2_[k].y;
		}
		skin2[0] = new Snake2DScale(xSnakeContour2_, ySnakeContour2_, xSnakeContourSubdivision_.length,
				SNAKE_CONTOUR_COLOR, true);

		skin2[1] = new Snake2DScale(xpoints2, ypoints2, M2_,
				CONTROL_POLYGON_COLOR, true);
		
		//Compute Jaccard
		double jaccard = 0.0;
		double and = 0.0;
		double or = 0.0;

		PolygonRoi roi1 = new PolygonRoi((skin1[0].xpoints).clone(),
				(skin1[0].ypoints).clone(), skin1[0].npoints, Roi.TRACED_ROI);
		PolygonRoi roi2 = new PolygonRoi((skin2[0].xpoints).clone(),
				(skin2[0].ypoints).clone(), skin2[0].npoints, Roi.TRACED_ROI);
		if (roi1.getMask() == null || roi2.getMask() == null) {
			return 0.0;
		}

		Sequence seq1 = Icy.getMainInterface().getActiveSequence();//Icy.getMainInterface().getSequences("testFly.tif").get(0); // sequence
																				// (i.e.
																				// image)
																				// must
																				// be
																				// open
																				// and
																				// name
																				// hardcoded
																				// !!

		
		ImagePlus imp_ = ImageJUtil.convertToImageJImage(seq1, null);
		//imp_.show();
		

		ByteProcessor bp1 = new ByteProcessor(imp_.getWidth(), imp_.getHeight());
		bp1.insert(roi1.getMask(), roi1.getBounds().x, roi1.getBounds().y);
		ByteProcessor bp2 = new ByteProcessor(imp_.getWidth(), imp_.getHeight());
		bp2.insert(roi2.getMask(), roi2.getBounds().x, roi2.getBounds().y);

		// debug
		//(new ImagePlus("bp1", bp1)).show("");
		//(new ImagePlus("bp2", bp2)).show("");

		double a = 0.0, b = 0.0;
		for (int x = 0; x < imp_.getWidth(); x++) {
			for (int y = 0; y < imp_.getHeight(); y++) {
				a = bp1.getPixel(x, y);
				b = bp2.getPixel(x, y);				
				
				if (a != 0 || b != 0) {
					or += 1.0;
				}
				if (a != 0 && b != 0) {
					and += 1.0;
				}
			}
		}
		
		jaccard = and / or;
		return jaccard;
	}

	public void saveSnakesTXT(){
		File file=new File("/Users/Anais/snake.txt");
		try {
			FileWriter fw=new FileWriter(file);
			//PrintWriter pw=new PrintWriter(file);
			
			for(int i=0; i<xSnakeContourSubdivision_.length; i++){
				//fw.write(xSnakeContour_[i]);
				fw.write(String.valueOf(xSnakeContourSubdivision_[i]+" "+ySnakeContourSubdivision_[i]));
				fw.write("\r\n");
			}
			fw.close();
		}
		
		catch(IOException exception){
			System.out.println("Error while writting : "+exception.getMessage());
		}
	}

	@Override
	public int getSizeRefinementCoef() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public LinkCurveControlPoints[] getCurveControlledCP() {
		// TODO Auto-generated method stub
		return null;
	}
	
	/*public int getLife(){
		return life_;
	}*/
}
