package plugins.big.bigsnake.snake;

import java.util.ArrayList;
import java.util.List;

import plugins.big.bigsnake.core.SettingsSD;
import plugins.big.bigsnake.snake.Enums.ESnakeEnergyTypeSD;
import plugins.big.bigsnake.snake.Enums.ESnakePriorShapeTypeSD;
import plugins.big.bigsnake.snake.Enums.ESnakeTargetTypeSD;
import plugins.big.bigsnake.snake.Enums.GlobalSnakeEnergyTypeSD;
import plugins.big.bigsnake.snake.Enums.MultiresolutionTypeSD;
import plugins.big.bigsnakeutils.icy.snake2D.Snake2DNode;
import plugins.big.bigsnakeutils.process.process1D.BSplineBasis.BSplineBasisType;

/**
 * Class that wraps the parameters of a GlobalSnake.
 * 
 * @version January 13, 2018
 * 
 * @author Aymeric Galan (aymeric.galan@gmail.com)
 */
public class GlobalSnakeParametersSD extends ESnakeParametersSD {
	
	/**
	 * Subsnake fields
	 */
	private int nSubsnakes_ = 1;
	private int globalLevel_ = SettingsSD.SDLEVEL_DEFAULT;
	
	/**
	 * To be used when the whole list of control nodes is provided.
	 */
	private List<SubsnakeInitialShape> allSubsnakeCoefs_ = null;
	
	private ESnakeParametersSD[] allSubsnakeParameters_ = null;
	
	/**
	 * For removing common edges.
	 */
	private boolean showCommonEdges_ = false;
	
	/** Energy type */
	private GlobalSnakeEnergyTypeSD globalEnergyType_ = null;
	
	/**
	 * Intermediate subdivision
	 */
	private int intermediateSubdiv_ = 0;
	
	// Default value (for now)
	final static int SCALING_FACTOR = 2;
	
	//
	// PUBLIC METHODS
	
	/** Constructor. */
	public GlobalSnakeParametersSD(
			List<SubsnakeInitialShape> allSubsnakeCoefs,
			int subdivLevel,
			ESnakeTargetTypeSD targetType, 
			BSplineBasisType basisFunctionType,
			ESnakeEnergyTypeSD energyType,
			GlobalSnakeEnergyTypeSD globalEnergyType,
			int maxLife, boolean immortal,
			boolean showCommonEdges,
			int intermediateSubdiv) {
		
		// M = 3 (but will be updated during GlobalSNake creation), alpha = 0 by default
		// prior shape to None (no need)
		super(targetType, 3, subdivLevel, basisFunctionType, SCALING_FACTOR, 
				energyType, 0, ESnakePriorShapeTypeSD.NONE, 
				maxLife, immortal, MultiresolutionTypeSD.MANUAL);
		
		nSubsnakes_ = allSubsnakeCoefs.size();
		allSubsnakeCoefs_ = allSubsnakeCoefs;
		globalLevel_ = subdivLevel;
		
		intermediateSubdiv_ = intermediateSubdiv;
		showCommonEdges_ = showCommonEdges;
		
		if ((energyType == ESnakeEnergyTypeSD.REGION) ||
				(energyType == ESnakeEnergyTypeSD.MIXTURE)) {
			System.err.println("Region or mixture energies not supported "
					+ "--> Ridge by default.");
			energyType_ = ESnakeEnergyTypeSD.RIDGE;
		}
		globalEnergyType_ = globalEnergyType;
		
		buildSubsnakeCurveParamaters();
	}

	
	public int getNSubsnakes() {
		return nSubsnakes_;
	}
	
	public int getGlobalLevel() {
		return globalLevel_;
	}
	
	public void setGlobalLevel(int globalLevel) {
		globalLevel_ = globalLevel;
	}
	
	public List<SubsnakeInitialShape> getAllSubsnakeCoefs() {
		return allSubsnakeCoefs_;
	}
	
	public SubsnakeInitialShape getSubsnakeCoefs(int index) {
		if (allSubsnakeCoefs_ == null) return null;
		return allSubsnakeCoefs_.get(index);
	}
	
	public ESnakeParametersSD[] getAllSubsnakeParameters() {
		return allSubsnakeParameters_;
	}
	
	public boolean showCommonEdges() {
		return showCommonEdges_;
	}
	
	public GlobalSnakeEnergyTypeSD getGlobalEnergyType() {
		return globalEnergyType_;
	}
	
	public void setGlobalEnergyType(GlobalSnakeEnergyTypeSD globalEnergyType) {
		globalEnergyType_ = globalEnergyType;
	}
	
	public int getIntermediateSubdiv() {
		return intermediateSubdiv_;
	}
	
	public void setIntermediateSubdiv(int intermediateSubdiv) {
		intermediateSubdiv_ = intermediateSubdiv;
	}
	
	// -------------------
	// PRIVATE METHODS
	
	private void buildSubsnakeCurveParamaters() {
		allSubsnakeParameters_ = new ESnakeParametersSD[nSubsnakes_];
		for (int i = 0 ; i < nSubsnakes_ ; i++) {
			int M = allSubsnakeCoefs_.get(i).size();
			int level = Level_; /* same level for all Subsnakes */
			
			allSubsnakeParameters_[i] = new ESnakeParametersSD(
					ESnakeTargetTypeSD.BRIGHT,
					M, // number of control points
					level, // subdivision level
					basisFunctionType_,
					mScaling_, // scaling factor
					energyType_,
					0, // alpha
					ESnakePriorShapeTypeSD.CIRCLE, // might be overridden if subsnake nodes are provided
					maxLife_, // max number of iterations
					immortal_, // is immortal
					MultiresolutionTypeSD.MANUAL);
		}
	}
	
}
