package plugins.big.bigsnake.tests;

import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import plugins.big.bigsnake.tests.output.CSVUtils;

public class TestOutputWriter {
	
	private static final char CSV_SEPARATOR = ',';
	
	private FileWriter writer_ = null;
	private boolean headerWritten_ = false;
	private String outputPath_ = "/Users";
	private boolean forceFlush_ = true;
	
	public TestOutputWriter(String outputPath, boolean forceFlush) {
		outputPath_ = outputPath;
		forceFlush_ = forceFlush;
		try {
			writer_ = new FileWriter(outputPath_);
		} catch (IOException e) {
			e.printStackTrace();
		}
		headerWritten_ = false;
	}
	
	public String getOutputPath() {
		return outputPath_;
	}
	
	public void outputJaccardStatistics(String testFilename, double average, double std, 
			Map<String, Double> allIndices, double runtimeSeconds) throws IOException {
		
		List<String> header = null;
		if (!headerWritten_) {
			header = new ArrayList<String>();
			header.add("Filename");
			header.add("Average");
			header.add("Std dev");
		}
		
		List<String> row = new ArrayList<String>();
		row.add(testFilename);
		row.add(String.valueOf(average));
		row.add(String.valueOf(std));
		
		for (Map.Entry<String, Double> entry : allIndices.entrySet()) {
			if (!headerWritten_) {
				final String subsnakeId = entry.getKey();
				header.add("Subsnake " + subsnakeId);
			}
			final Double value = entry.getValue();
			row.add(String.valueOf(value));
		}
		
		if (!headerWritten_ ) {
			header.add("Runtime [s]");
			System.out.println("CSV header : " + header);
			CSVUtils.writeLine(writer_, header, CSV_SEPARATOR);
			headerWritten_ = true;
		}
		row.add(String.valueOf(runtimeSeconds));
		System.out.println("CSV row : " + row);
		CSVUtils.writeLine(writer_, row, CSV_SEPARATOR);
	
		if (forceFlush_) {
			try {
				writer_.flush();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public void finish() {
		/* Close the writer stream */
		try {
			if (!forceFlush_) writer_.flush();
	        writer_.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
