package plugins.big.bigsnake.tests.watersheds;

import ij.ImagePlus;
import ij.io.Opener;


public class JaccardMasks{
	
	//Groundtruth (mask)
	private double[][] mask_ = null;

	//Binary Segmentation
	private double[][] binSeg_ = null;


	public JaccardMasks(double[][] binSeg, double[][] mask) {
		binSeg_ = binSeg;
		mask_ = mask;
	}

	public double run() {
		double jacc = computeJaccard(binSeg_, mask_);
		System.out.println("jaccard index = " + jacc);
		return jacc;

	}

	public static double computeJaccard(double[][] binSeg, double[][] mask) {
		double jaccard = 0.0;
		double and = 0.0;
		double or = 0.0;
		
        String path="/Users/anais/Documents/workspaceAymeric/ssnakebig/test_images/ideal_emb1_z36_t22/components/membrane.png";
        double[][] boundaries = loadImage(path,mask.length, mask[0].length);

		double a = 0.0, b = 0.0;
		for (int x = 0; x < mask.length; x++) {
			for (int y = 0; y < mask[0].length; y++) {
				a = binSeg[x][y];
				b = mask[x][y];			
                
                if(boundaries[x][y]==0){//Pour le Jaccard on ne veut garder que l'intérieur de la cellule. Pas la membrane
                    if (a != 0 || b != 0) {
                        or += 1.0;
                    }
                    if (a != 0 && b != 0) {
                        and += 1.0;
                    }
                }
            }
        }
        
        jaccard = and / or;
        return jaccard;
	}
	
	 public static double[][] loadImage(String filename, int width, int height){
	        
	        Opener opener = new Opener();
	        
	        double[][] imageMask = new double[width][height];
	        
	        //String filename=folder +"/"+ file.getName();
	        
	        ImagePlus image = opener.openImage(filename);
	        imageMask= ImagePlus2Array2D(image);
	        
	        return imageMask;
	    }
	    
	    /**
	     * Convert an ImagePlus to a 2D array
	     * @param imp image as ImagePlus
	     * @return 2D array
	     */
	    public static double[][] ImagePlus2Array2D(ImagePlus imp) {
	        
	        int nx = imp.getWidth();
	        int ny = imp.getHeight();
	        
	        double[][] output = new double[nx][ny];
	        
	        for(int y = 0; y < ny; y++) {
	            for(int x = 0; x < nx; x++) {
	                output[x][y] = imp.getProcessor().getPixelValue(x, y);
	                
	            }
	        }
	        
	        return output;
	    }
}
