package plugins.big.bigsnake.utils;

import java.io.File;
import java.io.FilenameFilter;

public class FileUtils {
	
	private static final String[] DEFAULT_EXTENSIONS = {"png", "jpg"};
	
	public static String[] getAllFilenamesInDirectory(String directory, String[] extensions) {
		
		File[] foundFiles;
		foundFiles = new File(directory).listFiles(
				getFilenameFilter(extensions));
		if (foundFiles == null) {
			return null;
		}
		
		String[] filenames = new String[foundFiles.length];
		for (int i = 0 ; i < foundFiles.length ; i ++) {
			filenames[i] = foundFiles[i].getName();
		}
		return filenames;
	}
	
	
	private static FilenameFilter getFilenameFilter(String[] allowedExtentions) {
		FilenameFilter filter = new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				final String[] extensions = (allowedExtentions == null) ? 
						DEFAULT_EXTENSIONS : allowedExtentions;
				for (String ext : extensions) {
					if (name.endsWith(ext)) return true;
				}
				return false;
			}
		};
		return filter;
	}
	
	
	/**
	 * Return a String array containing 
	 * the file name without the extension as first element
	 * and the extension as second element.
	 * 
	 * Inspired by :
	 * https://stackoverflow.com/questions/3449218/remove-filename-extension-in-java
	 */
	public static String[] removeAndGetExtension(String filePath) {
	    File f = new File(filePath);

	    // if it's a directory, don't remove the extension
	    if (f.isDirectory()) {
	    		String[] newNameAndExt = {filePath, ""};
	    		return newNameAndExt;
	    }

	    String name = f.getName();

	    // Now we know it's a file - don't need to do any special hidden
	    // checking or contains() checking because of:
	    final int lastPeriodPos = name.lastIndexOf('.');
	    if (lastPeriodPos <= 0) {
	        // No period after first character - return name as it was passed in
	    		String[] newNameAndExt = {filePath, ""};
	    		return newNameAndExt;
	    }
	    else
	    {
	        // Remove the last period and everything after it
	        File renamed = new File(f.getParent(), name.substring(0, lastPeriodPos));
	        String extension = name.substring(lastPeriodPos, name.length());
	        String[] newNameAndExt = {renamed.getPath(), extension};
	        return newNameAndExt;
	    }
	}
}
