package plugins.big.bigsnake.utils;

import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.io.File;
import java.io.IOException;
import java.nio.file.Paths;

import javax.imageio.ImageIO;

import icy.common.exception.UnsupportedFormatException;
import icy.file.Loader;
import icy.file.SequenceFileImporter;
import icy.image.IcyBufferedImage;
import icy.image.IcyBufferedImageUtil;
import icy.sequence.Sequence;
import icy.type.DataType;
import ij.IJ;
import ij.process.ImageProcessor;

public class ImageUtils {
	
	public static double[] extractImageFromSequence(Sequence sequence) {
		IcyBufferedImage inputImage = sequence.getFirstImage();
		inputImage = IcyBufferedImageUtil.convertToType(
				inputImage, DataType.DOUBLE, false);
		return inputImage.getDataXYAsDouble(0);
	}
	
	
	@Deprecated
	public static Sequence getSequenceFromPath(String path) {
		BufferedImage nucleiBuffImg = null;
		try {
			nucleiBuffImg = ImageIO.read(new File(path));
		} catch (IOException e) {
			System.err.println("Cannnot read the nuclei image file.");
			e.printStackTrace();
			return null;
		}
		final String name = Paths.get(path).getFileName().toString();
		return new Sequence(name, nucleiBuffImg);
	}
	
	
	public static Sequence getSequenceFromPathWithIJ(String path) {
		BufferedImage nucleiBuffImg = IJ.openImage(path).getBufferedImage();
		final String name = Paths.get(path).getFileName().toString();
		return new Sequence(name, nucleiBuffImg);
	}
	
	
	public static Sequence getSequenceLikeInIcy(String path) throws IOException {
		SequenceFileImporter importer = Loader.getSequenceFileImporter(path, false);
		Sequence seq = null;

		try {
			if (importer.open(path, 0)) {
				final IcyBufferedImage img = 
						IcyBufferedImageUtil.convertToType(importer.getImage(0, 0), DataType.DOUBLE, false);
				importer.close();
				
				final IcyBufferedImage image = 
						IcyBufferedImageUtil.convertToType(img, DataType.DOUBLE, false);
				
				final String name = Paths.get(path).getFileName().toString();
				seq = new Sequence(name, image);
			}
		} catch (UnsupportedFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return seq;
	}
	
	
	public static double[][] convert1DTo2DImage(Object image1DRepr, 
			int width, int height, boolean enforceBinary) {
		
		double[][] image2DRepr = new double[width][height];
		
		if (image1DRepr instanceof float[]) {
			final float[] image1DReprC = (float[]) image1DRepr;
			for (int x = 0 ; x < width ; x++) {
				for (int y = 0 ; y < height ; y++) {
					final float pixel = image1DReprC[x + y*width];
					double newPixel = (double) pixel;
					if (enforceBinary && (pixel != 0) && (pixel != 1)) {
						System.out.println("Info : the mask image provided is being clipped to {0, 1}");
						if (pixel < 0)
							newPixel = 0;
						else if (pixel > 1)
							newPixel = 1;
					}
					image2DRepr[x][y] = newPixel;
				}
			}
		} else if (image1DRepr instanceof double[]) {
			final double[] image1DReprC = (double[]) image1DRepr;
			for (int x = 0 ; x < width ; x++) {
				for (int y = 0 ; y < height ; y++) {
					final double pixel = image1DReprC[x + y*width];
					double newPixel = pixel;
					if (enforceBinary && (pixel != 0) && (pixel != 1)) {
						System.out.println("Info : the mask image provided is being clipped to {0, 1}");
						if (pixel < 0)
							newPixel = 0;
						else if (pixel > 1)
							newPixel = 1;
					}
					image2DRepr[x][y] = newPixel;
				}
			}
		} else if (image1DRepr instanceof byte[]) {
			final byte[] image1DReprC = (byte[]) image1DRepr;
			for (int x = 0 ; x < width ; x++) {
				for (int y = 0 ; y < height ; y++) {
					final byte pixel = image1DReprC[x + y*width];
					byte newPixel = (byte) pixel;
					if (enforceBinary && (pixel != 0) && (pixel != 1)) {
						System.out.println("Info : the mask image provided is being clipped to {0, 1}");
						if (pixel < 0)
							newPixel = 0;
						else if (pixel > 1)
							newPixel = 1;
					}
					image2DRepr[x][y] = newPixel;
				}
			}
		} else {
			System.err.println("1D representation has unspported type");
			return null;
		}
		return image2DRepr;
	}
	
	
	public static double[][] convertBufferedImageTo2Ddouble(BufferedImage buffImg, 
			boolean enforceBinary) {
		
		final int width = buffImg.getWidth();
		final int height = buffImg.getHeight();
		final DataBufferByte bufferByte = (DataBufferByte) buffImg.getRaster().getDataBuffer();
		final byte[] image1DRepr = bufferByte.getData();
		
		double[][] image2DRepr = new double[width][height];
		
		for (int x = 0 ; x < width ; x++) {
			for (int y = 0 ; y < height ; y++) {
				byte pixel = image1DRepr[x + y*width];

				if (enforceBinary && (pixel != 0) && (pixel != 1)) {
					System.out.println("Warning : "
							+ "the mask image provided is not binary "
							+ "--> clip pixels to {0, 1}.");
					if (pixel < 0)
						pixel = 0;
					else if (pixel > 1)
						pixel = 1;
				}
				
				image2DRepr[x][y] = (double) Math.round(pixel);
			}
		}
		return image2DRepr;
	}
	
	
	/** Taken from ImageAccess constructor (credits : Daniel Sage, BIG) */
	public static double[] extractPixelsFromProcessor(ImageProcessor ip) {
		final int nx = ip.getWidth();
		final int ny = ip.getHeight();
		final int size = nx*ny;
		double[] pixels = new double[size];
		if (ip.getPixels() instanceof byte[]) {
			byte[] bsrc = (byte[])ip.getPixels();
			for (int k=0; k<size; k++)
				pixels[k] = (double)(bsrc[k] & 0xFF);
		
		}	
		else if (ip.getPixels() instanceof short[]) {
			 short[] ssrc = (short[])ip.getPixels();
			 for (int k=0; k<size; k++)
				pixels[k] = (double)(ssrc[k] & 0xFFFF);
		}	
		else if (ip.getPixels() instanceof float[]) {
			 float[] fsrc = (float[])ip.getPixels();
			 for (int k=0; k<size; k++)
				pixels[k] = (double)fsrc[k];
		}
		else  {
			throw new ArrayStoreException("Unexpected image type.");
		}
		return pixels;
	}
	
	
	/** Taken from ImageAccess constructor (credits : Daniel Sage, BIG) */
	public static double[][] extractPixelsFromProcessor(ImageProcessor ip, 
			int width, int height, boolean enforceBinary) {
		final double[] pixels1D = extractPixelsFromProcessor(ip);
		return convert1DTo2DImage(pixels1D, width, height, enforceBinary);
	}
	
	
	public static double[][] getPixelsFromPath(String imageFilePath, boolean enforceBinary) {
		BufferedImage buffImg = null;
		try {
			buffImg = ImageIO.read(new File(imageFilePath));
		}
		catch (IOException e) {
			System.err.println("Cannnot read the mask image file.");
			e.printStackTrace();
			return null;
		}
		
//		display(buffImg); // debug
		
		return ImageUtils.convertBufferedImageTo2Ddouble(buffImg, enforceBinary);
	}
	
}
